@@ -1288,7 +1288,7 @@ void TDirectory::SetName(const char* newname)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Encode the name and cycle into buffer like: "aap;2".
-/// @note if cycle is `9999`, the name is copied to the buffer without any `;`
+/// @note if `cycle` is 9999, its value will not appear in the output and `name` will be used verbatim.
 
 void TDirectory::EncodeNameCycle(char *buffer, const char *name, Short_t cycle)
 {
@@ -1317,15 +1317,13 @@ void TDirectory::DecodeNameCycle(const char *buffer, char *name, Short_t &cycle,
                                  const size_t namesize)
 {
    if (!buffer) {
-      ::Warning("TDirectory::DecodeNameCycle",
-         "An empty buffer was passed, cannot decode.");
       cycle = 9999;
       return;
    }
 
    size_t len = 0;
    const char *ni = strchr(buffer, ';');
-   
+
    if (ni) {
       // Found ';'
       len = ni - buffer;
@@ -1412,7 +1410,7 @@ void TDirectory::RegisterGDirectory(TDirectory::SharedGDirectory_t &gdirectory_p
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// \copydoc TDirectory::WriteTObject().
+/// \copydoc TDirectory::WriteObject(const T*,const char*,Option_t*,Int_t).
 
 Int_t TDirectory::WriteTObject(const TObject *obj, const char *name, Option_t * /*option*/, Int_t /*bufsize*/)
 {
