@@ -32,6 +32,7 @@ TQConnection:
 #include <iostream>
 #include "TVirtualMutex.h"
 #include "THashTable.h"
+#include "strlcpy.h"
 
 ClassImpQ(TQConnection)
 
@@ -47,7 +48,7 @@ protected:
    CallFunc_t    *fFunc;      // CINT method invocation environment
    ClassInfo_t   *fClass;     // CINT class for fFunc
    TFunction     *fMethod;    // slot method or global function
-   Long_t         fOffset;    // offset added to object pointer
+   Longptr_t      fOffset;    // offset added to object pointer
    TString        fName;      // full name of method
    Int_t          fExecuting; // true if one of this slot's ExecuteMethod methods is being called
 public:
@@ -56,14 +57,12 @@ public:
    virtual ~TQSlot();
 
    Bool_t      CheckSlot(Int_t nargs) const;
-   Long_t      GetOffset() const { return fOffset; }
+   Longptr_t   GetOffset() const { return fOffset; }
    CallFunc_t *StartExecuting();
    CallFunc_t *GetFunc() const { return fFunc; }
    void        EndExecuting();
 
-   const char *GetName() const {
-      return fName.Data();
-   }
+   const char *GetName() const override { return fName.Data(); }
 
    Int_t GetMethodNargs() { return fMethod->GetNargs(); }
 
@@ -73,15 +72,11 @@ public:
    void ExecuteMethod(void *object, Long64_t param);
    void ExecuteMethod(void *object, Double_t param);
    void ExecuteMethod(void *object, const char *params);
-   void ExecuteMethod(void *object, Long_t *paramArr, Int_t nparam = -1);
-   void Print(Option_t *opt = "") const;
-   void ls(Option_t *opt = "") const {
-      Print(opt);
-   }
+   void ExecuteMethod(void *object, Longptr_t *paramArr, Int_t nparam = -1);
+   void Print(Option_t *opt = "") const override;
+   void ls(Option_t *opt = "") const override { Print(opt); }
 
-   Bool_t IsExecuting() const {
-      return fExecuting > 0;
-   }
+   Bool_t IsExecuting() const { return fExecuting > 0; }
 };
 
 
@@ -100,10 +95,10 @@ public:
 TQSlot::TQSlot(TClass *cl, const char *method_name,
                const char *funcname) : TObject(), TRefCnt()
 {
-   fFunc      = 0;
-   fClass     = 0;
+   fFunc      = nullptr;
+   fClass     = nullptr;
    fOffset    = 0;
-   fMethod    = 0;
+   fMethod    = nullptr;
    fName      = "";
    fExecuting = 0;
 
@@ -111,12 +106,14 @@ TQSlot::TQSlot(TClass *cl, const char *method_name,
 
    fName = method_name;
 
-   char *method = new char[strlen(method_name) + 1];
-   if (method) strcpy(method, method_name);
+   auto len = strlen(method_name) + 1;
+   char *method = new char[len];
+   if (method)
+      strlcpy(method, method_name, len);
 
    char *proto;
    char *tmp;
-   char *params = 0;
+   char *params = nullptr;
 
    // separate method and prototype strings
 
@@ -175,19 +172,21 @@ TQSlot::TQSlot(TClass *cl, const char *method_name,
 TQSlot::TQSlot(const char *class_name, const char *funcname) :
    TObject(), TRefCnt()
 {
-   fFunc      = 0;
-   fClass     = 0;
+   fFunc      = nullptr;
+   fClass     = nullptr;
    fOffset    = 0;
-   fMethod    = 0;
+   fMethod    = nullptr;
    fName      = funcname;
    fExecuting = 0;
 
-   char *method = new char[strlen(funcname) + 1];
-   if (method) strcpy(method, funcname);
+   auto len = strlen(funcname) + 1;
+   char *method = new char[len];
+   if (method)
+      strlcpy(method, funcname, len);
 
    char *proto;
    char *tmp;
-   char *params = 0;
+   char *params = nullptr;
 
    // separate method and prototype strings
 
@@ -202,11 +201,9 @@ TQSlot::TQSlot(const char *class_name, const char *funcname) :
    gCling->CallFunc_IgnoreExtraArgs(fFunc, true);
 
    fClass = gCling->ClassInfo_Factory();
-   TClass *cl = 0;
+   TClass *cl = nullptr;
 
-   if (!class_name)
-      ;                       // function
-   else {
+   if (class_name) {
       gCling->ClassInfo_Init(fClass, class_name);  // class
       cl = TClass::GetClass(class_name);
    }
@@ -246,7 +243,7 @@ TQSlot::~TQSlot()
 
 inline void TQSlot::ExecuteMethod(void *object)
 {
-   ExecuteMethod(object, (Long_t*)nullptr, 0);
+   ExecuteMethod(object, (Longptr_t*)nullptr, 0);
 
 }
 
@@ -287,7 +284,7 @@ CallFunc_t *TQSlot::StartExecuting() {
 
 void TQSlot::EndExecuting() {
    fExecuting--;
-   if (!TestBit(kNotDeleted) && !fExecuting)
+   if (ROOT::Detail::HasBeenDeleted(this) && !fExecuting)
       gCling->CallFunc_Delete(fFunc);
 }
 
@@ -297,8 +294,7 @@ void TQSlot::EndExecuting() {
 
 inline void TQSlot::ExecuteMethod(void *object, Long_t param)
 {
-   ExecuteMethod(object, &param, 1);
-
+   ExecuteMethod(object, (Longptr_t *)&param, 1);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -307,9 +303,8 @@ inline void TQSlot::ExecuteMethod(void *object, Long_t param)
 
 inline void TQSlot::ExecuteMethod(void *object, Long64_t param)
 {
-   Long_t *arg = reinterpret_cast<Long_t *>(&param);
+   Longptr_t *arg = reinterpret_cast<Longptr_t *>(&param);
    ExecuteMethod(object, arg, 1);
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -318,9 +313,8 @@ inline void TQSlot::ExecuteMethod(void *object, Long64_t param)
 
 inline void TQSlot::ExecuteMethod(void *object, Double_t param)
 {
-   Long_t *arg = reinterpret_cast<Long_t *>(&param);
+   Longptr_t *arg = reinterpret_cast<Longptr_t *>(&param);
    ExecuteMethod(object, arg, 1);
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -328,9 +322,8 @@ inline void TQSlot::ExecuteMethod(void *object, Double_t param)
 
 inline void TQSlot::ExecuteMethod(void *object, const char *param)
 {
-   Long_t arg = reinterpret_cast<Long_t>(param);
+   Longptr_t arg = reinterpret_cast<Longptr_t>(param);
    ExecuteMethod(object, &arg, 1);
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -342,16 +335,16 @@ inline void TQSlot::ExecuteMethod(void *object, const char *param)
 /// Nargs is the number of all arguments and NargsOpt is the number
 /// of default arguments.
 
-inline void TQSlot::ExecuteMethod(void *object, Long_t *paramArr, Int_t nparam)
+inline void TQSlot::ExecuteMethod(void *object, Longptr_t *paramArr, Int_t nparam)
 {
-   void *address = 0;
+   void *address = nullptr;
    R__LOCKGUARD(gInterpreterMutex);
    if (paramArr) gCling->CallFunc_SetArgArray(fFunc, paramArr, nparam);
-   if (object) address = (void *)((Long_t)object + fOffset);
+   if (object) address = (void *)((Longptr_t)object + fOffset);
    fExecuting++;
    gCling->CallFunc_Exec(fFunc, address);
    fExecuting--;
-   if (!TestBit(kNotDeleted) && !fExecuting)
+   if (ROOT::Detail::HasBeenDeleted(this) && !fExecuting)
       gCling->CallFunc_Delete(fFunc);
 }
 
@@ -442,13 +435,13 @@ void TQSlotPool::Free(TQSlot *slot)
 
 static TQSlotPool gSlotPool;  // global pool of slots
 
-void TQConnection::SetArg(const Long_t *params, Int_t nparam/* = -1*/) {
+void TQConnection::SetArg(const Longptr_t *params, Int_t nparam/* = -1*/) {
    if (nparam == -1)
       nparam = fSlot->GetMethodNargs();
 
    // FIXME: Why TInterpreter needs non-const SetArgArray. TClingCallFunc
    // doesn't modify the value.
-   gInterpreter->CallFunc_SetArgArray(fSlot->GetFunc(), const_cast<Long_t*>(params), nparam);
+   gInterpreter->CallFunc_SetArgArray(fSlot->GetFunc(), const_cast<Longptr_t*>(params), nparam);
 }
 
 
@@ -461,7 +454,7 @@ void TQConnection::SetArg(const Long_t *params, Int_t nparam/* = -1*/) {
 TQConnection::TQConnection(TClass *cl, void *receiver, const char *method_name)
    : TQObject()
 {
-   const char *funcname = 0;
+   const char *funcname = nullptr;
    fReceiver = receiver;      // fReceiver is pointer to receiver
 
    if (!cl) {
@@ -618,7 +611,7 @@ void TQConnection::ExecuteMethod(Double_t param)
 /// Apply slot-method to the fReceiver object with variable
 /// number of argument values.
 
-void TQConnection::ExecuteMethod(Long_t *params, Int_t nparam)
+void TQConnection::ExecuteMethod(Longptr_t *params, Int_t nparam)
 {
    // This connection might be deleted in result of the method execution
    // (for example in case of a Disconnect).  Hence we do not assume
@@ -654,7 +647,7 @@ Bool_t TQConnection::CheckSlot(Int_t nargs) const {
 /// Return the object address to be passed to the function.
 
 void *TQConnection::GetSlotAddress() const {
-   if (fReceiver) return (void *)((Long_t)fReceiver + fSlot->GetOffset());
+   if (fReceiver) return (void *)((Longptr_t)fReceiver + fSlot->GetOffset());
    else return nullptr;
 }
 
