@@ -5,7 +5,18 @@
 
 #include <memory>
 #include <set>
+#include <unordered_map>
 
+namespace std
+{
+template <> struct hash<ROOT::Internal::RConcurrentHashColl::HashValue>
+{
+   std::size_t operator()(const ROOT::Internal::RConcurrentHashColl::HashValue& key) const noexcept
+   {
+      return key.Hash();
+   }
+};
+}
 namespace ROOT {
 namespace Internal {
 
@@ -23,20 +34,24 @@ RConcurrentHashColl::HashValue::HashValue(const char *data, int len)
    Sha256(reinterpret_cast<const unsigned char *>(data), len, fDigest);
 }
 
-struct RHashSet {
-   std::set<ROOT::Internal::RConcurrentHashColl::HashValue> fSet;
+struct RHashMap {
+   std::unordered_map<ROOT::Internal::RConcurrentHashColl::HashValue, RUidColl> fHashMap;
 };
 
 RConcurrentHashColl::RConcurrentHashColl()
-   : fHashSet(std::make_unique<RHashSet>()), fRWLock(std::make_unique<ROOT::TRWSpinLock>()){};
+   : fHashMap(std::make_unique<RHashMap>()), fRWLock(std::make_unique<ROOT::TRWSpinLock>()){};
 
 RConcurrentHashColl::~RConcurrentHashColl() = default;
 
 /// Return true if the hash is already in already there
-bool RConcurrentHashColl::Find(const HashValue &hash) const
+const RUidColl* RConcurrentHashColl::Find(const HashValue &hash) const
 {
    ROOT::TRWSpinLockReadGuard rg(*fRWLock);
-   return (fHashSet->fSet.end() != fHashSet->fSet.find(hash));
+   auto iter = fHashMap->fHashMap.find(hash);
+   if (iter != fHashMap->fHashMap.end())
+      return &(iter->second);
+   else
+      return nullptr;
 }
 
 /// If the buffer is there, return false. Otherwise, insert the hash and return true
@@ -46,27 +61,10 @@ RConcurrentHashColl::HashValue RConcurrentHashColl::Hash(char *buffer, int len)
 }
 
 /// If the buffer is there, return false. Otherwise, insert the hash and return true
-bool RConcurrentHashColl::Insert(char *buffer, int len) const
-{
-   HashValue hash(buffer, len);
-
-   {
-      ROOT::TRWSpinLockReadGuard rg(*fRWLock);
-      if (fHashSet->fSet.end() != fHashSet->fSet.find(hash))
-         return false;
-   }
-   {
-      ROOT::TRWSpinLockWriteGuard wg(*fRWLock);
-      fHashSet->fSet.insert(hash);
-      return true;
-   }
-}
-
-/// If the buffer is there, return false. Otherwise, insert the hash and return true
-bool RConcurrentHashColl::Insert(const HashValue &hash) const
+bool RConcurrentHashColl::Insert(const HashValue &hash, RUidColl &&values) const
 {
    ROOT::TRWSpinLockWriteGuard wg(*fRWLock);
-   auto ret = fHashSet->fSet.insert(hash);
+   auto ret = fHashMap->fHashMap.insert({hash, std::move(values)});
    return ret.second;
 }
 
