@@ -9,16 +9,16 @@
  * For the list of contributors see $ROOTSYS/README/CREDITS.             *
  *************************************************************************/
 
-//////////////////////////////////////////////////////////////////////////
-//                                                                      //
-// TThread                                                              //
-//                                                                      //
-// This class implements threads. A thread is an execution environment  //
-// much lighter than a process. A single process can have multiple      //
-// threads. The actual work is done via the TThreadImp class (either    //
-// TPosixThread or TWin32Thread).                                       //
-//                                                                      //
-//////////////////////////////////////////////////////////////////////////
+/** \class TThread
+
+\legacy{TThread}
+
+This class implements threads. A thread is an execution environment
+much lighter than a process. A single process can have multiple
+threads. The actual work is done via the TThreadImp class (either
+TPosixThread or TWin32Thread).
+
+**/
 
 #include "RConfigure.h"
 
@@ -178,7 +178,7 @@ Int_t TJoinHelper::Join()
          // If we received the signal or timed out, let's check the value
          if (fJoined) break;
       } else {
-         // If any other error occured, there is no point in trying again
+         // If any other error occurred, there is no point in trying again
          break;
       }
 
@@ -562,7 +562,7 @@ Long_t TThread::SelfId()
 ////////////////////////////////////////////////////////////////////////////////
 /// Start the thread. This starts the static method TThread::Function()
 /// which calls the user function specified in the TThread ctor with
-/// the arg argument. 
+/// the arg argument.
 /// If affinity is specified (>=0), a CPU affinity will be associated
 /// with the current thread.
 /// Returns 0 on success, otherwise an error number will
@@ -1085,7 +1085,7 @@ void TThread::XAction()
    enum { kPRTF = 0, kCUPD = 5, kCANV = 10, kCDEL = 15,
           kPDCD = 20, kMETH = 25, kERRO = 30 };
    int iact = strstr(acts, fgXAct) - acts;
-   char *cmd = 0;
+   TString cmd;
 
    switch (iact) {
 
@@ -1123,8 +1123,8 @@ void TThread::XAction()
 
             case 2:
                //((TCanvas*)fgXArr[1])->Constructor();
-               cmd = Form("((TCanvas *)0x%zx)->Constructor();",(size_t)fgXArr[1]);
-               gROOT->ProcessLine(cmd);
+               cmd.Form("((TCanvas *)0x%zx)->Constructor();",(size_t)fgXArr[1]);
+               gROOT->ProcessLine(cmd.Data());
                break;
 
             case 5:
@@ -1132,8 +1132,8 @@ void TThread::XAction()
                //                 (char*)fgXArr[2],
                //                 (char*)fgXArr[3],
                //                *((Int_t*)(fgXArr[4])));
-               cmd = Form("((TCanvas *)0x%zx)->Constructor((char*)0x%zx,(char*)0x%zx,*((Int_t*)(0x%zx)));",(size_t)fgXArr[1],(size_t)fgXArr[2],(size_t)fgXArr[3],(size_t)fgXArr[4]);
-               gROOT->ProcessLine(cmd);
+               cmd.Form("((TCanvas *)0x%zx)->Constructor((char*)0x%zx,(char*)0x%zx,*((Int_t*)(0x%zx)));",(size_t)fgXArr[1],(size_t)fgXArr[2],(size_t)fgXArr[3],(size_t)fgXArr[4]);
+               gROOT->ProcessLine(cmd.Data());
                break;
             case 6:
                //((TCanvas*)fgXArr[1])->Constructor(
@@ -1141,8 +1141,8 @@ void TThread::XAction()
                //                 (char*)fgXArr[3],
                //                *((Int_t*)(fgXArr[4])),
                //                *((Int_t*)(fgXArr[5])));
-               cmd = Form("((TCanvas *)0x%zx)->Constructor((char*)0x%zx,(char*)0x%zx,*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)));",(size_t)fgXArr[1],(size_t)fgXArr[2],(size_t)fgXArr[3],(size_t)fgXArr[4],(size_t)fgXArr[5]);
-               gROOT->ProcessLine(cmd);
+               cmd.Form("((TCanvas *)0x%zx)->Constructor((char*)0x%zx,(char*)0x%zx,*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)));",(size_t)fgXArr[1],(size_t)fgXArr[2],(size_t)fgXArr[3],(size_t)fgXArr[4],(size_t)fgXArr[5]);
+               gROOT->ProcessLine(cmd.Data());
                break;
 
             case 8:
@@ -1153,8 +1153,8 @@ void TThread::XAction()
                //               *((Int_t*)(fgXArr[5])),
                //               *((Int_t*)(fgXArr[6])),
                //               *((Int_t*)(fgXArr[7])));
-               cmd = Form("((TCanvas *)0x%zx)->Constructor((char*)0x%zx,(char*)0x%zx,*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)));",(size_t)fgXArr[1],(size_t)fgXArr[2],(size_t)fgXArr[3],(size_t)fgXArr[4],(size_t)fgXArr[5],(size_t)fgXArr[6],(size_t)fgXArr[7]);
-               gROOT->ProcessLine(cmd);
+               cmd.Form("((TCanvas *)0x%zx)->Constructor((char*)0x%zx,(char*)0x%zx,*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)),*((Int_t*)(0x%zx)));",(size_t)fgXArr[1],(size_t)fgXArr[2],(size_t)fgXArr[3],(size_t)fgXArr[4],(size_t)fgXArr[5],(size_t)fgXArr[6],(size_t)fgXArr[7]);
+               gROOT->ProcessLine(cmd.Data());
                break;
 
          }
@@ -1162,8 +1162,8 @@ void TThread::XAction()
 
       case kCDEL:
          //((TCanvas*)fgXArr[1])->Destructor();
-         cmd = Form("((TCanvas *)0x%zx)->Destructor();",(size_t)fgXArr[1]);
-         gROOT->ProcessLine(cmd);
+         cmd.Form("((TCanvas *)0x%zx)->Destructor();",(size_t)fgXArr[1]);
+         gROOT->ProcessLine(cmd.Data());
          break;
 
       case kPDCD:
