@@ -15,24 +15,24 @@ Bridge class for using a VecGeom solid as TGeoShape.
 
 #include "TGeoVGShape.h"
 
-#include "volumes/PlacedVolume.h"
-#include "volumes/UnplacedVolume.h"
-#include "volumes/UnplacedBox.h"
-#include "volumes/UnplacedTube.h"
-#include "volumes/UnplacedCone.h"
-#include "volumes/UnplacedParaboloid.h"
-#include "volumes/UnplacedParallelepiped.h"
-#include "volumes/UnplacedPolyhedron.h"
-#include "volumes/UnplacedTrd.h"
-#include "volumes/UnplacedOrb.h"
-#include "volumes/UnplacedSphere.h"
-#include "volumes/UnplacedBooleanVolume.h"
-#include "volumes/UnplacedTorus2.h"
-#include "volumes/UnplacedTrapezoid.h"
-#include "volumes/UnplacedPolycone.h"
-#include "volumes/UnplacedScaledShape.h"
-#include "volumes/UnplacedGenTrap.h"
-#include "volumes/UnplacedSExtruVolume.h"
+#include "VecGeom/volumes/PlacedVolume.h"
+#include "VecGeom/volumes/UnplacedVolume.h"
+#include "VecGeom/volumes/UnplacedBox.h"
+#include "VecGeom/volumes/UnplacedTube.h"
+#include "VecGeom/volumes/UnplacedCone.h"
+#include "VecGeom/volumes/UnplacedParaboloid.h"
+#include "VecGeom/volumes/UnplacedParallelepiped.h"
+#include "VecGeom/volumes/UnplacedPolyhedron.h"
+#include "VecGeom/volumes/UnplacedTrd.h"
+#include "VecGeom/volumes/UnplacedOrb.h"
+#include "VecGeom/volumes/UnplacedSphere.h"
+#include "VecGeom/volumes/UnplacedBooleanVolume.h"
+#include "VecGeom/volumes/UnplacedTorus2.h"
+#include "VecGeom/volumes/UnplacedTrapezoid.h"
+#include "VecGeom/volumes/UnplacedPolycone.h"
+#include "VecGeom/volumes/UnplacedScaledShape.h"
+#include "VecGeom/volumes/UnplacedGenTrap.h"
+#include "VecGeom/volumes/UnplacedSExtruVolume.h"
 #include "TError.h"
 #include "TGeoManager.h"
 #include "TGeoMaterial.h"
