@@ -13,6 +13,8 @@
 #include <ROOT/RBrowser.hxx>
 
 #include <ROOT/Browsable/RSysFile.hxx>
+#include <ROOT/Browsable/RLevelIter.hxx>
+#include <ROOT/Browsable/RProvider.hxx>
 
 #include <ROOT/RLogger.hxx>
 #include <ROOT/RFileDialog.hxx>
@@ -22,6 +24,7 @@
 
 #include "TString.h"
 #include "TSystem.h"
+#include "TError.h"
 #include "TROOT.h"
 #include "TBufferJSON.h"
 #include "TApplication.h"
@@ -48,7 +51,7 @@ public:
    std::string fTitle;
    std::string fFileName;
    std::string fContent;
-   bool fFirstSend{false};  ///<! if editor content was send at least one
+   bool fFirstSend{false};  ///<! if editor content was send at least once
    std::string fItemPath;   ///<! item path in the browser
 
    RBrowserEditorWidget(const std::string &name, bool is_editor = true) : RBrowserWidget(name), fIsEditor(is_editor) {}
@@ -62,7 +65,7 @@ public:
 
    void Show(const std::string &) override {}
 
-   bool DrawElement(std::shared_ptr<Browsable::RElement> &elem, const std::string &) override
+   bool DrawElement(std::shared_ptr<Browsable::RElement> &elem, const std::string & = "") override
    {
       if (fIsEditor && elem->IsCapable(Browsable::RElement::kActEdit)) {
          auto code = elem->GetContent("text");
@@ -116,6 +119,106 @@ public:
 
 };
 
+
+class RBrowserInfoWidget : public RBrowserWidget {
+public:
+
+   enum { kMaxContentLen = 10000000 };
+
+   std::string fTitle;
+   std::string fContent;
+   bool fFirstSend{false};  ///<! if editor content was send at least once
+
+   RBrowserInfoWidget(const std::string &name) : RBrowserWidget(name)
+   {
+      fTitle = "Cling info"s;
+      Refresh();
+   }
+
+   virtual ~RBrowserInfoWidget() = default;
+
+   void ResetConn() override { fFirstSend = false; }
+
+   std::string GetKind() const override { return "info"s; }
+   std::string GetTitle() override { return fTitle; }
+   std::string GetUrl() override { return ""s; }
+
+   void Show(const std::string &) override {}
+
+   bool DrawElement(std::shared_ptr<Browsable::RElement> &, const std::string & = "") override { return false; }
+
+   void Refresh()
+   {
+      fFirstSend = false;
+      fContent = "";
+
+      std::ostringstream pathtmp;
+      pathtmp << gSystem->TempDirectory() << "/info." << gSystem->GetPid() << ".log";
+
+      std::ofstream ofs(pathtmp.str(), std::ofstream::out | std::ofstream::app);
+      ofs << "";
+      ofs.close();
+
+      gSystem->RedirectOutput(pathtmp.str().c_str(), "a");
+      gROOT->ProcessLine(".g");
+      gSystem->RedirectOutput(nullptr);
+
+      std::ifstream infile(pathtmp.str());
+      if (infile) {
+         std::string line;
+         while (std::getline(infile, line) && (fContent.length() < kMaxContentLen)) {
+            fContent.append(line);
+            fContent.append("\n");
+         }
+      }
+
+      gSystem->Unlink(pathtmp.str().c_str());
+   }
+
+   void RefreshFromLogs(const std::string &promt, const std::vector<std::string> &logs)
+   {
+      int indx = 0, last_prompt = -1;
+      for (auto &line : logs) {
+         if (line == promt)
+            last_prompt = indx;
+         indx++;
+      }
+
+      if (last_prompt < 0) {
+         Refresh();
+         return;
+      }
+
+      fFirstSend = false;
+      fContent = "";
+
+      indx = 0;
+      for (auto &line : logs) {
+         if ((indx++ > last_prompt) && (fContent.length() < kMaxContentLen)) {
+            fContent.append(line);
+            fContent.append("\n");
+         }
+      }
+   }
+
+
+   std::string SendWidgetContent() override
+   {
+      if (fFirstSend)
+         return ""s;
+
+      if (fContent.empty())
+         Refresh();
+
+      fFirstSend = true;
+      std::vector<std::string> args = { GetName(), fTitle, fContent };
+
+      return "INFO:"s + TBufferJSON::ToJSON(&args).Data();
+   }
+
+};
+
+
 class RBrowserCatchedWidget : public RBrowserWidget {
 public:
 
@@ -124,7 +227,7 @@ public:
 
    void Show(const std::string &) override {}
 
-   std::string GetKind() const override { return "catched"; }
+   std::string GetKind() const override { return "catched"s; }
 
    std::string GetUrl() override { return fUrl; }
 
@@ -141,11 +244,7 @@ public:
 
 /** \class ROOT::Experimental::RBrowser
 \ingroup rbrowser
-\brief Web-based %ROOT file browser
-
-RBrowser requires one of the supported web browsers:
-  - Google Chrome (preferable)
-  - Mozilla Firefox
+\brief Web-based %ROOT files and objects browser
 
 \image html v7_rbrowser.png
 
@@ -156,6 +255,15 @@ RBrowser requires one of the supported web browsers:
 
 RBrowser::RBrowser(bool use_rcanvas)
 {
+   if (gROOT->IsWebDisplayBatch()) {
+      ::Warning("RBrowser::RBrowser", "The RBrowser cannot run in web batch mode");
+      return;
+   }
+
+   std::ostringstream pathtmp;
+   pathtmp << gSystem->TempDirectory() << "/command." << gSystem->GetPid() << ".log";
+   fPromptFileOutput = pathtmp.str();
+
    SetUseRCanvas(use_rcanvas);
 
    fBrowsable.CreateDefaultElements();
@@ -164,8 +272,8 @@ RBrowser::RBrowser(bool use_rcanvas)
    fWebWindow->SetDefaultPage("file:rootui5sys/browser/browser.html");
 
    // this is call-back, invoked when message received via websocket
-   fWebWindow->SetCallBacks([this](unsigned connid) { fConnId = connid; SendInitMsg(connid); },
-                            [this](unsigned connid, const std::string &arg) { ProcessMsg(connid, arg); });
+   fWebWindow->SetCallBacks([this](unsigned connid) { if (!fConnId) { fConnId = connid; SendInitMsg(connid); } else fConnId = 0xffffff; },
+                            [this](unsigned connid, const std::string &arg) { if ((connid == fConnId) && (fConnId != 0xffffff)) ProcessMsg(connid, arg); });
    fWebWindow->SetGeometry(1200, 700); // configure predefined window geometry
    fWebWindow->SetConnLimit(1); // the only connection is allowed
    fWebWindow->SetMaxQueueLength(30); // number of allowed entries in the window queue
@@ -178,8 +286,10 @@ RBrowser::RBrowser(bool use_rcanvas)
          kind = "rcanvas";
       else if (args.GetWidgetKind() == "TCanvas")
          kind = "tcanvas";
-      else if (args.GetWidgetKind() == "REveGeomViewer")
+      else if (args.GetWidgetKind() == "RGeomViewer")
          kind = "geom";
+      else if (args.GetWidgetKind() == "RTreeViewer")
+         kind = "tree";
 
       if (!fWebWindow || !fCatchWindowShow || kind.empty()) return false;
 
@@ -212,10 +322,10 @@ RBrowser::RBrowser(bool use_rcanvas)
 
 RBrowser::~RBrowser()
 {
-   fWebWindow->GetManager()->SetShowCallback(nullptr);
+   if (fWebWindow)
+      fWebWindow->GetManager()->SetShowCallback(nullptr);
 }
 
-
 //////////////////////////////////////////////////////////////////////////////////////////////
 /// Process browser request
 
@@ -272,8 +382,8 @@ std::string RBrowser::ProcessDblClick(std::vector<std::string> &args)
 {
    args.pop_back(); // remove exec string, not used now
 
-   std::string drawingOptions = args.back();
-   args.pop_back(); // remove draw option
+   std::string opt = args.back();
+   args.pop_back(); // remove option
 
    auto path = fBrowsable.GetWorkingPath();
    path.insert(path.end(), args.begin(), args.end());
@@ -301,6 +411,9 @@ std::string RBrowser::ProcessDblClick(std::vector<std::string> &args)
       if (!new_widget)
          return ""s;
 
+      // assign back pointer
+      new_widget->fBrowser = this;
+
       new_widget->Show("embed");
       fWidgets.emplace_back(new_widget);
       fActiveWidgetName = new_widget->GetName();
@@ -308,8 +421,13 @@ std::string RBrowser::ProcessDblClick(std::vector<std::string> &args)
       return NewWidgetMsg(new_widget);
    }
 
+   // before display tree or geometry ensure that they read and cached inside element
+   if (elem->IsCapable(Browsable::RElement::kActGeom) || elem->IsCapable(Browsable::RElement::kActTree)) {
+      elem->GetChildsIter();
+   }
+
    auto widget = GetActiveWidget();
-   if (widget && widget->DrawElement(elem, drawingOptions)) {
+   if (widget && widget->DrawElement(elem, opt)) {
       widget->SetPath(path);
       return widget->SendWidgetContent();
    }
@@ -327,11 +445,12 @@ std::string RBrowser::ProcessDblClick(std::vector<std::string> &args)
 
    std::string widget_kind;
    switch(dflt_action) {
-      case Browsable::RElement::kActGeom: widget_kind = "geom"; break;
       case Browsable::RElement::kActDraw6: widget_kind = "tcanvas"; break;
       case Browsable::RElement::kActDraw7: widget_kind = "rcanvas"; break;
       case Browsable::RElement::kActEdit: widget_kind = "editor"; break;
       case Browsable::RElement::kActImage: widget_kind = "image"; break;
+      case Browsable::RElement::kActTree: widget_kind = "tree"; break;
+      case Browsable::RElement::kActGeom: widget_kind = "geom"; break;
       default: widget_kind.clear();
    }
 
@@ -340,7 +459,7 @@ std::string RBrowser::ProcessDblClick(std::vector<std::string> &args)
       if (new_widget) {
          // draw object before client side is created - should not be a problem
          // after widget add in browser, connection will be established and data provided
-         if (new_widget->DrawElement(elem, drawingOptions))
+         if (new_widget->DrawElement(elem, opt))
             new_widget->SetPath(path);
          return NewWidgetMsg(new_widget);
       }
@@ -361,6 +480,7 @@ std::string RBrowser::ProcessDblClick(std::vector<std::string> &args)
 /// Show or update RBrowser in web window
 /// If web window already started - just refresh it like "reload" button does
 /// If no web window exists or \param always_start_new_browser configured, starts new window
+/// \param args display arguments
 
 void RBrowser::Show(const RWebDisplayArgs &args, bool always_start_new_browser)
 {
@@ -376,10 +496,8 @@ void RBrowser::Show(const RWebDisplayArgs &args, bool always_start_new_browser)
 
 void RBrowser::Hide()
 {
-   if (!fWebWindow)
-      return;
-
-   fWebWindow->CloseConnections();
+   if (fWebWindow)
+      fWebWindow->CloseConnections();
 }
 
 
@@ -392,10 +510,12 @@ std::shared_ptr<RBrowserWidget> RBrowser::AddWidget(const std::string &kind)
 
    std::shared_ptr<RBrowserWidget> widget;
 
-   if (kind == "editor")
+   if (kind == "editor"s)
       widget = std::make_shared<RBrowserEditorWidget>(name, true);
-   else if (kind == "image")
+   else if (kind == "image"s)
       widget = std::make_shared<RBrowserEditorWidget>(name, false);
+   else if (kind == "info"s)
+      widget = std::make_shared<RBrowserInfoWidget>(name);
    else
       widget = RBrowserWidgetProvider::CreateWidget(kind, name);
 
@@ -404,6 +524,7 @@ std::shared_ptr<RBrowserWidget> RBrowser::AddWidget(const std::string &kind)
       return nullptr;
    }
 
+   widget->fBrowser = this;
    widget->Show("embed");
    fWidgets.emplace_back(widget);
 
@@ -442,12 +563,14 @@ void RBrowser::AddInitWidget(const std::string &kind)
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////
-/// Returns active geometry viewer (if any)
+/// Find widget by name or kind
 
-std::shared_ptr<RBrowserWidget> RBrowser::FindWidget(const std::string &name) const
+std::shared_ptr<RBrowserWidget> RBrowser::FindWidget(const std::string &name, const std::string &kind) const
 {
    auto iter = std::find_if(fWidgets.begin(), fWidgets.end(),
-         [name](const std::shared_ptr<RBrowserWidget> &widget) { return name == widget->GetName(); });
+         [name, kind](const std::shared_ptr<RBrowserWidget> &widget) {
+           return kind.empty() ? name == widget->GetName() : kind == widget->GetKind();
+   });
 
    if (iter != fWidgets.end())
       return *iter;
@@ -498,10 +621,7 @@ std::vector<std::string> RBrowser::GetRootLogs()
 {
    std::vector<std::string> arr;
 
-   std::ostringstream pathtmp;
-   pathtmp << gSystem->TempDirectory() << "/command." << gSystem->GetPid() << ".log";
-
-   std::ifstream infile(pathtmp.str());
+   std::ifstream infile(fPromptFileOutput);
    if (infile) {
       std::string line;
       while (std::getline(infile, line) && (arr.size() < 10000)) {
@@ -527,20 +647,27 @@ void RBrowser::SendInitMsg(unsigned connid)
    }
 
    if (!fActiveWidgetName.empty())
-      reply.emplace_back(std::vector<std::string>({ "active", fActiveWidgetName }));
+      reply.emplace_back(std::vector<std::string>({ "active"s, fActiveWidgetName }));
 
    auto history = GetRootHistory();
    if (history.size() > 0) {
-      history.insert(history.begin(), "history");
+      history.insert(history.begin(), "history"s);
       reply.emplace_back(history);
    }
 
    auto logs = GetRootLogs();
    if (logs.size() > 0) {
-      logs.insert(logs.begin(), "logs");
+      logs.insert(logs.begin(), "logs"s);
       reply.emplace_back(logs);
    }
 
+   reply.emplace_back(std::vector<std::string>({
+      "drawoptions"s,
+      Browsable::RProvider::GetClassDrawOption("TH1"),
+      Browsable::RProvider::GetClassDrawOption("TH2"),
+      Browsable::RProvider::GetClassDrawOption("TProfile")
+   }));
+
    std::string msg = "INMSG:";
    msg.append(TBufferJSON::ToJSON(&reply, TBufferJSON::kNoSpaces).Data());
 
@@ -560,7 +687,8 @@ std::string RBrowser::GetCurrentWorkingDirectory()
 
 std::string RBrowser::NewWidgetMsg(std::shared_ptr<RBrowserWidget> &widget)
 {
-   std::vector<std::string> arr = { widget->GetKind(), widget->GetUrl(), widget->GetName(), widget->GetTitle() };
+   std::vector<std::string> arr = { widget->GetKind(), widget->GetUrl(), widget->GetName(), widget->GetTitle(),
+                                    Browsable::RElement::GetPathAsString(widget->GetPath()) };
    return "NEWWIDGET:"s + TBufferJSON::ToJSON(&arr, TBufferJSON::kNoSpaces).Data();
 }
 
@@ -628,21 +756,35 @@ void RBrowser::ProcessMsg(unsigned connid, const std::string &arg0)
       fWebWindow->Send(connid, GetCurrentWorkingDirectory());
    } else if (kind == "CMD") {
       std::string sPrompt = "root []";
-      std::ostringstream pathtmp;
-      pathtmp << gSystem->TempDirectory() << "/command." << gSystem->GetPid() << ".log";
       TApplication *app = gROOT->GetApplication();
       if (app->InheritsFrom("TRint")) {
          sPrompt = ((TRint*)gROOT->GetApplication())->GetPrompt();
          Gl_histadd((char *)msg.c_str());
       }
 
-      std::ofstream ofs(pathtmp.str(), std::ofstream::out | std::ofstream::app);
+      std::ofstream ofs(fPromptFileOutput, std::ofstream::out | std::ofstream::app);
       ofs << sPrompt << msg << std::endl;
       ofs.close();
 
-      gSystem->RedirectOutput(pathtmp.str().c_str(), "a");
+      gSystem->RedirectOutput(fPromptFileOutput.c_str(), "a");
       gROOT->ProcessLine(msg.c_str());
       gSystem->RedirectOutput(nullptr);
+
+      if (msg == ".g"s) {
+         auto widget = std::dynamic_pointer_cast<RBrowserInfoWidget>(FindWidget(""s, "info"s));
+         if (!widget) {
+            auto new_widget = AddWidget("info"s);
+            fWebWindow->Send(connid, NewWidgetMsg(new_widget));
+            widget = std::dynamic_pointer_cast<RBrowserInfoWidget>(new_widget);
+         } else if (fActiveWidgetName != widget->GetName()) {
+            fWebWindow->Send(connid, "SELECT_WIDGET:"s + widget->GetName());
+            fActiveWidgetName = widget->GetName();
+         }
+
+         if (widget)
+            widget->RefreshFromLogs(sPrompt + msg, GetRootLogs());
+      }
+
       CheckWidgtesModified();
    } else if (kind == "GETHISTORY") {
 
@@ -673,6 +815,12 @@ void RBrowser::ProcessMsg(unsigned connid, const std::string &arg0)
             }
          }
       }
+   } else if (kind == "GETINFO") {
+      auto info = std::dynamic_pointer_cast<RBrowserInfoWidget>(FindWidget(msg));
+      if (info) {
+         info->Refresh();
+         fWebWindow->Send(connid, info->SendWidgetContent());
+      }
    } else if (kind == "NEWWIDGET") {
       auto widget = AddWidget(msg);
       if (widget)
@@ -685,6 +833,13 @@ void RBrowser::ProcessMsg(unsigned connid, const std::string &arg0)
          fBrowsable.SetWorkingPath({});
       }
       fWebWindow->Send(connid, GetCurrentWorkingDirectory());
+   } else if (kind == "OPTIONS") {
+      auto arr = TBufferJSON::FromJSON<std::vector<std::string>>(msg);
+      if (arr && (arr->size() == 3)) {
+         Browsable::RProvider::SetClassDrawOption("TH1", (*arr)[0]);
+         Browsable::RProvider::SetClassDrawOption("TH2", (*arr)[1]);
+         Browsable::RProvider::SetClassDrawOption("TProfile", (*arr)[2]);
+      }
    }
 }
 
@@ -702,3 +857,31 @@ void RBrowser::SetWorkingPath(const std::string &path)
    }
 }
 
+//////////////////////////////////////////////////////////////////////////////////////////////
+/// Activate widget in RBrowser
+/// One should specify title and (optionally) kind of widget like "tcanvas" or "geom"
+
+bool RBrowser::ActivateWidget(const std::string &title, const std::string &kind)
+{
+   if (title.empty())
+      return false;
+
+   for (auto &widget : fWidgets) {
+
+      if (widget->GetTitle() != title)
+         continue;
+
+      if (!kind.empty() && (widget->GetKind() != kind))
+         continue;
+
+      if (fWebWindow)
+         fWebWindow->Send(0, "SELECT_WIDGET:"s + widget->GetName());
+      else
+         fActiveWidgetName = widget->GetName();
+      return true;
+   }
+
+   return false;
+}
+
+
