@@ -21,13 +21,14 @@
 #include "THttpServer.h"
 
 #include "TSystem.h"
-#include "TRandom.h"
 #include "TString.h"
 #include "TApplication.h"
 #include "TTimer.h"
+#include "TRandom.h"
 #include "TROOT.h"
 #include "TEnv.h"
 #include "TExec.h"
+#include "TSocket.h"
 
 #include <thread>
 #include <chrono>
@@ -37,8 +38,9 @@ using namespace ROOT::Experimental;
 
 ///////////////////////////////////////////////////////////////
 /// Parse boolean gEnv variable which should be "yes" or "no"
-/// Returns 1 for true or 0 for false
+/// \return 1 for true or 0 for false
 /// Returns \param dflt if result is not defined
+/// \param name name of the env variable
 
 int RWebWindowWSHandler::GetBoolEnv(const std::string &name, int dflt)
 {
@@ -64,7 +66,7 @@ Central instance to create and show web-based windows like Canvas or FitPanel.
 Manager responsible to creating THttpServer instance, which is used for RWebWindow's
 communication with clients.
 
-Method RWebWindowsManager::Show() used to show window in specified location.
+Method RWebWindows::Show() used to show window in specified location.
 */
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -87,6 +89,7 @@ std::shared_ptr<RWebWindowsManager> &RWebWindowsManager::Instance()
 
 static std::thread::id gWebWinMainThrd = std::this_thread::get_id();
 static bool gWebWinMainThrdSet = true;
+static bool gWebWinLoopbackMode = true;
 
 //////////////////////////////////////////////////////////////////////////////////////////
 /// Returns true when called from main process
@@ -113,6 +116,23 @@ void RWebWindowsManager::AssignMainThrd()
    gWebWinMainThrd = std::this_thread::get_id();
 }
 
+
+//////////////////////////////////////////////////////////////////////////////////////////
+/// Set loopback mode for THttpServer used for web widgets
+/// By default is on. Only local communication via localhost address is possible
+/// Disable it only if really necessary - it may open unauthorized access to your application from external nodes!!
+
+void RWebWindowsManager::SetLoopbackMode(bool on)
+{
+   gWebWinLoopbackMode = on;
+   if (!on) {
+      printf("\nWARNING!\n");
+      printf("Disabling loopback mode may leads to security problem.\n");
+      printf("See https://root.cern/about/security/ for more information.\n\n");
+   }
+}
+
+
 //////////////////////////////////////////////////////////////////////////////////////////
 /// window manager constructor
 /// Required here for correct usage of unique_ptr<THttpServer>
@@ -153,6 +173,43 @@ void RWebWindowsManager::AssignWindowThreadId(RWebWindow &win)
    }
 }
 
+//////////////////////////////////////////////////////////////////////////////////////////
+/// If ROOT_LISTENER_SOCKET variable is configured,
+/// message will be sent to that unix socket
+
+bool RWebWindowsManager::InformListener(const std::string &msg)
+{
+#ifdef R__WIN32
+   (void) msg;
+   return false;
+
+#else
+
+   const char *fname = gSystem->Getenv("ROOT_LISTENER_SOCKET");
+   if (!fname || !*fname)
+      return false;
+
+   TSocket s(fname);
+   if (!s.IsValid()) {
+      R__LOG_ERROR(WebGUILog()) << "Problem with open listener socket " << fname << ", check ROOT_LISTENER_SOCKET environment variable";
+      return false;
+   }
+
+   int res = s.SendRaw(msg.c_str(), msg.length());
+
+   s.Close();
+
+   if (res > 0) {
+      // workaround to let handle socket by system outside ROOT process
+      gSystem->ProcessEvents();
+      gSystem->Sleep(10);
+   }
+
+   return res > 0;
+#endif
+}
+
+
 //////////////////////////////////////////////////////////////////////////////////////////
 /// Creates http server, if required - with real http engine (civetweb)
 /// One could configure concrete HTTP port, which should be used for the server,
@@ -181,6 +238,14 @@ void RWebWindowsManager::AssignWindowThreadId(RWebWindow &win)
 ///      WebGui.UseHttps: yes
 ///      WebGui.ServerCert: sertificate_filename.pem
 ///
+/// Alternatively, one can specify unix socket to handle requests:
+///
+///      WebGui.UnixSocket: /path/to/unix/socket
+///      WebGui.UnixSocketMode: 0700
+///
+/// Typically one used unix sockets together with server mode like `root --web=server:/tmp/root.socket` and
+/// then redirect it via ssh tunnel (e.g. using `rootssh`) to client node
+///
 /// All incoming requests processed in THttpServer in timer handler with 10 ms timeout.
 /// One may decrease value to improve latency or increase value to minimize CPU load
 ///
@@ -221,6 +286,10 @@ void RWebWindowsManager::AssignWindowThreadId(RWebWindow &win)
 ///
 ///      WebGui.HttpMaxAge: 3600
 ///
+/// Also one can provide extra URL options for, see TCivetweb::Create for list of supported options
+///
+///      WebGui.HttpExtraArgs: winsymlinks=no
+///
 /// One also can configure usage of FastCGI server for web windows:
 ///
 ///      WebGui.FastCgiPort: 4000
@@ -291,22 +360,31 @@ bool RWebWindowsManager::CreateServer(bool with_http)
    int http_thrds = gEnv->GetValue("WebGui.HttpThreads", 10);
    int http_wstmout = gEnv->GetValue("WebGui.HttpWSTmout", 10000);
    int http_maxage = gEnv->GetValue("WebGui.HttpMaxAge", -1);
+   const char *extra_args = gEnv->GetValue("WebGui.HttpExtraArgs", "");
    int fcgi_port = gEnv->GetValue("WebGui.FastCgiPort", 0);
    int fcgi_thrds = gEnv->GetValue("WebGui.FastCgiThreads", 10);
    const char *fcgi_serv = gEnv->GetValue("WebGui.FastCgiServer", "");
    fLaunchTmout = gEnv->GetValue("WebGui.LaunchTmout", 30.);
-   // always use loopback
-   bool assign_loopback = true; // RWebWindowWSHandler::GetBoolEnv("WebGui.HttpLoopback", 1) == 1;
+   bool assign_loopback = gWebWinLoopbackMode;
    const char *http_bind = gEnv->GetValue("WebGui.HttpBind", "");
    bool use_secure = RWebWindowWSHandler::GetBoolEnv("WebGui.UseHttps", 0) == 1;
    const char *ssl_cert = gEnv->GetValue("WebGui.ServerCert", "rootserver.pem");
 
+   const char *unix_socket = gSystem->Getenv("ROOT_WEBGUI_SOCKET");
+   if (!unix_socket || !*unix_socket)
+      unix_socket = gEnv->GetValue("WebGui.UnixSocket", "");
+   const char *unix_socket_mode = gEnv->GetValue("WebGui.UnixSocketMode", "0700");
+   bool use_unix_socket = unix_socket && *unix_socket;
+
+   if (use_unix_socket)
+      fcgi_port = http_port = -1;
+
    if (assign_loopback)
       fcgi_port = -1;
 
    int ntry = 100;
 
-   if ((http_port < 0) && (fcgi_port <= 0)) {
+   if ((http_port < 0) && (fcgi_port <= 0) && !use_unix_socket) {
       R__LOG_ERROR(WebGUILog()) << "Not allowed to create HTTP server, check WebGui.HttpPort variable";
       return false;
    }
@@ -328,8 +406,11 @@ bool RWebWindowsManager::CreateServer(bool with_http)
    if (fcgi_port > 0)
       ntry++;
 
+   if (use_unix_socket)
+      ntry++;
+
    while (ntry-- >= 0) {
-      if ((http_port == 0) && (fcgi_port <= 0)) {
+      if ((http_port == 0) && (fcgi_port <= 0) && !use_unix_socket) {
          if ((http_min <= 0) || (http_max <= http_min)) {
             R__LOG_ERROR(WebGUILog()) << "Wrong HTTP range configuration, check WebGui.HttpPortMin/Max variables";
             return false;
@@ -339,41 +420,61 @@ bool RWebWindowsManager::CreateServer(bool with_http)
       }
 
       TString engine, url;
-
       if (fcgi_port > 0) {
          engine.Form("fastcgi:%d?thrds=%d", fcgi_port, fcgi_thrds);
-         if (!fServer->CreateEngine(engine)) return false;
-         if (fcgi_serv && (strlen(fcgi_serv) > 0)) fAddr = fcgi_serv;
-         if (http_port < 0) return true;
+         if (!fServer->CreateEngine(engine))
+            return false;
+         if (fcgi_serv && (strlen(fcgi_serv) > 0))
+            fAddr = fcgi_serv;
+         if (http_port < 0)
+            return true;
          fcgi_port = 0;
-      } else if (http_port > 0) {
-         url = use_secure ? "https://" : "http://";
-         engine.Form("%s:%d?thrds=%d&websocket_timeout=%d", (use_secure ? "https" : "http"), http_port, http_thrds, http_wstmout);
-         if (assign_loopback) {
-            engine.Append("&loopback");
-            url.Append("localhost");
-         } else if (http_bind && (strlen(http_bind) > 0)) {
-            engine.Append("&bind=");
-            engine.Append(http_bind);
-            url.Append(http_bind);
+      } else {
+         if (use_unix_socket) {
+            engine.Form("socket:%s?socket_mode=%s&", unix_socket, unix_socket_mode);
          } else {
-            url.Append("localhost");
+            url = use_secure ? "https://" : "http://";
+            engine.Form("%s:%d?", (use_secure ? "https" : "http"), http_port);
+            if (assign_loopback) {
+               engine.Append("loopback&");
+               url.Append("localhost");
+            } else if (http_bind && (strlen(http_bind) > 0)) {
+               engine.Append(TString::Format("bind=%s&", http_bind));
+               url.Append(http_bind);
+            } else {
+               url.Append("localhost");
+            }
          }
 
+         engine.Append(TString::Format("thrds=%d&websocket_timeout=%d", http_thrds, http_wstmout));
+
          if (http_maxage >= 0)
             engine.Append(TString::Format("&max_age=%d", http_maxage));
 
-         if (use_secure) {
+         if (use_secure && !strchr(ssl_cert,'&')) {
             engine.Append("&ssl_cert=");
             engine.Append(ssl_cert);
          }
 
+         if (!use_unix_socket && !assign_loopback && extra_args && strlen(extra_args) > 0) {
+            engine.Append("&");
+            engine.Append(extra_args);
+         }
+
          if (fServer->CreateEngine(engine)) {
-            fAddr = url.Data();
-            fAddr.append(":");
-            fAddr.append(std::to_string(http_port));
+            if (use_unix_socket) {
+               fAddr = "socket://"; // fictional socket URL
+               fAddr.append(unix_socket);
+               // InformListener(std::string("socket:") + unix_socket + "\n");
+            } else if (http_port > 0) {
+               fAddr = url.Data();
+               fAddr.append(":");
+               fAddr.append(std::to_string(http_port));
+               // InformListener(std::string("http:") + std::to_string(http_port) + "\n");
+            }
             return true;
          }
+         use_unix_socket = false;
          http_port = 0;
       }
    }
@@ -475,6 +576,7 @@ std::string RWebWindowsManager::GetUrl(const RWebWindow &win, bool remote)
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 /// Show web window in specified location.
 ///
+/// \param[inout] win web window by reference
 /// \param user_args specifies where and how display web window
 ///
 /// As display args one can use string like "firefox" or "chrome" - these are two main supported web browsers.
@@ -486,6 +588,8 @@ std::string RWebWindowsManager::GetUrl(const RWebWindow &win, bool remote)
 ///
 /// Following parameters can be configured in rootrc file:
 ///
+///      WebGui.Display: kind of display like chrome or firefox or browser, can be overwritten by --web=value command line argument
+///      WebGui.OnetimeKey: if configured requires unique key every time window is connected (default no)
 ///      WebGui.Chrome: full path to Google Chrome executable
 ///      WebGui.ChromeBatch: command to start chrome in batch, used for image production, like "$prog --headless --disable-gpu $geometry $url"
 ///      WebGui.ChromeHeadless: command to start chrome in headless mode, like "fork: --headless --disable-gpu $geometry $url"
@@ -536,13 +640,8 @@ unsigned RWebWindowsManager::ShowWindow(RWebWindow &win, const RWebDisplayArgs &
       return 0;
    }
 
-   std::string key;
-   int ntry = 100000;
-
-   do {
-      key = std::to_string(gRandom->Integer(0x100000));
-   } while ((--ntry > 0) && win.HasKey(key));
-   if (ntry == 0) {
+   std::string key = win.GenerateKey();
+   if (key.empty()) {
       R__LOG_ERROR(WebGUILog()) << "Fail to create unique key for the window";
       return 0;
    }
@@ -554,11 +653,13 @@ unsigned RWebWindowsManager::ShowWindow(RWebWindow &win, const RWebDisplayArgs &
       return 0;
    }
 
-   if (args.GetWidth() <= 0) args.SetWidth(win.GetWidth());
-   if (args.GetHeight() <= 0) args.SetHeight(win.GetHeight());
+   if (args.GetWidth() <= 0)
+      args.SetWidth(win.GetWidth());
+   if (args.GetHeight() <= 0)
+      args.SetHeight(win.GetHeight());
 
    bool normal_http = !args.IsLocalDisplay();
-   if (!normal_http && (gEnv->GetValue("WebGui.ForceHttp",0) == 1))
+   if (!normal_http && (gEnv->GetValue("WebGui.ForceHttp", 0) == 1))
       normal_http = true;
 
    std::string url = GetUrl(win, normal_http);
@@ -574,15 +675,44 @@ unsigned RWebWindowsManager::ShowWindow(RWebWindow &win, const RWebDisplayArgs &
    args.SetUrl(url);
 
    args.AppendUrlOpt(std::string("key=") + key);
-   if (args.IsHeadless()) args.AppendUrlOpt("headless"); // used to create holder request
+   if (args.IsHeadless())
+      args.AppendUrlOpt("headless"); // used to create holder request
    if (!token.empty())
       args.AppendUrlOpt(std::string("token=") + token);
 
-   if (!args.IsHeadless() && (gROOT->GetWebDisplay() == "server")) {
+   if (!args.IsHeadless() && normal_http) {
+      auto winurl = args.GetUrl();
+      winurl.erase(0, fAddr.length());
+      InformListener(std::string("win:") + winurl);
+   }
+
+   if (!args.IsHeadless() && ((args.GetBrowserKind() == RWebDisplayArgs::kServer) || gROOT->IsWebDisplayBatch()) /*&& (RWebWindowWSHandler::GetBoolEnv("WebGui.OnetimeKey") != 1)*/) {
       std::cout << "New web window: " << args.GetUrl() << std::endl;
       return 0;
    }
 
+   if (fAddr.compare(0,9,"socket://") == 0)
+      return 0;
+
+#if !defined(R__MACOSX) && !defined(R__WIN32)
+   if (args.IsInteractiveBrowser()) {
+      const char *varname = "WebGui.CheckRemoteDisplay";
+      if (RWebWindowWSHandler::GetBoolEnv(varname, 1) == 1) {
+         const char *displ = gSystem->Getenv("DISPLAY");
+         if (displ && *displ && (*displ != ':')) {
+            gEnv->SetValue(varname, "no");
+            std::cout << "\n"
+               "ROOT web-based widget started in the session where DISPLAY set to " << displ << "\n" <<
+               "Means web browser will be displayed on remote X11 server which is usually very inefficient\n"
+               "One can start ROOT session in server mode like \"root -b --web=server:8877\" and forward http port to display node\n"
+               "Or one can use rootssh script to configure pore forwarding and display web widgets automatically\n"
+               "Find more info on https://root.cern/for_developers/root7/#rbrowser\n"
+               "This message can be disabled by setting \"" << varname << ": no\" in .rootrc file\n";
+         }
+      }
+   }
+#endif
+
    if (!normal_http)
       args.SetHttpServer(GetServer());
 
