@@ -54,8 +54,7 @@ macro gives an example.
 
 Begin_Macro(source)
 {
-   TCanvas *c1 = new TCanvas("c1","c1",600,400);
-   TH2F *h2 = new TH2F("h2","Example of a resized palette ",40,-4,4,40,-20,20);
+   auto h2 = new TH2F("h2","Example of a resized palette ",40,-4,4,40,-20,20);
    Float_t px, py;
    for (Int_t i = 0; i < 25000; i++) {
       gRandom->Rannor(px,py);
@@ -64,9 +63,8 @@ Begin_Macro(source)
    gStyle->SetPalette(1);
    h2->Draw("COLZ");
    gPad->Update();
-   TPaletteAxis *palette = (TPaletteAxis*)h2->GetListOfFunctions()->FindObject("palette");
+   auto palette = (TPaletteAxis*)h2->GetListOfFunctions()->FindObject("palette");
    palette->SetY2NDC(0.7);
-   return c1;
 }
 End_Macro
 
@@ -93,8 +91,8 @@ An example is shown here:
 Begin_Macro(source)
 {
    gStyle->SetOptStat(0);
-   TCanvas *c1 = new TCanvas("c1","exa_CJUST",300,10,400,400);
-   TH2F *hpxpy = new TH2F("hpxpy","py vs px",40,-4,4,40,-4,4);
+   auto c = new TCanvas("c","exa_CJUST",300,10,400,400);
+   auto hpxpy = new TH2F("hpxpy","py vs px",40,-4,4,40,-4,4);
    // Fill histograms randomly
    TRandom3 randomNum;
    Float_t px, py;
@@ -120,7 +118,7 @@ End_Macro
 
 TPaletteAxis::TPaletteAxis(): TPave()
 {
-   fH  = 0;
+   fH = nullptr;
    SetName("");
 }
 
@@ -132,6 +130,7 @@ TPaletteAxis::TPaletteAxis(Double_t x1, Double_t y1, Double_t x2, Double_t  y2,
    : TPave(x1, y1, x2, y2)
 {
    fH = h;
+   if (!fH) return;
    SetName("palette");
    TAxis *zaxis = fH->GetZaxis();
    fAxis.ImportAxisAttributes(zaxis);
@@ -152,7 +151,7 @@ TPaletteAxis::~TPaletteAxis()
 
 TPaletteAxis::TPaletteAxis(const TPaletteAxis &palette) : TPave(palette)
 {
-   ((TPaletteAxis&)palette).Copy(*this);
+   palette.TPaletteAxis::Copy(*this);
 }
 
 
@@ -161,7 +160,8 @@ TPaletteAxis::TPaletteAxis(const TPaletteAxis &palette) : TPave(palette)
 
 TPaletteAxis& TPaletteAxis::operator=(const TPaletteAxis &orig)
 {
-   orig.Copy( *this );
+   if (this != &orig)
+      orig.TPaletteAxis::Copy(*this);
    return *this;
 }
 
@@ -258,26 +258,28 @@ void TPaletteAxis::ExecuteEvent(Int_t event, Int_t px, Int_t py)
             ratio2 = temp;
          }
          if (ratio2 - ratio1 > 0.05) {
-            if (fH->GetDimension() == 2) {
-               Double_t zmin = fH->GetMinimum();
-               Double_t zmax = fH->GetMaximum();
-               if (gPad->GetLogz()) {
-                  if (zmin <= 0 && zmax > 0) zmin = TMath::Min((Double_t)1,
-                                                                  (Double_t)0.001 * zmax);
-                  zmin = TMath::Log10(zmin);
-                  zmax = TMath::Log10(zmax);
+            if (fH) {
+               if (fH->GetDimension() == 2) {
+                  Double_t zmin = fH->GetMinimum();
+                  Double_t zmax = fH->GetMaximum();
+                  if (gPad->GetLogz()) {
+                     if (zmin <= 0 && zmax > 0) zmin = TMath::Min((Double_t)1,
+                                                                     (Double_t)0.001 * zmax);
+                     zmin = TMath::Log10(zmin);
+                     zmax = TMath::Log10(zmax);
+                  }
+                  Double_t newmin = zmin + (zmax - zmin) * ratio1;
+                  Double_t newmax = zmin + (zmax - zmin) * ratio2;
+                  if (newmin < zmin)newmin = fH->GetBinContent(fH->GetMinimumBin());
+                  if (newmax > zmax)newmax = fH->GetBinContent(fH->GetMaximumBin());
+                  if (gPad->GetLogz()) {
+                     newmin = TMath::Exp(2.302585092994 * newmin);
+                     newmax = TMath::Exp(2.302585092994 * newmax);
+                  }
+                  fH->SetMinimum(newmin);
+                  fH->SetMaximum(newmax);
+                  fH->SetBit(TH1::kIsZoomed);
                }
-               Double_t newmin = zmin + (zmax - zmin) * ratio1;
-               Double_t newmax = zmin + (zmax - zmin) * ratio2;
-               if (newmin < zmin)newmin = fH->GetBinContent(fH->GetMinimumBin());
-               if (newmax > zmax)newmax = fH->GetBinContent(fH->GetMaximumBin());
-               if (gPad->GetLogz()) {
-                  newmin = TMath::Exp(2.302585092994 * newmin);
-                  newmax = TMath::Exp(2.302585092994 * newmax);
-               }
-               fH->SetMinimum(newmin);
-               fH->SetMaximum(newmax);
-               fH->SetBit(TH1::kIsZoomed);
             }
             gPad->Modified(kTRUE);
          }
@@ -307,6 +309,7 @@ void TPaletteAxis::ExecuteEvent(Int_t event, Int_t px, Int_t py)
 
 Int_t TPaletteAxis::GetBinColor(Int_t i, Int_t j)
 {
+   if (!fH) return 0;
    Double_t zc = fH->GetBinContent(i, j);
    return GetValueColor(zc);
 }
@@ -320,8 +323,12 @@ char *TPaletteAxis::GetObjectInfo(Int_t /* px */, Int_t py) const
    Double_t z;
    static char info[64];
 
-   Double_t zmin = fH->GetMinimum();
-   Double_t zmax = fH->GetMaximum();
+   Double_t zmin = 0.;
+   Double_t zmax = 0.;
+   if (fH) {
+      zmin = fH->GetMinimum();
+      zmax = fH->GetMaximum();
+   }
    Int_t   y1   = gPad->GetWh() - gPad->VtoPixel(fY1NDC);
    Int_t   y2   = gPad->GetWh() - gPad->VtoPixel(fY2NDC);
    Int_t   y    = gPad->GetWh() - py;
@@ -361,6 +368,8 @@ char *TPaletteAxis::GetObjectInfo(Int_t /* px */, Int_t py) const
 
 Int_t TPaletteAxis::GetValueColor(Double_t zc)
 {
+   if (!fH) return 0;
+
    Double_t wmin  = fH->GetMinimum();
    Double_t wmax  = fH->GetMaximum();
    Double_t wlmin = wmin;
@@ -374,7 +383,8 @@ Int_t TPaletteAxis::GetValueColor(Double_t zc)
    }
 
    Int_t ncolors = gStyle->GetNumberOfColors();
-   Int_t ndivz = fH->GetContour();
+   Int_t ndivz =0;
+   if (fH) ndivz = fH->GetContour();
    if (ndivz == 0) return 0;
    ndivz = TMath::Abs(ndivz);
    Int_t theColor, color;
@@ -395,6 +405,9 @@ Int_t TPaletteAxis::GetValueColor(Double_t zc)
 
 void TPaletteAxis::Paint(Option_t *)
 {
+
+   if (!fH) return;
+
    ConvertNDCtoPad();
 
    SetFillStyle(1001);
@@ -434,8 +447,8 @@ void TPaletteAxis::Paint(Option_t *)
    // import Attributes already here since we might need them for CJUST
    if (fH->GetDimension() == 2) fAxis.ImportAxisAttributes(fH->GetZaxis());
    // case option "CJUST": put labels directly at color boundaries
-   TLatex *label = NULL;
-   TLine *line = NULL;
+   TLatex *label = nullptr;
+   TLine *line = nullptr;
    Double_t prevlab = 0;
    TString opt(fH->GetDrawOption());
    if (opt.Contains("CJUST", TString::kIgnoreCase)) {
@@ -554,7 +567,10 @@ void TPaletteAxis::Paint(Option_t *)
 
 void TPaletteAxis::SavePrimitive(std::ostream &out, Option_t * /*= ""*/)
 {
-   //char quote = '"';
+   if (!fH) return;
+
+   char quote = '"';
+
    out << "   " << std::endl;
    if (gROOT->ClassSaved(TPaletteAxis::Class())) {
       out << "   ";
@@ -568,12 +584,19 @@ void TPaletteAxis::SavePrimitive(std::ostream &out, Option_t * /*= ""*/)
       out << "palette = new " << ClassName() << "(" << fX1 << "," << fY1 << "," << fX2 << "," << fY2
           << "," << fH->GetName() << ");" << std::endl;
    }
-   out << "   palette->SetLabelColor(" << fAxis.GetLabelColor() << ");" << std::endl;
-   out << "   palette->SetLabelFont("  << fAxis.GetLabelFont() << ");" << std::endl;
-   out << "   palette->SetLabelOffset(" << fAxis.GetLabelOffset() << ");" << std::endl;
-   out << "   palette->SetLabelSize("  << fAxis.GetLabelSize() << ");" << std::endl;
-   out << "   palette->SetTitleOffset(" << fAxis.GetTitleOffset() << ");" << std::endl;
-   out << "   palette->SetTitleSize("  << fAxis.GetTitleSize() << ");" << std::endl;
+   out << "   palette->SetNdivisions(" << fH->GetZaxis()->GetNdivisions() << ");" << std::endl;
+   out << "   palette->SetAxisColor(" << fH->GetZaxis()->GetAxisColor() << ");" << std::endl;
+   out << "   palette->SetLabelColor(" << fH->GetZaxis()->GetLabelColor() << ");" << std::endl;
+   out << "   palette->SetLabelFont(" << fH->GetZaxis()->GetLabelFont() << ");" << std::endl;
+   out << "   palette->SetLabelOffset(" << fH->GetZaxis()->GetLabelOffset() << ");" << std::endl;
+   out << "   palette->SetLabelSize(" << fH->GetZaxis()->GetLabelSize() << ");" << std::endl;
+   out << "   palette->SetMaxDigits(" << fH->GetZaxis()->GetMaxDigits() << ");" << std::endl;
+   out << "   palette->SetTickLength(" << fH->GetZaxis()->GetTickLength() << ");" << std::endl;
+   out << "   palette->SetTitleOffset(" << fH->GetZaxis()->GetTitleOffset() << ");" << std::endl;
+   out << "   palette->SetTitleSize(" << fH->GetZaxis()->GetTitleSize() << ");" << std::endl;
+   out << "   palette->SetTitleColor(" << fH->GetZaxis()->GetTitleColor() << ");" << std::endl;
+   out << "   palette->SetTitleFont(" << fH->GetZaxis()->GetTitleFont() << ");" << std::endl;
+   out << "   palette->SetTitle(" << quote << fH->GetZaxis()->GetTitle() << quote << ");" << std::endl;
    SaveFillAttributes(out, "palette", -1, -1);
    SaveLineAttributes(out, "palette", 1, 1, 1);
 }
@@ -584,10 +607,11 @@ void TPaletteAxis::SavePrimitive(std::ostream &out, Option_t * /*= ""*/)
 
 void TPaletteAxis::UnZoom()
 {
-   TView *view = gPad->GetView();
+   if (!fH) return;
+   TView *view = gPad ? gPad->GetView() : nullptr;
    if (view) {
       delete view;
-      gPad->SetView(0);
+      gPad->SetView(nullptr);
    }
    fH->GetZaxis()->SetRange(0, 0);
    if (fH->GetDimension() == 2) {
