@@ -58,25 +58,21 @@ TSpectrum2Transform::TSpectrum2Transform()
 
 TSpectrum2Transform::TSpectrum2Transform(Int_t sizeX, Int_t sizeY) :TObject()
 {
-   Int_t j1, j2, n;
+   Int_t n;
    if (sizeX <= 0 || sizeY <= 0){
       Error ("TSpectrumTransform","Invalid length, must be > than 0");
       return;
    }
-   j1 = 0;
    n = 1;
    for (; n < sizeX;) {
-      j1 += 1;
       n = n * 2;
    }
    if (n != sizeX){
       Error ("TSpectrumTransform","Invalid length, must be power of 2");
       return;
    }
-   j2 = 0;
    n = 1;
    for (; n < sizeY;) {
-      j2 += 1;
       n = n * 2;
    }
    if (n != sizeY){
@@ -780,7 +776,7 @@ void TSpectrum2Transform::HaarWalsh2(Double_t **working_matrix,
 void TSpectrum2Transform::FourCos2(Double_t **working_matrix, Double_t *working_vector,
                             Int_t numx, Int_t numy, Int_t direction, Int_t type)
 {
-   Int_t i, j, iterx, itery, n, size;
+   Int_t i, j, n, size;
    Double_t pi = 3.14159265358979323846;
    j = 0;
    n = 1;
@@ -795,15 +791,11 @@ void TSpectrum2Transform::FourCos2(Double_t **working_matrix, Double_t *working_
       n = n * 2;
    }
    i = numx;
-   iterx = 0;
    for (; i > 1;) {
-      iterx += 1;
       i = i / 2;
    }
    i = numy;
-   itery = 0;
    for (; i > 1;) {
-      itery += 1;
       i = i / 2;
    }
    size = numx;
