@@ -903,7 +903,7 @@ void  TDocParser::DeleteDirectiveOutput() const
 /// Input: line - line containing the CPP statement,
 ///        pos  - position of '#'
 ///
-///  NOTE: Looks for the #include statements and
+///  NOTE: Looks for the `#include` statements and
 ///        creates link to the corresponding file
 ///        if such file exists
 ///
@@ -1127,7 +1127,7 @@ Bool_t TDocParser::HandleDirective(TString& line, Ssiz_t& pos, TString& word,
          if (fParseContext.size()>1)
             fParseContext.pop_back();
          if (isInCxxComment && !InContext(kComment)) {
-            fParseContext.push_back(kComment | kCXXComment);
+            fParseContext.push_back((int)kComment | (int)kCXXComment);
             fDocOutput->DecorateEntityBegin(line, pos, kComment);
          }
       }
