@@ -110,7 +110,7 @@ static void AppendAnyDeclLocation(const CompilerInstance* compiler,
     const SourceManager &sourceManager = compiler->getSourceManager();
     if (loc.isValid() && sourceManager.isLoadedSourceLocation(loc)) {
       // No line numbers as they would touich disk.
-      baseName = llvm::sys::path::filename(sourceManager.getFilename(loc));
+      baseName = llvm::sys::path::filename(sourceManager.getFilename(loc)).str();
       lineNo = -1;
     } else {
       PresumedLoc ploc(sourceManager.getPresumedLoc(loc));
@@ -196,7 +196,9 @@ void AppendClassName(const CXXRecordDecl* classDecl, std::string& name)
   assert(classDecl != 0 && "AppendClassName, 'classDecl' parameter is null");
 
   const LangOptions langOpts;
-  const PrintingPolicy printingPolicy(langOpts);
+  PrintingPolicy printingPolicy(langOpts);
+  // Print the default template arguments when asking for a class name.
+  printingPolicy.SuppressDefaultTemplateArgs = false;
   std::string tmp;
   //Name for diagnostic will include template arguments if any.
   llvm::raw_string_ostream stream(tmp);
@@ -277,6 +279,7 @@ void AppendMemberFunctionSignature(const Decl* methodDecl, std::string& name)
   const LangOptions langOpts;
   PrintingPolicy printingPolicy(langOpts);
   printingPolicy.TerseOutput = true;//Do not print the body of an inlined function.
+  printingPolicy.SuppressDefaultTemplateArgs = false;
   printingPolicy.SuppressSpecifiers = false; //Show 'static', 'inline', etc.
 
   methodDecl->print(out, printingPolicy, 0, false);
@@ -523,7 +526,7 @@ void ClassPrinter::DisplayAllClasses()const
   const TranslationUnitDecl* const tuDecl = compiler->getASTContext().getTranslationUnitDecl();
   assert(tuDecl != 0 && "DisplayAllClasses, translation unit is empty");
 
-  fOut.Print("List of classes");
+  fOut.Print("List of classes\n");
   // Could trigger deserialization of decls.
   Interpreter::PushTransactionRAII RAII(const_cast<Interpreter*>(fInterpreter));
   for (decl_iterator decl = tuDecl->decls_begin(); decl != tuDecl->decls_end(); ++decl)
@@ -766,15 +769,15 @@ void ClassPrinter::DisplayClassDecl(const CXXRecordDecl* classDecl)const
     fOut.Print("\n");
 
     if (classDecl->bases_begin() != classDecl->bases_end())
-      fOut.Print("Base classes: --------------------------------------------------------\n");
+      fOut.Print("Base classes: -------------------------------------------------------------\n");
 
     DisplayBasesAsTree(classDecl, 0);
     //now list all members.40963410
 
-    fOut.Print("List of member variables --------------------------------------------------\n");
+    fOut.Print("List of member variables: -------------------------------------------------\n");
     DisplayDataMembers(classDecl, 0);
 
-    fOut.Print("List of member functions :---------------------------------------------------\n");
+    fOut.Print("List of member functions: -------------------------------------------------\n");
     //CINT has a format like %-15s blah-blah.
     fOut.Print("filename     line:size busy function type and name\n");
     DisplayMemberFunctions(classDecl);
@@ -1416,7 +1419,7 @@ void TypedefPrinter::DisplayTypedefs()const
   const TranslationUnitDecl* const tuDecl = compiler->getASTContext().getTranslationUnitDecl();
   assert(tuDecl != 0 && "DisplayTypedefs, translation unit is empty");
 
-  fOut.Print("List of typedefs");
+  fOut.Print("List of typedefs\n");
   ProcessNestedDeclarations(tuDecl);
 }
 
@@ -1505,6 +1508,7 @@ void TypedefPrinter::DisplayTypedefDecl(TypedefNameDecl* typedefDecl)const
     llvm::raw_string_ostream out(textLine);
     typedefDecl->getUnderlyingType().
        getDesugaredType(typedefDecl->getASTContext()).print(out,printingPolicy);
+    out << ' ';
     //Name for diagnostic will include template arguments if any.
     typedefDecl->getNameForDiagnostic(out,
                                       printingPolicy,/*qualified=*/true);
@@ -1516,34 +1520,19 @@ void TypedefPrinter::DisplayTypedefDecl(TypedefNameDecl* typedefDecl)const
 
 }//unnamed namespace
 
-//______________________________________________________________________________
-void DisplayClasses(llvm::raw_ostream& stream, const Interpreter* interpreter,
-                    bool verbose)
-{
-  assert(interpreter != 0 && "DisplayClasses, 'interpreter' parameter is null");
-
-  ClassPrinter printer(stream, interpreter);
-  printer.SetVerbose(verbose);
-  printer.DisplayAllClasses();
-}
-
 //______________________________________________________________________________
 void DisplayClass(llvm::raw_ostream& stream, const Interpreter* interpreter,
                   const char* className, bool verbose)
 {
   assert(interpreter != 0 && "DisplayClass, 'interpreter' parameter is null");
-  assert(className != 0 && "DisplayClass, 'className' parameter is null");
-
-  while (std::isspace(*className))
-    ++className;
 
   ClassPrinter printer(stream, interpreter);
-
-  if (*className) {
-    printer.SetVerbose(verbose);
+  printer.SetVerbose(verbose);
+  if (className && *className) {
+    while (std::isspace(*className))
+      ++className;
     printer.DisplayClass(className);
   } else {
-    printer.SetVerbose(true);//?
     printer.DisplayAllClasses();
   }
 }
