@@ -1,5 +1,5 @@
 // @(#)root/roostats:$Id:  cranmer $
-// Author: Kyle Cranmer, George Lewis 
+// Author: Kyle Cranmer, George Lewis
 /*************************************************************************
  * Copyright (C) 1995-2008, Rene Brun and Fons Rademakers.               *
  * All rights reserved.                                                  *
@@ -46,10 +46,10 @@
 #include "RooRealVar.h"
 #include "RooArgList.h"
 #include "RooWorkspace.h"
-#include "RunContext.h"
 
 #include "TH1.h"
 
+#include <array>
 #include <sstream>
 #include <stdexcept>
 #include <iostream>
@@ -60,7 +60,7 @@ ClassImp(ParamHistFunc);
 ////////////////////////////////////////////////////////////////////////////////
 
 ParamHistFunc::ParamHistFunc()
-  : _normIntMgr(this), _numBins(0)
+  : _normIntMgr(this)
 {
   _dataSet.removeSelfFromDir(); // files must not delete _dataSet.
 }
@@ -71,32 +71,31 @@ ParamHistFunc::ParamHistFunc()
 /// This class contains N RooAbsReals's, one for each
 /// bin from the given RooRealVar.
 ///
-/// The value of the function in the ith bin is 
+/// The value of the function in the ith bin is
 /// given by:
 ///
 /// F(i) = gamma_i * nominal(i)
 ///
 /// Where the nominal values are simply fixed
 /// numbers (default = 1.0 for all i)
-ParamHistFunc::ParamHistFunc(const char* name, const char* title, 
-			     const RooArgList& vars, const RooArgList& paramSet) :
+ParamHistFunc::ParamHistFunc(const char* name, const char* title,
+              const RooArgList& vars, const RooArgList& paramSet) :
   RooAbsReal(name, title),
   _normIntMgr(this),
   _dataVars("!dataVars","data Vars",       this),
   _paramSet("!paramSet","bin parameters",  this),
-  _numBins(0),
   _dataSet( (std::string(name)+"_dataSet").c_str(), "", vars)
 {
 
   // Create the dataset that stores the binning info:
-  
+
   //  _dataSet = RooDataSet("
 
   _dataSet.removeSelfFromDir(); // files must not delete _dataSet.
 
   // Set the binning
   // //_binning = var.getBinning().clone() ;
-  
+
   // Create the set of parameters
   // controlling the height of each bin
 
@@ -114,7 +113,7 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
 /// This class allows to multiply bin contents of histograms
 /// with the values of a set of RooAbsReal.
 ///
-/// The value of the function in the ith bin is 
+/// The value of the function in the ith bin is
 /// given by:
 /// \f[
 ///   F(i) = \gamma_{i} * \mathrm{nominal}(i)
@@ -122,15 +121,14 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
 ///
 /// Where the nominal values are taken from the histogram,
 /// and the \f$ \gamma_{i} \f$ can be set from the outside.
-ParamHistFunc::ParamHistFunc(const char* name, const char* title, 
-			     const RooArgList& vars, const RooArgList& paramSet,
-			     const TH1* Hist ) :
+ParamHistFunc::ParamHistFunc(const char* name, const char* title,
+              const RooArgList& vars, const RooArgList& paramSet,
+              const TH1* Hist ) :
   RooAbsReal(name, title),
   _normIntMgr(this),
   //  _dataVar("!dataVar","data Var", this, (RooRealVar&) var),
   _dataVars("!dataVars","data Vars",       this),
   _paramSet("!paramSet","bin parameters",  this),
-  _numBins(0),
   _dataSet( (std::string(name)+"_dataSet").c_str(), "", vars, Hist)
 {
 
@@ -149,7 +147,7 @@ Int_t ParamHistFunc::GetNumBins( const RooArgSet& vars ) {
 
   // A helper method to get the number of bins
 
-  if( vars.getSize() == 0 ) return 0;
+  if( vars.empty() ) return 0;
 
   Int_t numBins = 1;
 
@@ -157,7 +155,7 @@ Int_t ParamHistFunc::GetNumBins( const RooArgSet& vars ) {
     if (!dynamic_cast<RooRealVar*>(comp)) {
       auto errorMsg = std::string("ParamHistFunc::GetNumBins") + vars.GetName() + ") ERROR: component "
                       + comp->GetName() + " in vars list is not of type RooRealVar";
-      oocoutE(static_cast<TObject*>(nullptr), InputArguments) <<  errorMsg << std::endl;
+      oocoutE(nullptr, InputArguments) <<  errorMsg << std::endl;
       throw std::runtime_error(errorMsg);
     }
     auto var = static_cast<RooRealVar*>(comp);
@@ -173,7 +171,7 @@ Int_t ParamHistFunc::GetNumBins( const RooArgSet& vars ) {
 ////////////////////////////////////////////////////////////////////////////////
 
 ParamHistFunc::ParamHistFunc(const ParamHistFunc& other, const char* name) :
-  RooAbsReal(other, name), 
+  RooAbsReal(other, name),
   _normIntMgr(other._normIntMgr, this),
   _dataVars("!dataVars", this, other._dataVars ),
   _paramSet("!paramSet", this, other._paramSet),
@@ -187,14 +185,6 @@ ParamHistFunc::ParamHistFunc(const ParamHistFunc& other, const char* name) :
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-
-ParamHistFunc::~ParamHistFunc() 
-{
-  ;
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Get the parameter associated with the index.
 /// The index follows RooDataHist indexing conventions.
@@ -230,7 +220,7 @@ RooAbsReal& ParamHistFunc::getParameter() const {
 }
 
 
-void ParamHistFunc::setParamConst( Int_t index, Bool_t varConst ) {
+void ParamHistFunc::setParamConst( Int_t index, bool varConst ) {
   RooAbsReal& var = getParameter( index );
   var.setAttribute( "Constant", varConst );
 }
@@ -250,17 +240,17 @@ void ParamHistFunc::setShape( TH1* shape ) {
 
   if( num_hist_bins != numBins() ) {
     std::cout << "Error - ParamHistFunc: cannot set Shape of ParamHistFunc: " << GetName()
-	      << " using histogram: " << shape->GetName()
-	      << ". Bins don't match" << std::endl;
+         << " using histogram: " << shape->GetName()
+         << ". Bins don't match" << std::endl;
     throw std::runtime_error("setShape");
   }
 
 
   Int_t TH1BinNumber = 0;
   for( Int_t i = 0; i < numBins(); ++i) {
-    
+
     TH1BinNumber++;
-    
+
     while( shape->IsBinUnderflow(TH1BinNumber) || shape->IsBinOverflow(TH1BinNumber) ){
       TH1BinNumber++;
     }
@@ -281,17 +271,17 @@ void ParamHistFunc::setShape( TH1* shape ) {
 /// Create the list of RooRealVar
 /// parameters which represent the
 /// height of the histogram bins.
-/// The list 'vars' represents the 
+/// The list 'vars' represents the
 /// observables (corresponding to histogram bins)
-/// that these newly created parameters will 
+/// that these newly created parameters will
 /// be mapped to. (ie, we create one parameter
 /// per observable in vars and per bin in each observable)
 
 /// Store them in a list using:
 /// _paramSet.add( createParamSet() );
 /// This list is stored in the "TH1" index order
-RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Prefix, 
-					 const RooArgList& vars) {
+RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Prefix,
+                const RooArgList& vars) {
 
 
   // Get the number of bins
@@ -304,8 +294,8 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
 
   if( numVars == 0 ) {
     std::cout << "Warning - ParamHistFunc::createParamSet() :"
-	 << " No Variables provided.  Not making constraint terms." 
-	 << std::endl;
+    << " No Variables provided.  Not making constraint terms."
+    << std::endl;
     return paramSet;
   }
 
@@ -318,13 +308,13 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
       VarNameStream << Prefix << "_bin_" << i;
       std::string VarName = VarNameStream.str();
 
-      RooRealVar gamma( VarName.c_str(), VarName.c_str(), 1.0 ); 
+      RooRealVar gamma( VarName.c_str(), VarName.c_str(), 1.0 );
       // "Hard-Code" a minimum of 0.0
       gamma.setMin( 0.0 );
       gamma.setConstant( false );
 
       w.import( gamma, RooFit::RecycleConflictNodes() );
-      RooRealVar* gamma_wspace = (RooRealVar*) w.var( VarName.c_str() );
+      RooRealVar* gamma_wspace = (RooRealVar*) w.var( VarName );
 
       paramSet.add( *gamma_wspace );
 
@@ -358,7 +348,7 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
         gamma.setConstant( false );
 
         w.import( gamma, RooFit::RecycleConflictNodes() );
-        RooRealVar* gamma_wspace = (RooRealVar*) w.var( VarName.c_str() );
+        RooRealVar* gamma_wspace = (RooRealVar*) w.var( VarName );
 
         paramSet.add( *gamma_wspace );
 
@@ -395,7 +385,7 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
           gamma.setConstant( false );
 
           w.import( gamma, RooFit::RecycleConflictNodes() );
-          RooRealVar* gamma_wspace = (RooRealVar*) w.var( VarName.c_str() );
+          RooRealVar* gamma_wspace = (RooRealVar*) w.var( VarName );
 
           paramSet.add( *gamma_wspace );
 
@@ -408,7 +398,7 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
     std::cout << " Error: ParamHistFunc doesn't support dimensions > 3D " <<  std::endl;
   }
 
-  return paramSet;  
+  return paramSet;
 
 }
 
@@ -429,9 +419,9 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
 /// _paramSet.add( createParamSet() );
 /// ```
 /// This list is stored in the "TH1" index order.
-RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Prefix, 
-					 const RooArgList& vars, 
-					 Double_t gamma_min, Double_t gamma_max) {
+RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Prefix,
+                const RooArgList& vars,
+                double gamma_min, double gamma_max) {
 
 
 
@@ -454,8 +444,8 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
 /// parameters which represent the
 /// height of the histogram bins.
 /// Store them in a list
-RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBins, 
-					 Double_t gamma_min, Double_t gamma_max) {
+RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBins,
+                double gamma_min, double gamma_max) {
 
   // Get the number of bins
   // in the nominal histogram
@@ -471,7 +461,7 @@ RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBin
 
   }
 
-  Double_t gamma_nominal = 1.0;
+  double gamma_nominal = 1.0;
 
   if( gamma_nominal < gamma_min ) {
     gamma_nominal = gamma_min;
@@ -488,8 +478,8 @@ RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBin
     VarNameStream << Prefix << "_bin_" << i;
     std::string VarName = VarNameStream.str();
 
-    RooRealVar* gamma = new RooRealVar( VarName.c_str(), VarName.c_str(), 
-					gamma_nominal, gamma_min, gamma_max );
+    RooRealVar* gamma = new RooRealVar( VarName.c_str(), VarName.c_str(),
+               gamma_nominal, gamma_min, gamma_max );
     gamma->setConstant( false );
     paramSet.add( *gamma );
 
@@ -505,7 +495,7 @@ ParamHistFunc::NumBins ParamHistFunc::getNumBinsPerDim(RooArgSet const& vars) {
 
   if (numVars > 3 || numVars < 1) {
     std::cout << "ParamHistFunc() - Only works for 1-3 variables (1d-3d)" << std::endl;
-    throw -1;  
+    throw -1;
   }
 
   int numBinsX = numVars >= 1 ? static_cast<RooRealVar const&>(*vars[0]).numBins() : 1;
@@ -528,9 +518,9 @@ Int_t ParamHistFunc::getCurrentBin() const {
 ////////////////////////////////////////////////////////////////////////////////
 /// return 0 for success
 /// return 1 for failure
-/// Check that the elements 
+/// Check that the elements
 /// are actually RooRealVar's
-/// If so, add them to the 
+/// If so, add them to the
 /// list of vars
 Int_t ParamHistFunc::addVarSet( const RooArgList& vars ) {
   for(auto const& comp : vars) {
@@ -559,11 +549,11 @@ Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
   Int_t numElements = params.getSize();
 
   if( numVarBins != numElements ) {
-    std::cout << "ParamHistFunc::addParamSet - ERROR - " 
-	      << "Supplied list of parameters " << params.GetName()
-	      << " has " << numElements << " elements but the ParamHistFunc"
-	      << GetName() << " has " << numVarBins << " bins."
-	      << std::endl;
+    std::cout << "ParamHistFunc::addParamSet - ERROR - "
+         << "Supplied list of parameters " << params.GetName()
+         << " has " << numElements << " elements but the ParamHistFunc"
+         << GetName() << " has " << numVarBins << " bins."
+         << std::endl;
     return 1;
 
   }
@@ -583,7 +573,7 @@ Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
 
     _paramSet.add( *comp );
   }
-  
+
   return 0;
 }
 
@@ -591,7 +581,7 @@ Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
 ////////////////////////////////////////////////////////////////////////////////
 /// Find the bin corresponding to the current value of the observable, and evaluate
 /// the associated parameter.
-Double_t ParamHistFunc::evaluate() const 
+double ParamHistFunc::evaluate() const
 {
   return getParameter().getVal();
 }
@@ -600,50 +590,49 @@ Double_t ParamHistFunc::evaluate() const
 ////////////////////////////////////////////////////////////////////////////////
 /// Find all bins corresponding to the values of the observables in `evalData`, and evaluate
 /// the associated parameters.
-/// \param[in/out] evalData Input/output data for evaluating the ParamHistFunc.
+/// \param[in,out] evalData Input/output data for evaluating the ParamHistFunc.
 /// \param[in] normSet Normalisation set passed on to objects that are serving values to us.
 void ParamHistFunc::computeBatch(cudaStream_t*, double* output, size_t size, RooFit::Detail::DataMap const& dataMap) const {
-  std::vector<double> oldValues;
-  std::vector<RooSpan<const double>> data;
-  oldValues.reserve(_dataVars.size());
-  data.reserve(_dataVars.size());
 
-  // Retrieve data for all variables
-  for (auto arg : _dataVars) {
-    const auto* var = static_cast<RooRealVar*>(arg);
-    oldValues.push_back(var->getVal());
-    data.push_back(dataMap.at(var));
+  auto const& n = _numBinsPerDim;
+  // check if _numBins needs to be filled
+  if(n.x == 0) {
+    _numBinsPerDim = getNumBinsPerDim(_dataVars);
   }
 
-  // Run computation for each entry in the dataset
+  // Different from the evaluate() funnction that first retrieves the indices
+  // corresponding to the RooDataHist and then transforms them, we can use the
+  // right bin multiplicators to begin with.
+  std::array<int, 3> idxMult{{1, n.x, n.xy}};
+
+  // As a working buffer for the bin indices, we use the tail of the output
+  // buffer. We can't use the same starting pointer, otherwise we would
+  // overwrite the later bin indices as we fill the output.
+  auto indexBuffer = reinterpret_cast<int*>(output + size) - size;
+  std::fill(indexBuffer, indexBuffer + size, 0); // output buffer for bin indices needs to be zero-initialized
+
+  // Use the vectorized RooAbsBinning::binNumbers() to update the total bin
+  // index for each dimension, using the `coef` parameter to multiply with the
+  // right index multiplication factor for each dimension.
+  for (std::size_t iVar = 0; iVar < _dataVars.size(); ++iVar) {
+    _dataSet.getBinnings()[iVar]->binNumbers(dataMap.at(&_dataVars[iVar]).data(), indexBuffer, size, idxMult[iVar]);
+  }
+
+  // Finally, look up the parameters and get their values to fill the output buffer
   for (std::size_t i = 0; i < size; ++i) {
-    for (unsigned int j = 0; j < _dataVars.size(); ++j) {
-      assert(i < data[j].size());
-      auto& var = static_cast<RooRealVar&>(_dataVars[j]);
-      var.setCachedValue(data[j][i], /*notifyClients=*/false);
-    }
-
-    const auto index = _dataSet.getIndex(_dataVars, /*fast=*/true);
-    const RooAbsReal& param = getParameter(index);
-    output[i] = param.getVal();
-  }
-
-  // Restore old values
-  for (unsigned int j = 0; j < _dataVars.size(); ++j) {
-    auto& var = static_cast<RooRealVar&>(_dataVars[j]);
-    var.setCachedValue(oldValues[j], /*notifyClients=*/false);
+    output[i] = static_cast<RooAbsReal const&>(_paramSet[indexBuffer[i]]).getVal();
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Advertise that all integrals can be handled internally.
 
-Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVars, 
-					     const RooArgSet* normSet, 
-					     const char* /*rangeName*/) const 
+Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVars,
+                    const RooArgSet* normSet,
+                    const char* /*rangeName*/) const
 {
   // Handle trivial no-integration scenario
-  if (allVars.getSize()==0) return 0 ;
+  if (allVars.empty()) return 0 ;
   if (_forceNumInt) return 0 ;
 
 
@@ -656,14 +645,14 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
   if (cache) {
     return _normIntMgr.lastIndex()+1 ;
   }
-  
+
   // Create new cache element
   cache = new CacheElem ;
 
   // Store cache element
   Int_t code = _normIntMgr.setObj(normSet,&analVars,(RooAbsCacheElement*)cache,0) ;
 
-  return code+1 ; 
+  return code+1 ;
 
 }
 
@@ -672,23 +661,22 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
 /// Implement analytical integrations by doing appropriate weighting from  component integrals
 /// functions to integrators of components
 
-Double_t ParamHistFunc::analyticalIntegralWN(Int_t /*code*/, const RooArgSet* /*normSet2*/,
-					     const char* /*rangeName*/) const 
+double ParamHistFunc::analyticalIntegralWN(Int_t /*code*/, const RooArgSet* /*normSet2*/,
+                    const char* /*rangeName*/) const
 {
-  Double_t value(0) ;
+  double value(0) ;
 
-  // Simply loop over bins, 
+  // Simply loop over bins,
   // get the height, and
   // multiply by the bind width
   auto binVolumes = _dataSet.binVolumes(0, _dataSet.numEntries());
 
   for (unsigned int i=0; i < _paramSet.size(); ++i) {
     const auto& param = static_cast<const RooAbsReal&>(_paramSet[i]);
-    assert(static_cast<Int_t>(i) == _dataSet.getIndex(param)); // We assume that each parameter i belongs to bin i
 
     // Get the gamma's value
     const double paramVal = param.getVal();
-    
+
     // Finally, get the subtotal
     value += paramVal * binVolumes[i];
   }
@@ -704,24 +692,24 @@ Double_t ParamHistFunc::analyticalIntegralWN(Int_t /*code*/, const RooArgSet* /*
 /// as the recursive division strategy of RooCurve cannot deal efficiently
 /// with the vertical lines that occur in a non-interpolated histogram
 
-std::list<Double_t>* ParamHistFunc::plotSamplingHint(RooAbsRealLValue& obs, Double_t xlo, 
-						Double_t xhi) const
+std::list<double>* ParamHistFunc::plotSamplingHint(RooAbsRealLValue& obs, double xlo,
+                  double xhi) const
 {
   // copied and edited from RooHistFunc
   RooAbsLValue* lvarg = &obs;
 
   // Retrieve position of all bin boundaries
-  const RooAbsBinning* binning = lvarg->getBinningPtr(0) ;
-  Double_t* boundaries = binning->array() ;
+  const RooAbsBinning* binning = lvarg->getBinningPtr(nullptr);
+  double* boundaries = binning->array() ;
 
-  std::list<Double_t>* hint = new std::list<Double_t> ;
+  std::list<double>* hint = new std::list<double> ;
 
   // Widen range slighty
   xlo = xlo - 0.01*(xhi-xlo) ;
   xhi = xhi + 0.01*(xhi-xlo) ;
 
-  Double_t delta = (xhi-xlo)*1e-8 ;
- 
+  double delta = (xhi-xlo)*1e-8 ;
+
   // Construct array with pairs of points positioned epsilon to the left and
   // right of the bin boundaries
   for (Int_t i=0 ; i<binning->numBoundaries() ; i++) {
@@ -739,17 +727,17 @@ std::list<Double_t>* ParamHistFunc::plotSamplingHint(RooAbsRealLValue& obs, Doub
 /// as the recursive division strategy of RooCurve cannot deal efficiently
 /// with the vertical lines that occur in a non-interpolated histogram
 
-std::list<Double_t>* ParamHistFunc::binBoundaries(RooAbsRealLValue& obs, Double_t xlo, 
-						  Double_t xhi) const 
+std::list<double>* ParamHistFunc::binBoundaries(RooAbsRealLValue& obs, double xlo,
+                    double xhi) const
 {
   // copied and edited from RooHistFunc
   RooAbsLValue* lvarg = &obs;
 
   // Retrieve position of all bin boundaries
-  const RooAbsBinning* binning = lvarg->getBinningPtr(0) ;
-  Double_t* boundaries = binning->array() ;
+  const RooAbsBinning* binning = lvarg->getBinningPtr(nullptr);
+  double* boundaries = binning->array() ;
 
-  std::list<Double_t>* hint = new std::list<Double_t> ;
+  std::list<double>* hint = new std::list<double> ;
 
   // Construct array with pairs of points positioned epsilon to the left and
   // right of the bin boundaries
