@@ -22,14 +22,11 @@ Two-dimensional kernel estimation PDF.
 <b>This function has been superseded by the more general RooNDKeysPdf.</b>
 */
 
-#include "RooFit.h"
-
 #include "Roo2DKeysPdf.h"
 #include "RooRealVar.h"
 #include "TTree.h"
 #include "TH2.h"
 #include "TFile.h"
-#include "TBranch.h"
 #include "TMath.h"
 
 //#include <math.h>
@@ -50,7 +47,7 @@ ClassImp(Roo2DKeysPdf);
 /// \param[in] widthScaleFactor
 
 Roo2DKeysPdf::Roo2DKeysPdf(const char *name, const char *title,
-                       RooAbsReal& xx, RooAbsReal & yy, RooDataSet& data,  TString options, Double_t widthScaleFactor):
+                       RooAbsReal& xx, RooAbsReal & yy, RooDataSet& data,  TString options, double widthScaleFactor):
   RooAbsPdf(name,title),
   x("x", "x dimension",this, xx),
   y("y", "y dimension",this, yy)
@@ -97,10 +94,10 @@ Roo2DKeysPdf::Roo2DKeysPdf(const Roo2DKeysPdf & other, const char* name) :
   _xoffset   = other._xoffset;
   _yoffset   = other._yoffset;
 
-  _x  = new Double_t[_nEvents];
-  _y  = new Double_t[_nEvents];
-  _hx = new Double_t[_nEvents];
-  _hy = new Double_t[_nEvents];
+  _x  = new double[_nEvents];
+  _y  = new double[_nEvents];
+  _hx = new double[_nEvents];
+  _hy = new double[_nEvents];
 
   //copy the data and bandwidths
   for(Int_t iEvt = 0; iEvt< _nEvents; iEvt++)
@@ -154,17 +151,17 @@ Int_t Roo2DKeysPdf::loadDataSet(RooDataSet& data, TString options)
   _loy       = y.min();
   _hiy       = y.max();
 
-  _x  = new Double_t[_nEvents];
-  _y  = new Double_t[_nEvents];
-  _hx = new Double_t[_nEvents];
-  _hy = new Double_t[_nEvents];
+  _x  = new double[_nEvents];
+  _y  = new double[_nEvents];
+  _hx = new double[_nEvents];
+  _hy = new double[_nEvents];
 
-  Double_t x0 = 0.0;
-  Double_t x1 = 0.0;
-  Double_t x_2 = 0.0;
-  Double_t y0 = 0.0;
-  Double_t y1 = 0.0;
-  Double_t y_2 = 0.0;
+  double x0 = 0.0;
+  double x1 = 0.0;
+  double x_2 = 0.0;
+  double y0 = 0.0;
+  double y1 = 0.0;
+  double y_2 = 0.0;
 
   //check that the data contain the variable we are interested in
   Int_t bad = 0;
@@ -217,7 +214,7 @@ Int_t Roo2DKeysPdf::loadDataSet(RooDataSet& data, TString options)
   _yMean  = y1/y0;
   _ySigma = sqrt(y_2/_nEvents-_yMean*_yMean);
 
-  _n = Double_t(1)/(_2pi*_nEvents*_xSigma*_ySigma);
+  _n = double(1)/(_2pi*_nEvents*_xSigma*_ySigma);
 
   //calculate the PDF
   return calculateBandWidth(_BandWidthType);
@@ -281,11 +278,11 @@ Int_t Roo2DKeysPdf::calculateBandWidth(Int_t kernel)
     _BandWidthType = kernel;
   }
 
-  Double_t h = 0.0;
+  double h = 0.0;
 
-  Double_t sigSum       = _xSigma*_xSigma + _ySigma*_ySigma;
-  Double_t sqrtSum      = sqrt( sigSum );
-  Double_t sigProd      = _ySigma*_xSigma;
+  double sigSum       = _xSigma*_xSigma + _ySigma*_ySigma;
+  double sqrtSum      = sqrt( sigSum );
+  double sigProd      = _ySigma*_xSigma;
   if(sigProd != 0.0)  h = _n16*sqrt( sigSum/sigProd );
   if(sqrtSum == 0)
   {
@@ -293,10 +290,10 @@ Int_t Roo2DKeysPdf::calculateBandWidth(Int_t kernel)
     return 1;
   }
 
-  Double_t hXSigma = h * _xSigma;
-  Double_t hYSigma = h * _ySigma;
-  Double_t xhmin   = hXSigma * sqrt(2.)/10;  //smallest anticipated bandwidth
-  Double_t yhmin   = hYSigma * sqrt(2.)/10;
+  double hXSigma = h * _xSigma;
+  double hYSigma = h * _ySigma;
+  double xhmin   = hXSigma * sqrt(2.)/10;  //smallest anticipated bandwidth
+  double yhmin   = hYSigma * sqrt(2.)/10;
 
   //////////////////////////////////////
   //calculate bandwidths from the data//
@@ -305,8 +302,8 @@ Int_t Roo2DKeysPdf::calculateBandWidth(Int_t kernel)
   {
     cout << "Roo2DKeysPdf::calculateBandWidth Using a normal bandwidth (same for a given dimension) based on"<<endl;
     cout << "                                 h_j = n^{-1/6}*sigma_j for the j^th dimension and n events * "<<_widthScaleFactor<<endl;
-    Double_t hxGaussian = _n16 * _xSigma * _widthScaleFactor;
-    Double_t hyGaussian = _n16 * _ySigma * _widthScaleFactor;
+    double hxGaussian = _n16 * _xSigma * _widthScaleFactor;
+    double hyGaussian = _n16 * _ySigma * _widthScaleFactor;
     for(Int_t j=0;j<_nEvents;++j)
     {
       _hx[j] = hxGaussian;
@@ -319,11 +316,11 @@ Int_t Roo2DKeysPdf::calculateBandWidth(Int_t kernel)
   {
     cout << "Roo2DKeysPdf::calculateBandWidth Using an adaptive bandwidth (in general different for all events) [default]"<<endl;
     cout << "                                 scaled by a factor of "<<_widthScaleFactor<<endl;
-    Double_t xnorm   = h * TMath::Power(_xSigma/sqrtSum, 1.5) * _widthScaleFactor;
-    Double_t ynorm   = h * TMath::Power(_ySigma/sqrtSum, 1.5) * _widthScaleFactor;
+    double xnorm   = h * TMath::Power(_xSigma/sqrtSum, 1.5) * _widthScaleFactor;
+    double ynorm   = h * TMath::Power(_ySigma/sqrtSum, 1.5) * _widthScaleFactor;
     for(Int_t j=0;j<_nEvents;++j)
     {
-      Double_t f_ti =  TMath::Power( g(_x[j], _x, hXSigma, _y[j], _y, hYSigma), -0.25 ) ;
+      double f_ti =  TMath::Power( g(_x[j], _x, hXSigma, _y[j], _y, hYSigma), -0.25 ) ;
       _hx[j] = xnorm * f_ti;
       _hy[j] = ynorm * f_ti;
       if(_hx[j]<xhmin) _hx[j] = xhmin;
@@ -342,7 +339,7 @@ Int_t Roo2DKeysPdf::calculateBandWidth(Int_t kernel)
 /// the grid and extrapolation approximation in the kernel estimation method
 /// implementation.
 
-Double_t Roo2DKeysPdf::evaluate() const
+double Roo2DKeysPdf::evaluate() const
 {
   if(_vverbosedebug) { cout << "Roo2DKeysPdf::evaluate()" << endl; }
   return evaluateFull(x,y);
@@ -358,13 +355,13 @@ Double_t Roo2DKeysPdf::evaluate() const
 /// \param[in] thisX
 /// \param[in] thisY
 
-Double_t Roo2DKeysPdf::evaluateFull(Double_t thisX, Double_t thisY) const
+double Roo2DKeysPdf::evaluateFull(double thisX, double thisY) const
 {
   if( _vverbosedebug ) { cout << "Roo2DKeysPdf::evaluateFull()" << endl; }
 
-  Double_t f=0.0;
+  double f=0.0;
 
-  Double_t rx2, ry2, zx, zy;
+  double rx2, ry2, zx, zy;
   if( _MirrorAtBoundary )
   {
     for (Int_t j = 0; j < _nEvents; ++j)
@@ -377,9 +374,9 @@ Double_t Roo2DKeysPdf::evaluateFull(Double_t thisX, Double_t thisY) const
       if(_hy[j] != 0.0) zy = exp(-0.5*ry2*ry2)/_hy[j];
 
       zx += highBoundaryCorrection(thisX, _hx[j], x.max(), _x[j])
- 	 +   lowBoundaryCorrection(thisX, _hx[j], x.min(), _x[j]);
+    +   lowBoundaryCorrection(thisX, _hx[j], x.min(), _x[j]);
       zy += highBoundaryCorrection(thisY, _hy[j], y.max(), _y[j])
- 	 +   lowBoundaryCorrection(thisY, _hy[j], y.min(), _y[j]);
+    +   lowBoundaryCorrection(thisY, _hy[j], y.min(), _y[j]);
       f += zy * zx;
       //      f += _n * zy * zx; // ooops this is a normalisation factor :(
     }
@@ -411,24 +408,24 @@ Double_t Roo2DKeysPdf::evaluateFull(Double_t thisX, Double_t thisY) const
 /// \param[in] high
 /// \param[in] tVar
 
-Double_t Roo2DKeysPdf::highBoundaryCorrection(Double_t thisVar, Double_t thisH, Double_t high, Double_t tVar) const
+double Roo2DKeysPdf::highBoundaryCorrection(double thisVar, double thisH, double high, double tVar) const
 {
   if(_vverbosedebug) { cout << "Roo2DKeysPdf::highBoundaryCorrection" << endl; }
 
   if(thisH == 0.0) return 0.0;
-  Double_t correction = (thisVar + tVar - 2.0* high )/thisH;
+  double correction = (thisVar + tVar - 2.0* high )/thisH;
   return exp(-0.5*correction*correction)/thisH;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Double_t Roo2DKeysPdf::lowBoundaryCorrection(Double_t thisVar, Double_t thisH, Double_t low, Double_t tVar) const
+double Roo2DKeysPdf::lowBoundaryCorrection(double thisVar, double thisH, double low, double tVar) const
 {
   if(_vverbosedebug) { cout << "Roo2DKeysPdf::lowBoundaryCorrection" << endl; }
 
   if(thisH == 0.0) return 0.0;
-  Double_t correction = (thisVar + tVar - 2.0* low )/thisH;
+  double correction = (thisVar + tVar - 2.0* low )/thisH;
   return exp(-0.5*correction*correction)/thisH;
 }
 
@@ -443,19 +440,19 @@ Double_t Roo2DKeysPdf::lowBoundaryCorrection(Double_t thisVar, Double_t thisH, D
 /// \param[in] _var2
 /// \param[in] sigma2
 
-Double_t Roo2DKeysPdf::g(Double_t varMean1, Double_t * _var1, Double_t sigma1, Double_t varMean2, Double_t * _var2, Double_t sigma2) const
+double Roo2DKeysPdf::g(double varMean1, double * _var1, double sigma1, double varMean2, double * _var2, double sigma2) const
 {
   if((_nEvents == 0.0) || (sigma1 == 0.0) || (sigma2 == 0)) return 0.0;
 
-  Double_t c1 = -1.0/(2.0*sigma1*sigma1);
-  Double_t c2 = -1.0/(2.0*sigma2*sigma2);
-  Double_t d  = 4.0*c1*c2  /(_sqrt2pi*_nEvents);
-  Double_t z  = 0.0;
+  double c1 = -1.0/(2.0*sigma1*sigma1);
+  double c2 = -1.0/(2.0*sigma2*sigma2);
+  double d  = 4.0*c1*c2  /(_sqrt2pi*_nEvents);
+  double z  = 0.0;
 
   for (Int_t i = 0; i < _nEvents; ++i)
   {
-    Double_t r1 =  _var1[i] - varMean1;
-    Double_t r2 =  _var2[i] - varMean2;
+    double r1 =  _var1[i] - varMean1;
+    double r2 =  _var2[i] - varMean2;
     z          += exp( c1 * r1*r1 ) * exp( c2 * r2*r2 );
   }
   z = z*d;
@@ -476,7 +473,7 @@ Int_t Roo2DKeysPdf::getBandWidthType() const
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Double_t Roo2DKeysPdf::getMean(const char * axis) const
+double Roo2DKeysPdf::getMean(const char * axis) const
 {
   if(!strcmp(axis,x.GetName()) || !strcmp(axis,"x") || !strcmp(axis,"X"))      return _xMean;
   else if(!strcmp(axis,y.GetName()) || !strcmp(axis,"y") || !strcmp(axis,"Y")) return _yMean;
@@ -490,7 +487,7 @@ Double_t Roo2DKeysPdf::getMean(const char * axis) const
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Double_t Roo2DKeysPdf::getSigma(const char * axis) const
+double Roo2DKeysPdf::getSigma(const char * axis) const
 {
   if(!strcmp(axis,x.GetName()) || !strcmp(axis,"x") || !strcmp(axis,"X"))      return _xSigma;
   else if(!strcmp(axis,y.GetName()) || !strcmp(axis,"y") || !strcmp(axis,"Y")) return _ySigma;
@@ -524,10 +521,9 @@ void Roo2DKeysPdf::writeToFile(char * outputFile, const char * name) const
 
 void Roo2DKeysPdf::writeHistToFile(char * outputFile, const char * histName) const
 {
-  TFile * file = 0;
   cout << "Roo2DKeysPdf::writeHistToFile This member function is temporarily disabled" <<endl;
   //make sure that any existing file is not over written
-  file = new TFile(outputFile, "UPDATE");
+  std::unique_ptr<TFile> file{TFile::Open(outputFile, "UPDATE")};
   if (!file)
   {
     cout << "Roo2DKeysPdf::writeHistToFile unable to open file "<< outputFile <<endl;
@@ -571,7 +567,7 @@ void Roo2DKeysPdf::writeNTupleToFile(char * outputFile, const char * name) const
   RooAbsReal & xArg = (RooAbsReal&)x.arg();
   RooAbsReal & yArg = (RooAbsReal&)y.arg();
 
-  Double_t theX, theY, hx/*, hy*/;
+  double theX, theY, hx/*, hy*/;
   TString label = name;
   label += " the source data for 2D Keys PDF";
   TTree * _theTree =  new TTree(name, label);
