@@ -43,37 +43,28 @@
 
 #include "RooAddModel.h"
 
-#include "RooFit.h"
+#include "RooAddHelpers.h"
 #include "RooMsgService.h"
 #include "RooDataSet.h"
 #include "RooRealProxy.h"
 #include "RooPlot.h"
 #include "RooRealVar.h"
 #include "RooAddGenContext.h"
-#include "RooRealConstant.h"
 #include "RooNameReg.h"
-#include "RooRealIntegral.h"
+#include "RooBatchCompute.h"
 
 using namespace std;
 
 ClassImp(RooAddModel);
-;
 
 
 ////////////////////////////////////////////////////////////////////////////////
 
 RooAddModel::RooAddModel() :
-  _refCoefNorm("!refCoefNorm","Reference coefficient normalization set",this,kFALSE,kFALSE),
-  _refCoefRangeName(0),
-  _projectCoefs(false),
+  _refCoefNorm("!refCoefNorm","Reference coefficient normalization set",this,false,false),
   _projCacheMgr(this,10),
-  _intCacheMgr(this,10),
-  _codeReg(10),
-  _snormList(0),
-  _haveLastCoef(false),
-  _allExtendable(false)
+  _intCacheMgr(this,10)
 {
-  _coefCache = new Double_t[10] ;
   _coefErrCount = _errorCount ;
 }
 
@@ -87,77 +78,83 @@ RooAddModel::RooAddModel() :
 ///
 /// All PDFs must inherit from RooAbsPdf. All coefficients must inherit from RooAbsReal.
 
-RooAddModel::RooAddModel(const char *name, const char *title, const RooArgList& inPdfList, const RooArgList& inCoefList, Bool_t ownPdfList) :
+RooAddModel::RooAddModel(const char *name, const char *title, const RooArgList& inPdfList, const RooArgList& inCoefList, bool ownPdfList) :
   RooResolutionModel(name,title,(static_cast<RooResolutionModel*>(inPdfList.at(0)))->convVar()),
-  _refCoefNorm("!refCoefNorm","Reference coefficient normalization set",this,kFALSE,kFALSE),
+  _refCoefNorm("!refCoefNorm","Reference coefficient normalization set",this,false,false),
   _refCoefRangeName(0),
-  _projectCoefs(kFALSE),
   _projCacheMgr(this,10),
   _intCacheMgr(this,10),
   _codeReg(10),
   _pdfList("!pdfs","List of PDFs",this),
   _coefList("!coefficients","List of coefficients",this),
-  _haveLastCoef(kFALSE),
-  _allExtendable(kFALSE)
-{ 
-  if (inPdfList.getSize()>inCoefList.getSize()+1) {
-    std::stringstream msgSs;
-    msgSs << "RooAddModel::RooAddModel(" << GetName()
-          << ") number of pdfs and coefficients inconsistent, must have Npdf=Ncoef or Npdf=Ncoef+1";
-    const std::string msgStr = msgSs.str();
-    coutE(InputArguments) << msgStr << endl;
-    throw std::runtime_error(msgStr);
-  }
- 
-  // Constructor with N PDFs and N or N-1 coefs
-  auto pdfIter = inPdfList.fwdIterator() ;
-
-  for(auto const& coef : inCoefList) {
-    auto pdf = pdfIter.next() ;
-    if (!pdf) {
+  _haveLastCoef(false),
+  _allExtendable(false)
+{
+   const std::string ownName(GetName() ? GetName() : "");
+   if (inPdfList.size() > inCoefList.size() + 1 || inPdfList.size() < inCoefList.size()) {
       std::stringstream msgSs;
-      msgSs << "RooAddModel::RooAddModel(" << GetName() 
+      msgSs << "RooAddModel::RooAddModel(" << ownName
             << ") number of pdfs and coefficients inconsistent, must have Npdf=Ncoef or Npdf=Ncoef+1";
       const std::string msgStr = msgSs.str();
-      coutE(InputArguments) << msgStr << endl;
+      coutE(InputArguments) << msgStr << "\n";
       throw std::runtime_error(msgStr);
-    }
-    if (!dynamic_cast<RooAbsReal*>(coef)) {
-      coutE(InputArguments) << "RooAddModel::RooAddModel(" << GetName() << ") coefficient " << coef->GetName() << " is not of type RooAbsReal, ignored" << endl ;
-      continue ;
-    }
-    if (!dynamic_cast<RooAbsPdf*>(pdf)) {
-      coutE(InputArguments) << "RooAddModel::RooAddModel(" << GetName() << ") pdf " << pdf->GetName() << " is not of type RooAbsPdf, ignored" << endl ;
-      continue ;
-    }
-    _pdfList.add(*pdf) ;
-    _coefList.add(*coef) ;    
-  }
+   }
 
-  if (auto pdf = pdfIter.next()) {
-    if (!dynamic_cast<RooAbsPdf*>(pdf)) {
-      std::stringstream msgSs;
-      msgSs << "RooAddModel::RooAddModel(" << GetName() << ") last pdf " << pdf->GetName() << " is not of type RooAbsPdf, fatal error";
-      const std::string msgStr = msgSs.str();
-      coutE(InputArguments) << msgStr << endl;
-      throw std::runtime_error(msgStr);
-    }
-    _pdfList.add(*pdf) ;  
-  } else {
-    _haveLastCoef=kTRUE ;
-  }
+   // Constructor with N PDFs and N or N-1 coefs
+   std::size_t i = 0;
+   for (auto const &coef : inCoefList) {
+      auto pdf = inPdfList.at(i);
+      if (!pdf) {
+         std::stringstream msgSs;
+         msgSs << "RooAddModel::RooAddModel(" << ownName
+               << ") number of pdfs and coefficients inconsistent, must have Npdf=Ncoef or Npdf=Ncoef+1";
+         const std::string msgStr = msgSs.str();
+         coutE(InputArguments) << msgStr << "\n";
+         throw std::runtime_error(msgStr);
+      }
+      if (!coef) {
+         coutE(InputArguments) << "RooAddModel::RooAddModel(" << ownName
+                               << ") encountered and undefined coefficient, ignored\n";
+         continue;
+      }
+      if (!dynamic_cast<RooAbsReal *>(coef)) {
+         auto coefName = coef->GetName();
+         coutE(InputArguments) << "RooAddModel::RooAddModel(" << ownName << ") coefficient "
+                               << (coefName != nullptr ? coefName : "") << " is not of type RooAbsReal, ignored\n";
+         continue;
+      }
+      if (!dynamic_cast<RooAbsPdf *>(pdf)) {
+         coutE(InputArguments) << "RooAddModel::RooAddModel(" << ownName << ") pdf "
+                               << (pdf->GetName() ? pdf->GetName() : "") << " is not of type RooAbsPdf, ignored\n";
+         continue;
+      }
+      _pdfList.add(*pdf);
+      _coefList.add(*coef);
+      i++;
+   }
 
-  _coefCache = new Double_t[_pdfList.getSize()] ;
-  _coefErrCount = _errorCount ;
+   if (i < inPdfList.size()) {
+      auto pdf = inPdfList.at(i);
+      if (!dynamic_cast<RooAbsPdf *>(pdf)) {
+         std::stringstream msgSs;
+         msgSs << "RooAddModel::RooAddModel(" << ownName << ") last pdf " << (pdf->GetName() ? pdf->GetName() : "")
+               << " is not of type RooAbsPdf, fatal error";
+         const std::string msgStr = msgSs.str();
+         coutE(InputArguments) << msgStr << "\n";
+         throw std::runtime_error(msgStr);
+      }
+      _pdfList.add(*pdf);
+   } else {
+      _haveLastCoef = true;
+   }
 
-  if (ownPdfList) {
-    _ownedComps.addOwned(_pdfList) ;
-  }
+   _coefErrCount = _errorCount;
 
+   if (ownPdfList) {
+      _ownedComps.addOwned(_pdfList);
+   }
 }
 
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Copy constructor
 
@@ -165,7 +162,6 @@ RooAddModel::RooAddModel(const RooAddModel& other, const char* name) :
   RooResolutionModel(other,name),
   _refCoefNorm("!refCoefNorm",this,other._refCoefNorm),
   _refCoefRangeName((TNamed*)other._refCoefRangeName),
-  _projectCoefs(other._projectCoefs),
   _projCacheMgr(other._projCacheMgr,this),
   _intCacheMgr(other._intCacheMgr,this),
   _codeReg(other._codeReg),
@@ -174,22 +170,11 @@ RooAddModel::RooAddModel(const RooAddModel& other, const char* name) :
   _haveLastCoef(other._haveLastCoef),
   _allExtendable(other._allExtendable)
 {
-  _coefCache = new Double_t[_pdfList.getSize()] ;
   _coefErrCount = _errorCount ;
 }
 
 
 
-////////////////////////////////////////////////////////////////////////////////
-/// Destructor
-
-RooAddModel::~RooAddModel()
-{
-  if (_coefCache) delete[] _coefCache ;
-}
-
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// By default the interpretation of the fraction coefficients is
 /// performed in the contextual choice of observables. This makes the
@@ -200,13 +185,11 @@ RooAddModel::~RooAddModel()
 /// from the reference normalization set to the contextual normalization
 /// set by ratios of integrals
 
-void RooAddModel::fixCoefNormalization(const RooArgSet& refCoefNorm) 
+void RooAddModel::fixCoefNormalization(const RooArgSet& refCoefNorm)
 {
-  if (refCoefNorm.getSize()==0) {
-    _projectCoefs = kFALSE ;
+  if (refCoefNorm.empty()) {
     return ;
   }
-  _projectCoefs = kTRUE ;  
 
   _refCoefNorm.removeAll() ;
   _refCoefNorm.add(refCoefNorm) ;
@@ -230,7 +213,6 @@ void RooAddModel::fixCoefNormalization(const RooArgSet& refCoefNorm)
 void RooAddModel::fixCoefRange(const char* rangeName)
 {
   _refCoefRangeName = (TNamed*)RooNameReg::ptr(rangeName) ;
-  if (_refCoefRangeName) _projectCoefs = kTRUE ;
 }
 
 
@@ -244,10 +226,10 @@ void RooAddModel::fixCoefRange(const char* rangeName)
 
 RooResolutionModel* RooAddModel::convolution(RooFormulaVar* inBasis, RooAbsArg* owner) const
 {
-  // Check that primary variable of basis functions is our convolution variable  
+  // Check that primary variable of basis functions is our convolution variable
   if (inBasis->getParameter(0) != x.absArg()) {
-    coutE(InputArguments) << "RooAddModel::convolution(" << GetName() 
-			  << ") convolution parameter of basis function and PDF don't match" << endl ;
+    coutE(InputArguments) << "RooAddModel::convolution(" << GetName()
+           << ") convolution parameter of basis function and PDF don't match" << endl ;
     ccoutE(InputArguments) << "basis->findServer(0) = " << inBasis->findServer(0) << " " << inBasis->findServer(0)->GetName() << endl ;
     ccoutE(InputArguments) << "x.absArg()           = " << x.absArg() << " " << x.absArg()->GetName() << endl ;
     inBasis->Print("v") ;
@@ -269,16 +251,16 @@ RooResolutionModel* RooAddModel::convolution(RooFormulaVar* inBasis, RooAbsArg*
   for (auto obj : _pdfList) {
     auto model = static_cast<RooResolutionModel*>(obj);
     // Create component convolution
-    RooResolutionModel* conv = model->convolution(inBasis,owner) ;    
+    RooResolutionModel* conv = model->convolution(inBasis,owner) ;
     modelList.add(*conv) ;
   }
 
-  RooArgList theCoefList ;  
+  RooArgList theCoefList ;
   for (auto coef : _coefList) {
     theCoefList.add(*coef) ;
   }
-    
-  RooAddModel* convSum = new RooAddModel(newName,newTitle,modelList,theCoefList,kTRUE) ;
+
+  RooAddModel* convSum = new RooAddModel(newName,newTitle,modelList,theCoefList,true) ;
   for (std::set<std::string>::const_iterator attrIt = _boolAttrib.begin();
       attrIt != _boolAttrib.end(); ++attrIt) {
     convSum->setAttribute((*attrIt).c_str()) ;
@@ -299,15 +281,15 @@ RooResolutionModel* RooAddModel::convolution(RooFormulaVar* inBasis, RooAbsArg*
 /// if the basis is supported by all components. Otherwise 0
 /// is returned
 
-Int_t RooAddModel::basisCode(const char* name) const 
+Int_t RooAddModel::basisCode(const char* name) const
 {
-  Bool_t first(kTRUE), code(0) ;
+  bool first(true), code(0) ;
   for (auto obj : _pdfList) {
     auto model = static_cast<RooResolutionModel*>(obj);
     Int_t subCode = model->basisCode(name) ;
     if (first) {
       code = subCode ;
-      first = kFALSE ;
+      first = false ;
     } else if (subCode==0) {
       code = 0 ;
     }
@@ -325,306 +307,68 @@ Int_t RooAddModel::basisCode(const char* name) const
 /// integrals to calculated transformed fraction coefficients when a frozen reference frame is provided
 /// and projection integrals for similar transformations when a frozen reference range is provided.
 
-RooAddModel::CacheElem* RooAddModel::getProjCache(const RooArgSet* nset, const RooArgSet* iset, const char* rangeName) const
+AddCacheElem* RooAddModel::getProjCache(const RooArgSet* nset, const RooArgSet* iset) const
 {
-  // Check if cache already exists 
-  CacheElem* cache = (CacheElem*) _projCacheMgr.getObj(nset,iset,0,RooNameReg::ptr(rangeName)) ;
+  // Check if cache already exists
+  auto cache = static_cast<AddCacheElem*>(_projCacheMgr.getObj(nset,iset,0,normRange()));
   if (cache) {
     return cache ;
   }
 
-  //Create new cache 
-  cache = new CacheElem ;
+  //Create new cache
+  cache = new AddCacheElem{*this, _pdfList, _coefList, nset, iset, _refCoefNorm,
+                           _refCoefRangeName ? RooNameReg::str(_refCoefRangeName) : "",
+                           _verboseEval};
 
-  // *** PART 1 : Create supplemental normalization list ***
+  _projCacheMgr.setObj(nset,iset,cache,RooNameReg::ptr(normRange())) ;
 
-  // Retrieve the combined set of dependents of this PDF ;
-  RooArgSet *fullDepList = getObservables(nset) ;
-  if (iset) {
-    fullDepList->remove(*iset,kTRUE,kTRUE) ;
-  }    
-
-  // Fill with dummy unit RRVs for now
-  for (unsigned int i = 0; i < _pdfList.size(); ++i) {
-    auto pdf = static_cast<RooAbsPdf*>(&_pdfList[i]);
-    auto coef = i < _coefList.size() ? &_coefList[i] : nullptr;
-
-    // Start with full list of dependents
-    RooArgSet supNSet(*fullDepList) ;
-
-    // Remove PDF dependents
-    RooArgSet* pdfDeps = pdf->getObservables(nset) ;
-    if (pdfDeps) {
-      supNSet.remove(*pdfDeps,kTRUE,kTRUE) ;
-      delete pdfDeps ; 
-    }
-
-    // Remove coef dependents
-    if (coef) {
-      RooArgSet* coefDeps = coef->getObservables(nset);
-      supNSet.remove(*coefDeps,kTRUE,kTRUE) ;
-      delete coefDeps ;
-    }
-    
-    RooAbsReal* snorm ;
-    TString name(GetName()) ;
-    name.Append("_") ;
-    name.Append(pdf->GetName()) ;
-    name.Append("_SupNorm") ;
-    if (supNSet.getSize()>0) {
-      snorm = new RooRealIntegral(name,"Supplemental normalization integral",RooRealConstant::value(1.0),supNSet) ;
-    } else {
-      snorm = new RooRealVar(name,"Unit Supplemental normalization integral",1.0) ;
-    }
-    cache->_suppNormList.addOwned(*snorm) ;
-  }
-
-  delete fullDepList ;
-    
-  if (_verboseEval>1) {
-    cxcoutD(Caching) << "RooAddModel::syncSuppNormList(" << GetName() << ") synching supplemental normalization list for norm" << (nset?*nset:RooArgSet()) << endl ;
-    if (dologD(Caching)) {
-      cache->_suppNormList.Print("v") ;
-    }
-  }
-
-
-  // *** PART 2 : Create projection coefficients ***
-
-  // If no projections required stop here
-  if (!_projectCoefs || _basis!=0 ) {
-    _projCacheMgr.setObj(nset,iset,cache,RooNameReg::ptr(rangeName)) ;
-    return cache ;
-  }
-
-
-  // Reduce iset/nset to actual dependents of this PDF
-  RooArgSet* nset2 = nset ? getObservables(nset) : new RooArgSet() ;
-
-  // Check if requested transformation is not identity 
-  if (!nset2->equals(_refCoefNorm) || _refCoefRangeName !=0 || rangeName !=0 ) {
-    
-    coutI(Caching)  << "RooAddModel::syncCoefProjList(" << GetName() << ") creating coefficient projection integrals" << endl ;
-    ccoutI(Caching) << "  from current normalization: "  ; nset2->Print("1") ;
-    ccoutI(Caching) << "          with current range: " << (rangeName?rangeName:"<none>") << endl ;
-    ccoutI(Caching) << "  to reference normalization: "  ; _refCoefNorm.Print("1") ; 
-    ccoutI(Caching) << "        with reference range: " << (_refCoefRangeName?RooNameReg::str(_refCoefRangeName):"<none>") << endl ;
-    
-    // Recalculate projection integrals of PDFs 
-    for (const auto obj : _pdfList) {
-        const auto thePdf = static_cast<RooAbsPdf*>(obj);
-
-      // Calculate projection integral
-      RooAbsReal* pdfProj ;
-      if (!nset2->equals(_refCoefNorm)) {
-	pdfProj = thePdf->createIntegral(*nset2,_refCoefNorm) ;
-	pdfProj->setOperMode(operMode()) ;
-      } else {
-	TString name(GetName()) ;
-	name.Append("_") ;
-	name.Append(thePdf->GetName()) ;
-	name.Append("_ProjectNorm") ;
-	pdfProj = new RooRealVar(name,"Unit Projection normalization integral",1.0) ;
-      }
-
-      cache->_projList.addOwned(*pdfProj) ;
-
-      // Calculation optional supplemental normalization term
-      RooArgSet supNormSet(_refCoefNorm) ;
-      RooArgSet* deps = thePdf->getParameters(RooArgSet()) ;
-      supNormSet.remove(*deps,kTRUE,kTRUE) ;
-      delete deps ;
-
-      RooAbsReal* snorm ;
-      TString name(GetName()) ;
-      name.Append("_") ;
-      name.Append(thePdf->GetName()) ;
-      name.Append("_ProjSupNorm") ;
-      if (supNormSet.getSize()>0) {
-	snorm = new RooRealIntegral(name,"Projection Supplemental normalization integral",
-				    RooRealConstant::value(1.0),supNormSet) ;
-      } else {
-	snorm = new RooRealVar(name,"Unit Projection Supplemental normalization integral",1.0) ;
-      }
-      cache->_suppProjList.addOwned(*snorm) ;
-
-      // Calculate reference range adjusted projection integral
-      RooAbsReal* rangeProj1 ;
-      if (_refCoefRangeName && _refCoefNorm.getSize()>0) {
-	rangeProj1 = thePdf->createIntegral(_refCoefNorm,_refCoefNorm,RooNameReg::str(_refCoefRangeName)) ;
-	rangeProj1->setOperMode(operMode()) ;
-      } else {
-	TString theName(GetName()) ;
-	theName.Append("_") ;
-	theName.Append(thePdf->GetName()) ;
-	theName.Append("_RangeNorm1") ;
-	rangeProj1 = new RooRealVar(theName,"Unit range normalization integral",1.0) ;
-      }
-      cache->_refRangeProjList.addOwned(*rangeProj1) ;
-      
-
-      // Calculate range adjusted projection integral
-      RooAbsReal* rangeProj2 ;
-      if (rangeName && _refCoefNorm.getSize()>0) {
-	rangeProj2 = thePdf->createIntegral(_refCoefNorm,_refCoefNorm,rangeName) ;
-	rangeProj2->setOperMode(operMode()) ;
-      } else {
-	TString theName(GetName()) ;
-	theName.Append("_") ;
-	theName.Append(thePdf->GetName()) ;
-	theName.Append("_RangeNorm2") ;
-	rangeProj2 = new RooRealVar(theName,"Unit range normalization integral",1.0) ;
-      }
-      cache->_rangeProjList.addOwned(*rangeProj2) ;
-
-    }               
-
-  }
-
-  delete nset2 ;
-
-  _projCacheMgr.setObj(nset,iset,cache,RooNameReg::ptr(rangeName)) ;
-
-  return cache ;
+  return cache;
 }
 
 
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Update the coefficient values in the given cache element: calculate new remainder
-/// fraction, normalize fractions obtained from extended ML terms to unity and
-/// multiply these the various range and dimensional corrections needed in the
-/// current use context
+/// fraction, normalize fractions obtained from extended ML terms to unity, and
+/// multiply the various range and dimensional corrections needed in the
+/// current use context.
 
-void RooAddModel::updateCoefficients(CacheElem& cache, const RooArgSet* nset) const 
+void RooAddModel::updateCoefficients(AddCacheElem& cache, const RooArgSet* nset) const
 {
-  // cxcoutD(ChangeTracking) << "RooAddModel::updateCoefficients(" << GetName() << ") update coefficients" << endl ;
-  
-  Int_t i ;
-
-  // Straight coefficients
-  if (_allExtendable) {
-    
-    // coef[i] = expectedEvents[i] / SUM(expectedEvents)
-    Double_t coefSum(0) ;
-    for (i=0 ; i<_pdfList.getSize() ; i++) {
-      _coefCache[i] = ((RooAbsPdf*)_pdfList.at(i))->expectedEvents(_refCoefNorm.getSize()>0?&_refCoefNorm:nset) ;
-      coefSum += _coefCache[i] ;
-    }
-    if (coefSum==0.) {
-      coutW(Eval) << "RooAddModel::updateCoefCache(" << GetName() << ") WARNING: total number of expected events is 0" << endl ;
-    } else {
-      for (i=0 ; i<_pdfList.getSize() ; i++) {
-	_coefCache[i] /= coefSum ;
-      }			            
-    }
-    
-  } else {
-    if (_haveLastCoef) {
-      
-      // coef[i] = coef[i] / SUM(coef)
-      Double_t coefSum(0) ;
-      for (i=0 ; i<_coefList.getSize() ; i++) {
-	_coefCache[i] = ((RooAbsPdf*)_coefList.at(i))->getVal(nset) ;
-	coefSum += _coefCache[i] ;
-      }		
-      for (i=0 ; i<_coefList.getSize() ; i++) {
-	_coefCache[i] /= coefSum ;
-      }			
-    } else {
-      
-      // coef[i] = coef[i] ; coef[n] = 1-SUM(coef[0...n-1])
-      Double_t lastCoef(1) ;
-      for (i=0 ; i<_coefList.getSize() ; i++) {
-	_coefCache[i] = ((RooAbsPdf*)_coefList.at(i))->getVal(nset) ;
- 	cxcoutD(Caching) << "SYNC: orig coef[" << i << "] = " << _coefCache[i] << endl ;
-	lastCoef -= _coefCache[i] ;
-      }			
-      _coefCache[_coefList.getSize()] = lastCoef ;
-      cxcoutD(Caching) << "SYNC: orig coef[" << _coefList.getSize() << "] = " << _coefCache[_coefList.getSize()] << endl ;
-      
-      
-      // Warn about coefficient degeneration
-      if ((lastCoef<-1e-05 || (lastCoef-1)>1e-5) && _coefErrCount-->0) {
-	coutW(Eval) << "RooAddModel::updateCoefCache(" << GetName() 
-		    << " WARNING: sum of PDF coefficients not in range [0-1], value=" 
-		    << 1-lastCoef << endl ;
-	if (_coefErrCount==0) {
-	  coutW(Eval) << " (no more will be printed)" << endl  ;
-	}
-      } 
-    }
+  _coefCache.resize(_pdfList.getSize());
+  for(std::size_t i = 0; i < _coefList.size(); ++i) {
+    _coefCache[i] = static_cast<RooAbsReal const&>(_coefList[i]).getVal(nset);
   }
-
-  
-
-  // Stop here if not projection is required or needed
-  if ((!_projectCoefs) || cache._projList.getSize()==0) {
-    //     cout << "SYNC no projection required rangeName = " << (rangeName?rangeName:"<none>") << endl ;
-    return ;
-  }
-
-  // Adjust coefficients for given projection
-  Double_t coefSum(0) ;
-  for (i=0 ; i<_pdfList.getSize() ; i++) {
-    GlobalSelectComponentRAII compRAII(true);
-
-    RooAbsReal* pp = ((RooAbsReal*)cache._projList.at(i)) ; 
-    RooAbsReal* sn = ((RooAbsReal*)cache._suppProjList.at(i)) ; 
-    RooAbsReal* r1 = ((RooAbsReal*)cache._refRangeProjList.at(i)) ;
-    RooAbsReal* r2 = ((RooAbsReal*)cache._rangeProjList.at(i)) ;
-    
-    if (dologD(Eval)) {
-      cxcoutD(Eval) << "pp = " << pp->GetName() << endl 
-		    << "sn = " << sn->GetName() << endl 
-		    << "r1 = " << r1->GetName() << endl 
-		    << "r2 = " << r2->GetName() << endl ;
-      r1->printStream(ccoutD(Eval),kName|kArgs|kValue,kSingleLine) ;
-      r1->printCompactTree(ccoutD(Eval)) ;
-    }
-
-    Double_t proj = pp->getVal()/sn->getVal()*(r2->getVal()/r1->getVal()) ;  
-
-    _coefCache[i] *= proj ;
-    coefSum += _coefCache[i] ;
-  }
-  for (i=0 ; i<_pdfList.getSize() ; i++) {
-    _coefCache[i] /= coefSum ;
-//     cout << "POST-SYNC coef[" << i << "] = " << _coefCache[i] << endl ;
-  }
-   
-
-  
+  RooAddHelpers::updateCoefficients(*this, _coefCache, _pdfList, _haveLastCoef, cache, nset,
+                                    _refCoefNorm, _allExtendable, _coefErrCount);
 }
 
 
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Calculate the current value
 
-Double_t RooAddModel::evaluate() const 
+double RooAddModel::evaluate() const
 {
-  const RooArgSet* nset = _normSet ; 
-  CacheElem* cache = getProjCache(nset) ;
+  const RooArgSet* nset = _normSet ;
+  AddCacheElem* cache = getProjCache(nset) ;
 
   updateCoefficients(*cache,nset) ;
 
-  
+
   // Do running sum of coef/pdf pairs, calculate lastCoef.
-  Double_t snormVal ;
-  Double_t value(0) ;
+  double snormVal ;
+  double value(0) ;
   Int_t i(0) ;
   for (auto obj : _pdfList) {
     auto pdf = static_cast<RooAbsPdf*>(obj);
 
     if (_coefCache[i]!=0.) {
-      snormVal = nset ? ((RooAbsReal*)cache->_suppNormList.at(i))->getVal() : 1.0 ;
-      Double_t pdfVal = pdf->getVal(nset) ;
-      // Double_t pdfNorm = pdf->getNorm(nset) ;
+      snormVal = nset ? cache->suppNormVal(i) : 1.0 ;
+      double pdfVal = pdf->getVal(nset) ;
+      // double pdfNorm = pdf->getNorm(nset) ;
       if (pdf->isSelectedComp()) {
-	value += pdfVal*_coefCache[i]/snormVal ;
- 	cxcoutD(Eval) << "RooAddModel::evaluate(" << GetName() << ")  value += [" 
- 			<< pdf->GetName() << "] " << pdfVal << " * " << _coefCache[i] << " / " << snormVal << endl ;
+   value += pdfVal*_coefCache[i]/snormVal ;
+   cxcoutD(Eval) << "RooAddModel::evaluate(" << GetName() << ")  value += ["
+         << pdf->GetName() << "] " << pdfVal << " * " << _coefCache[i] << " / " << snormVal << endl ;
       }
     }
     i++ ;
@@ -633,6 +377,46 @@ Double_t RooAddModel::evaluate() const
   return value ;
 }
 
+void RooAddModel::computeBatch(cudaStream_t *stream, double *output, size_t nEvents,
+                               RooFit::Detail::DataMap const &dataMap) const
+{
+   // Like many other functions in this class, the implementation was copy-pasted from the RooAddPdf
+
+   _coefCache.resize(_pdfList.size());
+   for (std::size_t i = 0; i < _coefList.size(); ++i) {
+      auto coefVals = dataMap.at(&_coefList[i]);
+      // We don't support per-event coefficients in this function. If the CPU
+      // mode is used, we can just fall back to the RooAbsReal implementation.
+      // With CUDA, we can't do that because the inputs might be on the device.
+      // That's why we throw an exception then.
+      if (coefVals.size() > 1) {
+         if (stream) {
+            throw std::runtime_error("The RooAddPdf doesn't support per-event coefficients in CUDA mode yet!");
+         }
+         RooAbsReal::computeBatch(stream, output, nEvents, dataMap);
+         return;
+      }
+      _coefCache[i] = coefVals[0];
+   }
+
+   RooBatchCompute::VarVector pdfs;
+   RooBatchCompute::ArgVector coefs;
+   const RooArgSet *nset = nullptr;
+   AddCacheElem *cache = getProjCache(nullptr);
+   // We don't sync the coefficient values from the _coefList to the _coefCache
+   // because we have already done it using the dataMap.
+   updateCoefficients(*cache, nset);
+
+   for (unsigned int pdfNo = 0; pdfNo < _pdfList.size(); ++pdfNo) {
+      auto pdf = static_cast<RooAbsPdf *>(&_pdfList[pdfNo]);
+      if (pdf->isSelectedComp()) {
+         pdfs.push_back(dataMap.at(pdf));
+         coefs.push_back(_coefCache[pdfNo] / cache->suppNormVal(pdfNo));
+      }
+   }
+   auto dispatch = stream ? RooBatchCompute::dispatchCUDA : RooBatchCompute::dispatchCPU;
+   dispatch->compute(stream, RooBatchCompute::AddPdf, output, nEvents, pdfs, coefs);
+}
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -649,24 +433,24 @@ void RooAddModel::resetErrorCounters(Int_t resetValue)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Check if PDF is valid for given normalization set.
-/// Coeffient and PDF must be non-overlapping, but pdf-coefficient 
+/// Coeffient and PDF must be non-overlapping, but pdf-coefficient
 /// pairs may overlap each other
 
-Bool_t RooAddModel::checkObservables(const RooArgSet* nset) const 
+bool RooAddModel::checkObservables(const RooArgSet* nset) const
 {
-  Bool_t ret(kFALSE) ;
+  bool ret(false) ;
 
   for (unsigned int i = 0; i < _coefList.size(); ++i) {
     auto pdf = &_pdfList[i];
     auto coef = &_coefList[i];
 
     if (pdf->observableOverlaps(nset,*coef)) {
-      coutE(InputArguments) << "RooAddModel::checkObservables(" << GetName() << "): ERROR: coefficient " << coef->GetName() 
-			    << " and PDF " << pdf->GetName() << " have one or more dependents in common" << endl ;
-      ret = kTRUE ;
+      coutE(InputArguments) << "RooAddModel::checkObservables(" << GetName() << "): ERROR: coefficient " << coef->GetName()
+             << " and PDF " << pdf->GetName() << " have one or more dependents in common" << endl ;
+      ret = true ;
     }
   }
-  
+
   return ret ;
 }
 
@@ -674,8 +458,8 @@ Bool_t RooAddModel::checkObservables(const RooArgSet* nset) const
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Int_t RooAddModel::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVars, 
-					 const RooArgSet* normSet, const char* rangeName) const 
+Int_t RooAddModel::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVars,
+                const RooArgSet* normSet, const char* rangeName) const
 {
   if (_forceNumInt) return 0 ;
 
@@ -686,9 +470,9 @@ Int_t RooAddModel::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVa
   Int_t code ;
   RooArgList *cilist ;
   getCompIntList(normSet,&allVars,cilist,code,rangeName) ;
-  
+
   return code+1 ;
-  
+
 }
 
 
@@ -696,7 +480,7 @@ Int_t RooAddModel::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVa
 ////////////////////////////////////////////////////////////////////////////////
 /// Check if this configuration was created before
 
-void RooAddModel::getCompIntList(const RooArgSet* nset, const RooArgSet* iset, pRooArgList& compIntList, Int_t& code, const char* isetRangeName) const 
+void RooAddModel::getCompIntList(const RooArgSet* nset, const RooArgSet* iset, pRooArgList& compIntList, Int_t& code, const char* isetRangeName) const
 {
   Int_t sterileIdx(-1) ;
 
@@ -704,7 +488,7 @@ void RooAddModel::getCompIntList(const RooArgSet* nset, const RooArgSet* iset, p
   if (cache) {
     code = _intCacheMgr.lastIndex() ;
     compIntList = &cache->_intList ;
-    
+
     return ;
   }
 
@@ -715,8 +499,7 @@ void RooAddModel::getCompIntList(const RooArgSet* nset, const RooArgSet* iset, p
   for (auto obj : _pdfList) {
     auto model = static_cast<RooResolutionModel*>(obj);
 
-    RooAbsReal* intPdf = model->createIntegral(*iset,nset,0,isetRangeName) ;
-    cache->_intList.addOwned(*intPdf) ;
+    cache->_intList.addOwned(std::unique_ptr<RooAbsReal>{model->createIntegral(*iset,nset,0,isetRangeName)});
   }
 
   // Store the partial integral list and return the assigned code ;
@@ -731,7 +514,7 @@ void RooAddModel::getCompIntList(const RooArgSet* nset, const RooArgSet* iset, p
 ////////////////////////////////////////////////////////////////////////////////
 /// Return analytical integral defined by given scenario code
 
-Double_t RooAddModel::analyticalIntegralWN(Int_t code, const RooArgSet* normSet, const char* rangeName) const 
+double RooAddModel::analyticalIntegralWN(Int_t code, const RooArgSet* normSet, const char* rangeName) const
 {
   // No integration scenario
   if (code==0) {
@@ -740,7 +523,7 @@ Double_t RooAddModel::analyticalIntegralWN(Int_t code, const RooArgSet* normSet,
 
   // Partial integration scenarios
   IntCacheElem* cache = (IntCacheElem*) _intCacheMgr.getObjByIndex(code-1) ;
-  
+
   RooArgList* compIntList ;
 
   // If cache has been sterilized, revive this slot
@@ -758,29 +541,29 @@ Double_t RooAddModel::analyticalIntegralWN(Int_t code, const RooArgSet* normSet,
   }
 
   // Calculate the current value
-  const RooArgSet* nset = _normSet ; 
-  CacheElem* pcache = getProjCache(nset) ;
+  const RooArgSet* nset = _normSet ;
+  AddCacheElem* pcache = getProjCache(nset) ;
 
   updateCoefficients(*pcache,nset) ;
-  
+
   // Do running sum of coef/pdf pairs, calculate lastCoef.
-  Double_t snormVal ;
-  Double_t value(0) ;
+  double snormVal ;
+  double value(0) ;
   Int_t i(0) ;
   for (const auto obj : *compIntList) {
     auto pdfInt = static_cast<const RooAbsReal*>(obj);
     if (_coefCache[i]!=0.) {
-      snormVal = nset ? ((RooAbsReal*)pcache->_suppNormList.at(i))->getVal() : 1.0 ;
-      Double_t intVal = pdfInt->getVal(nset) ;
+      snormVal = nset ? pcache->suppNormVal(i) : 1.0 ;
+      double intVal = pdfInt->getVal(nset) ;
       value += intVal*_coefCache[i]/snormVal ;
-      cxcoutD(Eval) << "RooAddModel::evaluate(" << GetName() << ")  value += [" 
-		      << pdfInt->GetName() << "] " << intVal << " * " << _coefCache[i] << " / " << snormVal << endl ;
+      cxcoutD(Eval) << "RooAddModel::evaluate(" << GetName() << ")  value += ["
+            << pdfInt->GetName() << "] " << intVal << " * " << _coefCache[i] << " / " << snormVal << endl ;
     }
     i++ ;
   }
-  
+
   return value ;
-  
+
 }
 
 
@@ -789,25 +572,25 @@ Double_t RooAddModel::analyticalIntegralWN(Int_t code, const RooArgSet* normSet,
 /// Return the number of expected events, which is either the sum of all coefficients
 /// or the sum of the components extended terms
 
-Double_t RooAddModel::expectedEvents(const RooArgSet* nset) const 
-{  
-  Double_t expectedTotal(0.0);
+double RooAddModel::expectedEvents(const RooArgSet* nset) const
+{
+  double expectedTotal(0.0);
 
   if (_allExtendable) {
-    
+
     // Sum of the extended terms
     for (auto obj : _pdfList) {
       auto pdf = static_cast<RooAbsPdf*>(obj);
       expectedTotal += pdf->expectedEvents(nset) ;
-    }   
-    
+    }
+
   } else {
-    
+
     // Sum the coefficients
     for (const auto obj : _coefList) {
       auto coef = static_cast<RooAbsReal*>(obj);
       expectedTotal += coef->getVal() ;
-    }   
+    }
   }
 
   return expectedTotal;
@@ -819,7 +602,7 @@ Double_t RooAddModel::expectedEvents(const RooArgSet* nset) const
 /// Interface function used by test statistics to freeze choice of observables
 /// for interpretation of fraction coefficients
 
-void RooAddModel::selectNormalization(const RooArgSet* depSet, Bool_t force) 
+void RooAddModel::selectNormalization(const RooArgSet* depSet, bool force)
 {
   if (!force && _refCoefNorm.getSize()!=0) {
     return ;
@@ -830,9 +613,9 @@ void RooAddModel::selectNormalization(const RooArgSet* depSet, Bool_t force)
     return ;
   }
 
-  RooArgSet* myDepSet = getObservables(depSet) ;
-  fixCoefNormalization(*myDepSet) ;
-  delete myDepSet ;
+  RooArgSet myDepSet;
+  getObservables(depSet, myDepSet);
+  fixCoefNormalization(myDepSet);
 }
 
 
@@ -841,7 +624,7 @@ void RooAddModel::selectNormalization(const RooArgSet* depSet, Bool_t force)
 /// Interface function used by test statistics to freeze choice of range
 /// for interpretation of fraction coefficients
 
-void RooAddModel::selectNormalizationRange(const char* rangeName, Bool_t force) 
+void RooAddModel::selectNormalizationRange(const char* rangeName, bool force)
 {
   if (!force && _refCoefRangeName) {
     return ;
@@ -855,10 +638,10 @@ void RooAddModel::selectNormalizationRange(const char* rangeName, Bool_t force)
 ////////////////////////////////////////////////////////////////////////////////
 /// Return specialized context to efficiently generate toy events from RooAddModels.
 
-RooAbsGenContext* RooAddModel::genContext(const RooArgSet &vars, const RooDataSet *prototype, 
-					const RooArgSet* auxProto, Bool_t verbose) const 
+RooAbsGenContext* RooAddModel::genContext(const RooArgSet &vars, const RooDataSet *prototype,
+               const RooArgSet* auxProto, bool verbose) const
 {
-  return new RooAddGenContext(*this,vars,prototype,auxProto,verbose) ;
+  return RooAddGenContext::create(*this,vars,prototype,auxProto,verbose).release();
 }
 
 
@@ -866,16 +649,16 @@ RooAbsGenContext* RooAddModel::genContext(const RooArgSet &vars, const RooDataSe
 ////////////////////////////////////////////////////////////////////////////////
 /// Direct generation is safe if all components say so
 
-Bool_t RooAddModel::isDirectGenSafe(const RooAbsArg& arg) const 
+bool RooAddModel::isDirectGenSafe(const RooAbsArg& arg) const
 {
   for (auto obj : _pdfList) {
     auto pdf = static_cast<RooAbsPdf*>(obj);
 
     if (!pdf->isDirectGenSafe(arg)) {
-      return kFALSE ;
+      return false ;
     }
   }
-  return kTRUE ;
+  return true ;
 }
 
 
@@ -883,7 +666,7 @@ Bool_t RooAddModel::isDirectGenSafe(const RooAbsArg& arg) const
 ////////////////////////////////////////////////////////////////////////////////
 /// Return pseud-code that indicates if all components can do internal generation (1) or not (0)
 
-Int_t RooAddModel::getGenerator(const RooArgSet& directVars, RooArgSet &/*generateVars*/, Bool_t /*staticInitOK*/) const
+Int_t RooAddModel::getGenerator(const RooArgSet& directVars, RooArgSet &/*generateVars*/, bool /*staticInitOK*/) const
 {
   for (auto obj : _pdfList) {
     auto pdf = static_cast<RooAbsPdf*>(obj);
@@ -893,7 +676,7 @@ Int_t RooAddModel::getGenerator(const RooArgSet& directVars, RooArgSet &/*genera
       return 0 ;
     }
   }
-  return 1 ;  
+  return 1 ;
 }
 
 
@@ -908,28 +691,10 @@ void RooAddModel::generateEvent(Int_t /*code*/)
 }
 
 
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// List all RooAbsArg derived contents in this cache element
 
-RooArgList RooAddModel::CacheElem::containedArgs(Action) 
-{
-  RooArgList allNodes;
-  allNodes.add(_projList) ;
-  allNodes.add(_suppProjList) ;
-  allNodes.add(_refRangeProjList) ;
-  allNodes.add(_rangeProjList) ;
-
-  return allNodes ;
-}
-
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// List all RooAbsArg derived contents in this cache element
-
-RooArgList RooAddModel::IntCacheElem::containedArgs(Action) 
+RooArgList RooAddModel::IntCacheElem::containedArgs(Action)
 {
   RooArgList allNodes(_intList) ;
   return allNodes ;
@@ -940,10 +705,10 @@ RooArgList RooAddModel::IntCacheElem::containedArgs(Action)
 /// Customized printing of arguments of a RooAddModel to more intuitively reflect the contents of the
 /// product operator construction
 
-void RooAddModel::printMetaArgs(ostream& os) const 
+void RooAddModel::printMetaArgs(ostream& os) const
 {
-  Bool_t first(kTRUE) ;
-    
+  bool first(true) ;
+
   os << "(" ;
   for (unsigned int i=0; i < _coefList.size(); ++i) {
     auto coef = &_coefList[i];
@@ -951,13 +716,13 @@ void RooAddModel::printMetaArgs(ostream& os) const
     if (!first) {
       os << " + " ;
     } else {
-      first = kFALSE ;
+      first = false ;
     }
     os << coef->GetName() << " * " << pdf->GetName() ;
   }
   if (_pdfList.size() > _coefList.size()) {
     os << " + [%] * " << _pdfList[_pdfList.size()-1].GetName() ;
   }
-  os << ") " ;    
+  os << ") " ;
 }
 
