@@ -54,20 +54,20 @@ Check the tutorial rf506_msgservice.C for details.
 **/
 
 #include "RooFormula.h"
-
-#include "RooFit.h"
+#include "BracketAdapters.h"
 #include "RooAbsReal.h"
 #include "RooAbsCategory.h"
 #include "RooArgList.h"
 #include "RooMsgService.h"
+#include "RunContext.h"
 #include "RooBatchCompute.h"
 
-#include "ROOT/RMakeUnique.hxx"
 #include "TObjString.h"
-#include "TClass.h"
 
-#include <sstream>
+#include <memory>
 #include <regex>
+#include <sstream>
+#include <cctype>
 
 using namespace std;
 
@@ -89,6 +89,103 @@ std::vector<bool> findCategoryServers(const RooAbsCollection& collection) {
   return output;
 }
 
+/// Convert `@i`-style references to `x[i]`.
+void convertArobaseReferences(std::string &formula)
+{
+   bool match = false;
+   for (std::size_t i = 0; i < formula.size(); ++i) {
+      if (match && !isdigit(formula[i])) {
+         formula.insert(formula.begin() + i, ']');
+         i += 1;
+         match = false;
+      } else if (!match && formula[i] == '@') {
+         formula[i] = 'x';
+         formula.insert(formula.begin() + i + 1, '[');
+         i += 1;
+         match = true;
+      }
+   }
+   if (match)
+      formula += ']';
+}
+
+/// Replace all occurences of `what` with `with` inside of `inout`.
+void replaceAll(std::string &inout, std::string_view what, std::string_view with)
+{
+   for (std::string::size_type pos{}; inout.npos != (pos = inout.find(what.data(), pos, what.length()));
+        pos += with.length()) {
+      inout.replace(pos, what.length(), with.data(), with.length());
+   }
+}
+
+/// Find the word boundaries with a static std::regex and return a bool vector
+/// flagging their positions. The end of the string is considered a word
+/// boundary.
+std::vector<bool> getWordBoundaryFlags(std::string const &s)
+{
+   static const std::regex r{"\\b"};
+   std::vector<bool> out(s.size() + 1);
+
+   for (auto i = std::sregex_iterator(s.begin(), s.end(), r); i != std::sregex_iterator(); ++i) {
+      std::smatch m = *i;
+      out[m.position()] = true;
+   }
+
+   // The end of a string is also a word boundary
+   out[s.size()] = true;
+
+   return out;
+}
+
+/// Replace all named references with "x[i]"-style.
+void replaceVarNamesWithIndexStyle(std::string &formula, RooArgList const &varList)
+{
+   std::vector<bool> isWordBoundary = getWordBoundaryFlags(formula);
+   for (unsigned int i = 0; i < varList.size(); ++i) {
+      std::string_view varName = varList[i].GetName();
+
+      std::stringstream replacementStream;
+      replacementStream << "x[" << i << "]";
+      std::string replacement = replacementStream.str();
+
+      for (std::string::size_type pos{}; formula.npos != (pos = formula.find(varName.data(), pos, varName.length()));
+           pos += replacement.size()) {
+
+         std::string::size_type next = pos + varName.length();
+
+         // The matched variable name has to be surrounded by word boundaries
+         // std::cout << pos << "   " << next << std::endl;
+         if (!isWordBoundary[pos] || !isWordBoundary[next])
+            continue;
+
+         // Veto '[' and ']' as next characters. If the variable is called `x`
+         // or `0`, this might otherwise replace `x[0]`.
+         if (next < formula.size() && (formula[next] == '[' || formula[next] == ']')) {
+            continue;
+         }
+
+         // As we replace substrings in the middle of the string, we also have
+         // to update the word boundary flag vector. Note that we don't care
+         // the word boundaries in the `x[i]` are correct, as it has already
+         // been replaced.
+         std::size_t nOld = varName.length();
+         std::size_t nNew = replacement.size();
+         auto wbIter = isWordBoundary.begin() + pos;
+         if (nNew > nOld)
+            isWordBoundary.insert(wbIter + nOld, nNew - nOld, false);
+         else if (nNew < nOld)
+            isWordBoundary.erase(wbIter + nNew, wbIter + nOld);
+
+         // Do the actual replacment
+         formula.replace(pos, varName.length(), replacement);
+      }
+
+      oocxcoutD(static_cast<TObject *>(nullptr), InputArguments)
+         << "Preprocessing formula: replace named references: " << varName << " --> " << replacement << "\n\t"
+         << formula << endl;
+   }
+}
+
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -129,24 +226,29 @@ RooFormula::RooFormula(const char* name, const char* formula, const RooArgList&
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Copy constructor
-RooFormula::RooFormula(const RooFormula& other, const char* name) : 
+RooFormula::RooFormula(const RooFormula& other, const char* name) :
   TNamed(name ? name : other.GetName(), other.GetTitle()), RooPrintable(other)
 {
   _origList.add(other._origList);
   _isCategory = findCategoryServers(_origList);
-  
-  TFormula* newTF = nullptr;
+
+  std::unique_ptr<TFormula> newTF;
   if (other._tFormula) {
-    newTF = new TFormula(*other._tFormula);
+    newTF = std::make_unique<TFormula>(*other._tFormula);
     newTF->SetName(GetName());
   }
 
-  _tFormula.reset(newTF);
+  _tFormula = std::move(newTF);
 }
 
+
 #ifndef _MSC_VER
 #if !defined(__GNUC__) || defined(__clang__) || (__GNUC__ > 4) || ( __GNUC__ == 4 && __GNUC_MINOR__ > 8)
 #define ROOFORMULA_HAVE_STD_REGEX
+#endif //GCC < 4.9 Check
+#endif //_MSC_VER
+
+#ifdef ROOFORMULA_HAVE_STD_REGEX
 ////////////////////////////////////////////////////////////////////////////////
 /// Process given formula by replacing all ordinal and name references by
 /// `x[i]`, where `i` matches the position of the argument in `_origList`.
@@ -154,11 +256,16 @@ RooFormula::RooFormula(const RooFormula& other, const char* name) :
 /// by the category index.
 std::string RooFormula::processFormula(std::string formula) const {
 
+  // WARNING to developers: people use RooFormula a lot via RooGenericPdf and
+  // RooFormulaVar! Performance matters here. Avoid non-static std::regex,
+  // because constructing these can become a bottleneck because of the regex
+  // compilation.
+
   cxcoutD(InputArguments) << "Preprocessing formula step 1: find category tags (catName::catState) in "
       << formula << endl;
 
   // Step 1: Find all category tags and the corresponding index numbers
-  std::regex categoryReg("(\\w+)::(\\w+)");
+  static const std::regex categoryReg("(\\w+)::(\\w+)");
   std::map<std::string, int> categoryStates;
   for (sregex_iterator matchIt = sregex_iterator(formula.begin(), formula.end(), categoryReg);
        matchIt != sregex_iterator(); ++matchIt) {
@@ -188,35 +295,18 @@ std::string RooFormula::processFormula(std::string formula) const {
 
   // Step 2: Replace all category tags
   for (const auto& catState : categoryStates) {
-    std::stringstream replacement;
-    replacement << catState.second;
-    formula = std::regex_replace(formula, std::regex(catState.first), replacement.str());
+    replaceAll(formula, catState.first, std::to_string(catState.second));
   }
 
   cxcoutD(InputArguments) << "Preprocessing formula step 2: replace category tags\n\t" << formula << endl;
 
   // Step 3: Convert `@i`-style references to `x[i]`
-  std::regex ordinalRegex("@([0-9]+)");
-  formula = std::regex_replace(formula, ordinalRegex, "x[$1]");
+  convertArobaseReferences(formula);
 
   cxcoutD(InputArguments) << "Preprocessing formula step 3: replace '@'-references\n\t" << formula << endl;
 
   // Step 4: Replace all named references with "x[i]"-style
-  for (unsigned int i = 0; i < _origList.size(); ++i) {
-    const auto& var = _origList[i];
-    auto regex = std::string{"\\b"} + var.GetName();
-    regex = std::regex_replace(regex, std::regex("([\\[\\]\\{\\}])"), "\\$1"); // The name might contain [, ], {, or }.
-    regex += "\\b(?!\\[)"; // Veto '[' as next character. If the variable is called `x`, this might otherwise replace `x[0]`.
-    std::regex findParameterRegex(regex);
-
-    std::stringstream replacement;
-    replacement << "x[" << i << "]";
-    formula = std::regex_replace(formula, findParameterRegex, replacement.str());
-
-    cxcoutD(InputArguments) << "Preprocessing formula step 4: replace named references: "
-        << var.GetName() << " --> " << replacement.str()
-        << "\n\t" << formula << endl;
-  }
+  replaceVarNamesWithIndexStyle(formula, _origList);
 
   cxcoutD(InputArguments) << "Final formula:\n\t" << formula << endl;
 
@@ -234,7 +324,7 @@ RooArgList RooFormula::usedVariables() const {
   const std::string formula(_tFormula->GetTitle());
 
   std::set<unsigned int> matchedOrdinals;
-  std::regex newOrdinalRegex("\\bx\\[([0-9]+)\\]");
+  static const std::regex newOrdinalRegex("\\bx\\[([0-9]+)\\]");
   for (sregex_iterator matchIt = sregex_iterator(formula.begin(), formula.end(), newOrdinalRegex);
       matchIt != sregex_iterator(); ++matchIt) {
     assert(matchIt->size() == 2);
@@ -269,8 +359,7 @@ std::string RooFormula::reconstructFormula(std::string internalRepr) const {
 
   return internalRepr;
 }
-#endif //GCC < 4.9 Check
-#endif //_MSC_VER
+#endif //ROOFORMULA_HAVE_STD_REGEX
 
 
 
@@ -278,7 +367,7 @@ std::string RooFormula::reconstructFormula(std::string internalRepr) const {
 ////////////////////////////////////////////////////////////////////////////////
 /// Recompile formula with new expression. In case of error, the old formula is
 /// retained.
-Bool_t RooFormula::reCompile(const char* newFormula)
+bool RooFormula::reCompile(const char* newFormula)
 {
   try {
     installFormulaOrThrow(newFormula);
@@ -302,8 +391,8 @@ void RooFormula::dump() const
 /// Change used variables to those with the same name in given list.
 /// \param[in] newDeps New dependents to replace the old ones.
 /// \param[in] mustReplaceAll Will yield an error if one dependent does not have a replacement.
-/// \param[in] nameChange Passed down to RooAbsArg::findNewServer(const RooAbsCollection&, Bool_t) const.
-Bool_t RooFormula::changeDependents(const RooAbsCollection& newDeps, Bool_t mustReplaceAll, Bool_t nameChange)
+/// \param[in] nameChange Passed down to RooAbsArg::findNewServer(const RooAbsCollection&, bool) const.
+bool RooFormula::changeDependents(const RooAbsCollection& newDeps, bool mustReplaceAll, bool nameChange)
 {
   //Change current servers to new servers with the same name given in list
   bool errorStat = false;
@@ -339,7 +428,7 @@ Bool_t RooFormula::changeDependents(const RooAbsCollection& newDeps, Bool_t must
 /// and then the formula is evaluated.
 /// \param[in] nset Normalisation set passed to evaluation of arguments serving values.
 /// \return The result of the evaluation.
-Double_t RooFormula::eval(const RooArgSet* nset) const
+double RooFormula::eval(const RooArgSet* nset) const
 {
   if (!_tFormula) {
     coutF(Eval) << __func__ << " (" << GetName() << "): Formula didn't compile: " << GetTitle() << endl;
@@ -408,10 +497,25 @@ RooSpan<double> RooFormula::evaluateSpan(const RooAbsReal* dataOwner, RooBatchCo
   return output;
 }
 
+void RooFormula::computeBatch(cudaStream_t*, double* output, size_t nEvents, RooFit::Detail::DataMap const& dataMap) const
+{
+  const int nPars=_origList.size();
+  std::vector<RooSpan<const double>> inputSpans(nPars);
+  for (int i=0; i<nPars; i++)
+    inputSpans[i] = dataMap.at( static_cast<const RooAbsReal*>(&_origList[i]) );
+
+  std::vector<double> pars(nPars);
+  for (size_t i=0; i<nEvents; i++)
+  {
+    for (int j=0; j<nPars; j++) pars[j] = inputSpans[j].size()>1 ? inputSpans[j][i] : inputSpans[j][0];
+    output[i] = _tFormula->EvalPar( pars.data() );
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Printing interface
 
-void RooFormula::printMultiline(ostream& os, Int_t /*contents*/, Bool_t /*verbose*/, TString indent) const 
+void RooFormula::printMultiline(ostream& os, Int_t /*contents*/, bool /*verbose*/, TString indent) const
 {
   os << indent << "--- RooFormula ---" << endl;
   os << indent << " Formula:        '" << GetTitle() << "'" << endl;
@@ -425,7 +529,7 @@ void RooFormula::printMultiline(ostream& os, Int_t /*contents*/, Bool_t /*verbos
 ////////////////////////////////////////////////////////////////////////////////
 /// Print value of formula
 
-void RooFormula::printValue(ostream& os) const 
+void RooFormula::printValue(ostream& os) const
 {
   os << const_cast<RooFormula*>(this)->eval(0) ;
 }
@@ -434,7 +538,7 @@ void RooFormula::printValue(ostream& os) const
 ////////////////////////////////////////////////////////////////////////////////
 /// Print name of formula
 
-void RooFormula::printName(ostream& os) const 
+void RooFormula::printName(ostream& os) const
 {
   os << GetName() ;
 }
@@ -443,7 +547,7 @@ void RooFormula::printName(ostream& os) const
 ////////////////////////////////////////////////////////////////////////////////
 /// Print title of formula
 
-void RooFormula::printTitle(ostream& os) const 
+void RooFormula::printTitle(ostream& os) const
 {
   os << GetTitle() ;
 }
@@ -452,16 +556,16 @@ void RooFormula::printTitle(ostream& os) const
 ////////////////////////////////////////////////////////////////////////////////
 /// Print class name of formula
 
-void RooFormula::printClassName(ostream& os) const 
+void RooFormula::printClassName(ostream& os) const
 {
-  os << IsA()->GetName() ;
+  os << ClassName() ;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Print arguments of formula, i.e. dependents that are actually used
 
-void RooFormula::printArgs(ostream& os) const 
+void RooFormula::printArgs(ostream& os) const
 {
   os << "[ actualVars=";
   for (const auto arg : usedVariables()) {
@@ -596,7 +700,7 @@ std::string RooFormula::processFormula(std::string formula) const {
     const auto& var = _origList[i];
     TString regex = "\\b";
     regex += var.GetName();
-    regex += "\\b([^[]|$)"; //Negative lookahead. If the variable is called `x`, this might otherwise replace `x[0]`.
+    regex += "\\b([^\\[\\]]|$)"; //Negative lookahead. If the variable is called `x` or `0`, this might otherwise replace `x[0]`.
     TPRegexp findParameterRegex(regex);
 
     std::stringstream replacement;
@@ -617,6 +721,7 @@ std::string RooFormula::processFormula(std::string formula) const {
 }
 
 
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Analyse internal formula to find out which variables are actually in use.
 RooArgList RooFormula::usedVariables() const {
@@ -671,4 +776,4 @@ std::string RooFormula::reconstructFormula(std::string internalRepr) const {
 
   return internalReprT.Data();
 }
-#endif //GCC < 4.9 Check
+#endif //ROOFORMULA_HAVE_STD_REGEX
