@@ -22,10 +22,9 @@
 RooProdGenContext is an efficient implementation of the generator context
 specific for RooProdPdf PDFs. The sim-context owns a list of
 component generator contexts that are used to generate the dependents
-for each component PDF sequentially. 
+for each component PDF sequentially.
 **/
 
-#include "RooFit.h"
 #include "Riostream.h"
 #include "RooMsgService.h"
 
@@ -41,263 +40,251 @@ using namespace std;
 
 ClassImp(RooProdGenContext);
 ;
-  
+
 
 ////////////////////////////////////////////////////////////////////////////////
 
-RooProdGenContext::RooProdGenContext(const RooProdPdf &model, const RooArgSet &vars, 
-				     const RooDataSet *prototype, const RooArgSet* auxProto, Bool_t verbose) :
-  RooAbsGenContext(model,vars,prototype,auxProto,verbose), _uniIter(0), _pdf(&model)
+RooProdGenContext::RooProdGenContext(const RooProdPdf &model, const RooArgSet &vars,
+                 const RooDataSet *prototype, const RooArgSet* auxProto, bool verbose) :
+  RooAbsGenContext(model,vars,prototype,auxProto,verbose), _pdf(&model)
 {
   // Constructor of optimization generator context for RooProdPdf objects
 
   //Build an array of generator contexts for each product component PDF
-  cxcoutI(Generation) << "RooProdGenContext::ctor() setting up event special generator context for product p.d.f. " << model.GetName() 
-			<< " for generation of observable(s) " << vars ;
+  cxcoutI(Generation) << "RooProdGenContext::ctor() setting up event special generator context for product p.d.f. " << model.GetName()
+         << " for generation of observable(s) " << vars ;
   if (prototype) ccxcoutI(Generation) << " with prototype data for " << *prototype->get() ;
-  if (auxProto && auxProto->getSize()>0)  ccxcoutI(Generation) << " with auxiliary prototypes " << *auxProto ;
+  if (auxProto && !auxProto->empty())  ccxcoutI(Generation) << " with auxiliary prototypes " << *auxProto ;
   ccxcoutI(Generation) << endl ;
 
   // Make full list of dependents (generated & proto)
   RooArgSet deps(vars) ;
   if (prototype) {
-    RooArgSet* protoDeps = model.getObservables(*prototype->get()) ;
-    deps.remove(*protoDeps,kTRUE,kTRUE) ;
-    delete protoDeps ;
+    RooArgSet protoDeps;
+    model.getObservables(prototype->get(), protoDeps);
+    deps.remove(protoDeps,true,true) ;
   }
 
   // Factorize product in irreducible terms
   RooLinkedList termList,depsList,impDepList,crossDepList,intList ;
   model.factorizeProduct(deps,RooArgSet(),termList,depsList,impDepList,crossDepList,intList) ;
-  TIterator* termIter = termList.MakeIterator() ;
-  TIterator* normIter = depsList.MakeIterator() ;
-  TIterator* impIter = impDepList.MakeIterator() ;
-  
+
   if (dologD(Generation)) {
-    cxcoutD(Generation) << "RooProdGenContext::ctor() factorizing product expression in irriducible terms " ;    
-    while(RooArgSet* t=(RooArgSet*)termIter->Next()) {
+    cxcoutD(Generation) << "RooProdGenContext::ctor() factorizing product expression in irriducible terms " ;
+    for(auto * t : static_range_cast<RooArgSet*>(termList)) {
       ccxcoutD(Generation) << *t ;
     }
-    ccxcoutD(Generation) << endl ;
+    ccxcoutD(Generation) << std::endl;
   }
 
   RooArgSet genDeps ;
   // First add terms that do not import observables
-  
-  Bool_t anyAction = kTRUE ;
-  Bool_t go=kTRUE ; 
+
+  bool anyAction = true ;
+  bool go=true ;
   while(go) {
 
-    RooAbsPdf* pdf ;
-    RooArgSet* term ;
-    RooArgSet* impDeps ;
-    RooArgSet* termDeps ;
+    auto termIter = termList.begin();
+    auto impIter = impDepList.begin();
+    auto normIter = depsList.begin();
 
-    termIter->Reset() ;
-    impIter->Reset() ;
-    normIter->Reset() ;
+    bool anyPrevAction=anyAction ;
+    anyAction=false ;
 
-    Bool_t anyPrevAction=anyAction ;
-    anyAction=kFALSE ;
-
-    if (termList.GetSize()==0) {
+    if (termList.empty()) {
       break ;
     }
 
-    while((term=(RooArgSet*)termIter->Next())) {
+    while(termIter != termList.end()) {
 
-      impDeps = (RooArgSet*)impIter->Next() ;
-      termDeps = (RooArgSet*)normIter->Next() ;
-      if (impDeps==0 || termDeps==0) {
-	break ;
+      auto * term = static_cast<RooArgSet*>(*termIter);
+      auto * impDeps = static_cast<RooArgSet*>(*impIter);
+      auto * termDeps = static_cast<RooArgSet*>(*normIter);
+      if (impDeps==nullptr || termDeps==nullptr) {
+   break ;
       }
 
       cxcoutD(Generation) << "RooProdGenContext::ctor() analyzing product term " << *term << " with observable(s) " << *termDeps ;
-      if (impDeps->getSize()>0) {
-	ccxcoutD(Generation) << " which has dependence of external observable(s) " << *impDeps << " that to be generated first by other terms" ;
+      if (!impDeps->empty()) {
+   ccxcoutD(Generation) << " which has dependence of external observable(s) " << *impDeps << " that to be generated first by other terms" ;
       }
-      ccxcoutD(Generation) << endl ;
+      ccxcoutD(Generation) << std::endl;
 
       // Add this term if we have no imported dependents, or imported dependents are already generated
       RooArgSet neededDeps(*impDeps) ;
-      neededDeps.remove(genDeps,kTRUE,kTRUE) ;
+      neededDeps.remove(genDeps,true,true) ;
 
-      if (neededDeps.getSize()>0) {
-	if (!anyPrevAction) {
-	  cxcoutD(Generation) << "RooProdGenContext::ctor() no convergence in single term analysis loop, terminating loop and process remainder of terms as single unit " << endl ;
-	  go=kFALSE ;
-	  break ;
-	}
-	cxcoutD(Generation) << "RooProdGenContext::ctor() skipping this term for now because it needs imported dependents that are not generated yet" << endl ;	
-	continue ;
+      if (!neededDeps.empty()) {
+   if (!anyPrevAction) {
+     cxcoutD(Generation) << "RooProdGenContext::ctor() no convergence in single term analysis loop, terminating loop and process remainder of terms as single unit " << endl ;
+     go=false ;
+     break ;
+   }
+   cxcoutD(Generation) << "RooProdGenContext::ctor() skipping this term for now because it needs imported dependents that are not generated yet" << endl ;
+   ++termIter;
+   ++impIter;
+   ++normIter;
+   continue ;
       }
 
       // Check if this component has any dependents that need to be generated
       // e.g. it can happen that there are none if all dependents of this component are prototyped
-      if (termDeps->getSize()==0) {
-	cxcoutD(Generation) << "RooProdGenContext::ctor() term has no observables requested to be generated, removing it" << endl ;
-	termList.Remove(term) ;
-	depsList.Remove(termDeps) ;
-	impDepList.Remove(impDeps) ;
-	delete term ;
-	delete termDeps ;
-	delete impDeps ;
-	anyAction=kTRUE ;
-	continue ;
+      if (termDeps->empty()) {
+   cxcoutD(Generation) << "RooProdGenContext::ctor() term has no observables requested to be generated, removing it" << endl ;
+
+   // Increment the iterators first, because Removing the corresponding element
+   // would invalidate them otherwise.
+   ++termIter;
+   ++normIter;
+   ++impIter;
+   termList.Remove(term);
+   depsList.Remove(termDeps);
+   impDepList.Remove(impDeps);
+
+   delete term ;
+   delete termDeps ;
+   delete impDeps ;
+   anyAction=true ;
+   continue ;
       }
 
-      TIterator* pdfIter = term->createIterator() ;      
-      if (term->getSize()==1) {
-	// Simple term
-	
-	pdf = (RooAbsPdf*) pdfIter->Next() ;
-	RooArgSet* pdfDep = pdf->getObservables(termDeps) ;
-	if (pdfDep->getSize()>0) {
- 	  coutI(Generation) << "RooProdGenContext::ctor() creating subcontext for generation of observables " << *pdfDep << " from model " << pdf->GetName() << endl ;
-	  RooArgSet* auxProto2 = pdf->getObservables(impDeps) ;
-	  RooAbsGenContext* cx = pdf->genContext(*pdfDep,prototype,auxProto2,verbose) ;
-	  delete auxProto2 ;
-	  _gcList.push_back(cx) ;
-	} 
+      if (term->size()==1) {
+   // Simple term
 
-// 	cout << "adding following dependents to list of generated observables: " ; pdfDep->Print("1") ;
-	genDeps.add(*pdfDep) ;
+   auto pdf = static_cast<RooAbsPdf*>((*term)[0]);
+   std::unique_ptr<RooArgSet> pdfDep{pdf->getObservables(termDeps)};
+   if (!pdfDep->empty()) {
+     coutI(Generation) << "RooProdGenContext::ctor() creating subcontext for generation of observables " << *pdfDep << " from model " << pdf->GetName() << endl ;
+     std::unique_ptr<RooArgSet> auxProto2{pdf->getObservables(impDeps)};
+     _gcList.push_back(pdf->genContext(*pdfDep,prototype,auxProto2.get(),verbose)) ;
+   }
+
+//    cout << "adding following dependents to list of generated observables: " ; pdfDep->Print("1") ;
+   genDeps.add(*pdfDep) ;
 
-	delete pdfDep ;
-	
       } else {
-	
-	// Composite term
-	if (termDeps->getSize()>0) {
-	  const std::string name = model.makeRGPPName("PRODGEN_",*term,RooArgSet(),RooArgSet(),0) ;      
-	  
-	  // Construct auxiliary PDF expressing product of composite terms, 
-	  // following Conditional component specification of input model
-	  RooLinkedList cmdList ;
-	  RooLinkedList pdfSetList ;
-	  pdfIter->Reset() ;
-	  RooArgSet fullPdfSet ;
-	  while((pdf=(RooAbsPdf*)pdfIter->Next())) {
 
-	    RooArgSet* pdfnset = model.findPdfNSet(*pdf) ;
-	    RooArgSet* pdfSet = new RooArgSet(*pdf) ;
-	    pdfSetList.Add(pdfSet) ;
+   // Composite term
+   if (!termDeps->empty()) {
+     const std::string name = model.makeRGPPName("PRODGEN_",*term,RooArgSet(),RooArgSet(),0) ;
 
-	    if (pdfnset && pdfnset->getSize()>0) {
-	      // This PDF requires a Conditional() construction
-	      cmdList.Add(RooFit::Conditional(*pdfSet,*pdfnset).Clone()) ;
-//   	      cout << "Conditional " << pdf->GetName() << " " ; pdfnset->Print("1") ;
-	    } else {
-	      fullPdfSet.add(*pdfSet) ;
-	    }
-	    
-	  }
-	  RooProdPdf* multiPdf = new RooProdPdf(name.c_str(),name.c_str(),fullPdfSet,cmdList) ;
-	  cmdList.Delete() ;
-	  pdfSetList.Delete() ;
+     // Construct auxiliary PDF expressing product of composite terms,
+     // following Conditional component specification of input model
+     RooLinkedList cmdList ;
+     RooLinkedList pdfSetList ;
+     RooArgSet fullPdfSet ;
+     for(auto * pdf : static_range_cast<RooAbsPdf*>(*term)) {
 
-	  multiPdf->setOperMode(RooAbsArg::ADirty,kTRUE) ;
-	  multiPdf->useDefaultGen(kTRUE) ;
-	  _ownedMultiProds.addOwned(*multiPdf) ;
-	  
-   	  coutI(Generation) << "RooProdGenContext()::ctor creating subcontext for generation of observables " << *termDeps 
-			      << "for irriducuble composite term using sub-product object " << multiPdf->GetName() ;
-	  RooAbsGenContext* cx = multiPdf->genContext(*termDeps,prototype,auxProto,verbose) ;
-	  _gcList.push_back(cx) ;
+       RooArgSet* pdfnset = model.findPdfNSet(*pdf) ;
+       RooArgSet* pdfSet = new RooArgSet(*pdf) ;
+       pdfSetList.Add(pdfSet) ;
 
-	  genDeps.add(*termDeps) ;
+       if (pdfnset && !pdfnset->empty()) {
+         // This PDF requires a Conditional() construction
+         cmdList.Add(RooFit::Conditional(*pdfSet,*pdfnset).Clone()) ;
+//          cout << "Conditional " << pdf->GetName() << " " ; pdfnset->Print("1") ;
+       } else {
+         fullPdfSet.add(*pdfSet) ;
+       }
 
-	}
+     }
+     RooProdPdf* multiPdf = new RooProdPdf(name.c_str(),name.c_str(),fullPdfSet,cmdList) ;
+     cmdList.Delete() ;
+     pdfSetList.Delete() ;
+
+     multiPdf->setOperMode(RooAbsArg::ADirty,true) ;
+     multiPdf->useDefaultGen(true) ;
+     _ownedMultiProds.addOwned(*multiPdf) ;
+
+        coutI(Generation) << "RooProdGenContext()::ctor creating subcontext for generation of observables " << *termDeps
+               << "for irriducuble composite term using sub-product object " << multiPdf->GetName() ;
+     RooAbsGenContext* cx = multiPdf->genContext(*termDeps,prototype,auxProto,verbose) ;
+     _gcList.push_back(cx) ;
+
+     genDeps.add(*termDeps) ;
+
+   }
       }
-      
-      delete pdfIter ;
 
-//        cout << "added generator for this term, removing from list" << endl ;
-
-      termList.Remove(term) ;
-      depsList.Remove(termDeps) ;
-      impDepList.Remove(impDeps) ;
+      // Increment the iterators first, because Removing the corresponding
+      // element would invalidate them otherwise.
+      ++termIter;
+      ++normIter;
+      ++impIter;
+      termList.Remove(term);
+      depsList.Remove(termDeps);
+      impDepList.Remove(impDeps);
       delete term ;
       delete termDeps ;
       delete impDeps ;
-      anyAction=kTRUE ;
+      anyAction=true ;
     }
   }
 
-  // Check if there are any left over terms that cannot be generated 
+  // Check if there are any left over terms that cannot be generated
   // separately due to cross dependency of observables
-  if (termList.GetSize()>0) {
+  if (!termList.empty()) {
 
     cxcoutD(Generation) << "RooProdGenContext::ctor() there are left-over terms that need to be generated separately" << endl ;
- 
-    RooAbsPdf* pdf ;
-    RooArgSet* term ;
 
     // Concatenate remaining terms
-    termIter->Reset() ;
-    normIter->Reset() ;
+    auto normIter = depsList.begin();
     RooArgSet trailerTerm ;
     RooArgSet trailerTermDeps ;
-    while((term=(RooArgSet*)termIter->Next())) {
-      RooArgSet* termDeps = (RooArgSet*)normIter->Next() ;
+    for(auto * term : static_range_cast<RooArgSet*>(termList)) {
+      auto* termDeps = static_cast<RooArgSet*>(*normIter);
       trailerTerm.add(*term) ;
       trailerTermDeps.add(*termDeps) ;
+      ++normIter;
     }
 
-    const std::string name = model.makeRGPPName("PRODGEN_",trailerTerm,RooArgSet(),RooArgSet(),0) ;      
-      
-    // Construct auxiliary PDF expressing product of composite terms, 
+    const std::string name = model.makeRGPPName("PRODGEN_",trailerTerm,RooArgSet(),RooArgSet(),0) ;
+
+    // Construct auxiliary PDF expressing product of composite terms,
     // following Partial/Full component specification of input model
     RooLinkedList cmdList ;
     RooLinkedList pdfSetList ;
     RooArgSet fullPdfSet ;
 
-    TIterator* pdfIter = trailerTerm.createIterator() ;
-    while((pdf=(RooAbsPdf*)pdfIter->Next())) {
-	
+    for(auto * pdf : static_range_cast<RooAbsPdf*>(trailerTerm)) {
+
       RooArgSet* pdfnset = model.findPdfNSet(*pdf) ;
       RooArgSet* pdfSet = new RooArgSet(*pdf) ;
       pdfSetList.Add(pdfSet) ;
-      
-      if (pdfnset && pdfnset->getSize()>0) {
-	// This PDF requires a Conditional() construction
-	  cmdList.Add(RooFit::Conditional(*pdfSet,*pdfnset).Clone()) ;
+
+      if (pdfnset && !pdfnset->empty()) {
+   // This PDF requires a Conditional() construction
+     cmdList.Add(RooFit::Conditional(*pdfSet,*pdfnset).Clone()) ;
       } else {
-	fullPdfSet.add(*pdfSet) ;
+   fullPdfSet.add(*pdfSet) ;
       }
-      
+
     }
 //     cmdList.Print("v") ;
     RooProdPdf* multiPdf = new RooProdPdf(name.c_str(),name.c_str(),fullPdfSet,cmdList) ;
     cmdList.Delete() ;
     pdfSetList.Delete() ;
-   
-    multiPdf->setOperMode(RooAbsArg::ADirty,kTRUE) ;
-    multiPdf->useDefaultGen(kTRUE) ;
+
+    multiPdf->setOperMode(RooAbsArg::ADirty,true) ;
+    multiPdf->useDefaultGen(true) ;
     _ownedMultiProds.addOwned(*multiPdf) ;
-    
-    cxcoutD(Generation) << "RooProdGenContext(" << model.GetName() << "): creating context for irreducible composite trailer term " 
-	 << multiPdf->GetName() << " that generates observables " << trailerTermDeps << endl ;
+
+    cxcoutD(Generation) << "RooProdGenContext(" << model.GetName() << "): creating context for irreducible composite trailer term "
+    << multiPdf->GetName() << " that generates observables " << trailerTermDeps << endl ;
     RooAbsGenContext* cx = multiPdf->genContext(trailerTermDeps,prototype,auxProto,verbose) ;
-    _gcList.push_back(cx) ;    
+    _gcList.push_back(cx) ;
   }
 
   // Now check if the are observables in vars that are not generated by any of the above p.d.f.s
   // If not, generate uniform distributions for these using a special context
   _uniObs.add(vars) ;
-  _uniObs.remove(genDeps,kTRUE,kTRUE) ;
-  if (_uniObs.getSize()>0) {
-    _uniIter = _uniObs.createIterator() ;
-    coutI(Generation) << "RooProdGenContext(" << model.GetName() << "): generating uniform distribution for non-dependent observable(s) " << _uniObs << endl ;
+  _uniObs.remove(genDeps,true,true) ;
+  if (!_uniObs.empty()) {
+    coutI(Generation) << "RooProdGenContext(" << model.GetName() << "): generating uniform distribution for non-dependent observable(s) " << _uniObs << std::endl;
   }
 
 
-  delete termIter ;
-  delete impIter ;
-  delete normIter ;
-
-
-  // We own contents of lists filled by factorizeProduct() 
+  // We own contents of lists filled by factorizeProduct()
   termList.Delete() ;
   depsList.Delete() ;
   impDepList.Delete() ;
@@ -313,7 +300,6 @@ RooProdGenContext::RooProdGenContext(const RooProdPdf &model, const RooArgSet &v
 
 RooProdGenContext::~RooProdGenContext()
 {
-  delete _uniIter ;
   for (list<RooAbsGenContext*>::iterator iter=_gcList.begin() ; iter!=_gcList.end() ; ++iter) {
     delete (*iter) ;
   }
@@ -323,7 +309,7 @@ RooProdGenContext::~RooProdGenContext()
 ////////////////////////////////////////////////////////////////////////////////
 /// Attach generator to given event buffer
 
-void RooProdGenContext::attach(const RooArgSet& args) 
+void RooProdGenContext::attach(const RooArgSet& args)
 {
   //Forward initGenerator call to all components
   for (list<RooAbsGenContext*>::iterator iter=_gcList.begin() ; iter!=_gcList.end() ; ++iter) {
@@ -359,18 +345,15 @@ void RooProdGenContext::generateEvent(RooArgSet &theEvent, Int_t remaining)
     (*iter)->generateEvent(theEvent,remaining) ;
   }
 
-  // Generate uniform variables (non-dependents)  
-  if (_uniIter) {
-    _uniIter->Reset() ;
-    RooAbsArg* uniVar ;
-    while((uniVar=(RooAbsArg*)_uniIter->Next())) {
-      RooAbsLValue* arglv = dynamic_cast<RooAbsLValue*>(uniVar) ;
+  // Generate uniform variables (non-dependents)
+  if (!_uniObs.empty()) {
+    for(auto * arglv : dynamic_range_cast<RooAbsLValue*>(_uniObs)) {
       if (arglv) {
-	arglv->randomize() ;
+   arglv->randomize() ;
       }
     }
     theEvent.assign(_uniObs) ;
-  }  
+  }
 
 }
 
@@ -396,7 +379,7 @@ void RooProdGenContext::setProtoDataOrder(Int_t* lut)
 ////////////////////////////////////////////////////////////////////////////////
 /// Detailed printing interface
 
-void RooProdGenContext::printMultiline(ostream &os, Int_t content, Bool_t verbose, TString indent) const 
+void RooProdGenContext::printMultiline(ostream &os, Int_t content, bool verbose, TString indent) const
 {
   RooAbsGenContext::printMultiline(os,content,verbose,indent) ;
   os << indent << "--- RooProdGenContext ---" << endl ;
