@@ -28,8 +28,6 @@ but are not automatically destroyed at the end of the session. This class
 installs an atexit() function that takes care of this
 **/
 
-#include "RooFit.h"
-
 #include "RooSentinel.h"
 #include "RooArgSet.h"
 #include "RooRealConstant.h"
@@ -37,7 +35,7 @@ installs an atexit() function that takes care of this
 #include "RooExpensiveObjectCache.h"
 #include "RooDataSet.h"
 
-Bool_t RooSentinel::_active = kFALSE ;
+bool RooSentinel::_active = false ;
 
 static void CleanUpRooFitAtExit()
 {
@@ -56,10 +54,10 @@ static void CleanUpRooFitAtExit()
 void RooSentinel::activate()
 {
   if (!_active) {
-    _active = kTRUE ;
+    _active = true ;
     atexit(CleanUpRooFitAtExit) ;
   }
 }
 
 
- 
+
