@@ -28,21 +28,25 @@ Object          | function
 ----------------|------------
    RooPlot      | regPlot()
    RooFitResult | regResult()
-   Double_t     | regValue()
+   double       | regValue()
    RooTable     | regTable()
    TH1/2/3      | regTH()
    RooWorkspace | regWS()
 **/
 
-#include "RooFit.h"
 #include "RooUnitTest.h"
-#include "TDirectory.h"
-#include "TClass.h"
+
+#include "RooCurve.h"
 #include "RooHist.h"
 #include "RooMsgService.h"
 #include "RooDouble.h"
 #include "RooTrace.h"
 #include "RooRandom.h"
+
+#include <TClass.h>
+#include <TDirectory.h>
+#include <TFile.h>
+
 #include <cmath>
 
 ClassImp(RooUnitTest);
@@ -54,8 +58,8 @@ TDirectory* RooUnitTest::gMemDir = nullptr;
 
 ////////////////////////////////////////////////////////////////////////////////
 
-RooUnitTest::RooUnitTest(const char* name, TFile* refFile, Bool_t writeRef, Int_t verbose) : TNamed(name,name),
-  			                         _refFile(refFile), _debug(kFALSE), _write(writeRef), _verb(verbose)
+RooUnitTest::RooUnitTest(const char* name, TFile* refFile, bool writeRef, Int_t verbose) : TNamed(name,name),
+                                  _refFile(refFile), _debug(false), _write(writeRef), _verb(verbose)
 {
 }
 
@@ -75,7 +79,13 @@ void RooUnitTest::regPlot(RooPlot* frame, const char* refName)
   if (_refFile) {
     string refNameStr(refName) ;
     frame->SetName(refName) ;
-    _regPlots.push_back(make_pair(frame,refNameStr)) ;
+    // Since ROOT 6.28, the RooPlot doesn't clone the plot variable by default
+    // anymore. This is a problem for registering the RooPlots, because they
+    // need to survive without dangling pointers even after the RooUnitTest is
+    // done. For that reason, we ask the RooPlot to create an internal plot
+    // variable clone for itself.
+    frame->createInternalPlotVarClone();
+    _regPlots.emplace_back(frame,refNameStr);
   } else {
     delete frame ;
   }
@@ -84,20 +94,18 @@ void RooUnitTest::regPlot(RooPlot* frame, const char* refName)
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void RooUnitTest::regResult(RooFitResult* r, const char* refName)
+void RooUnitTest::regResult(std::unique_ptr<RooFitResult> r, const char* refName)
 {
   if (_refFile) {
     string refNameStr(refName) ;
-    _regResults.push_back(make_pair(r,refNameStr)) ;
-  } else {
-    delete r ;
+    _regResults.push_back(make_pair(r.release(),refNameStr)) ;
   }
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void RooUnitTest::regValue(Double_t d, const char* refName)
+void RooUnitTest::regValue(double d, const char* refName)
 {
   if (_refFile) {
     string refNameStr(refName) ;
@@ -151,8 +159,8 @@ RooWorkspace* RooUnitTest::getWS(const char* refName)
 {
   RooWorkspace* ws = dynamic_cast<RooWorkspace*>(_refFile->Get(refName)) ;
   if (!ws) {
-    cout << "RooUnitTest ERROR: cannot retrieve RooWorkspace " << refName
-	 << " from reference file, skipping " << endl ;
+    if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve RooWorkspace " << refName
+                             << " from reference file, skipping " << endl ;
     return 0 ;
   }
 
@@ -162,20 +170,20 @@ RooWorkspace* RooUnitTest::getWS(const char* refName)
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Bool_t RooUnitTest::areTHidentical(TH1* htest, TH1* href)
+bool RooUnitTest::areTHidentical(TH1* htest, TH1* href)
 {
   if (htest->GetDimension() != href->GetDimension()) {
-    return kFALSE ;
+    return false ;
   }
 
   // Use Kolmogorov distance as metric rather than probability
   // because we expect histograms to be identical rather
   // than drawn from the same parent distribution
-  Double_t kmax = htest->KolmogorovTest(href,"M") ;
+  double kmax = htest->KolmogorovTest(href,"M") ;
 
   if (kmax>htol()) {
 
-    cout << "KS distances = " << kmax << endl ;
+    if(_verb >= 0) std::cout << "KS distances = " << kmax << std::endl;
 
     Int_t ntest = htest->GetNbinsX() +2 ;
     Int_t nref  = href->GetNbinsX() +2 ;
@@ -189,28 +197,28 @@ Bool_t RooUnitTest::areTHidentical(TH1* htest, TH1* href)
     }
 
     if (ntest != nref) {
-      return kFALSE ;
+      return false ;
     }
 
     for (Int_t i=0 ; i<ntest ; i++) {
-      if (fabs(htest->GetBinContent(i)-href->GetBinContent(i))>htol()) {
-	cout << "htest[" << i << "] = " << htest->GetBinContent(i) << " href[" << i << "] = " << href->GetBinContent(i) << endl;
+      if (std::abs(htest->GetBinContent(i)-href->GetBinContent(i))>htol()) {
+        if(_verb >= 0) std::cout << "htest[" << i << "] = " << htest->GetBinContent(i) << " href[" << i << "] = " << href->GetBinContent(i) << endl;
       }
     }
 
-    return kFALSE ;
+    return false ;
   }
 
-  return kTRUE ;
+  return true ;
 }
 
 
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Bool_t RooUnitTest::runCompTests()
+bool RooUnitTest::runCompTests()
 {
-  Bool_t ret = kTRUE ;
+  bool ret = true ;
 
   list<pair<RooPlot*, string> >::iterator iter = _regPlots.begin() ;
   while (iter!=_regPlots.end()) {
@@ -222,92 +230,94 @@ Bool_t RooUnitTest::runCompTests()
       // Retrieve benchmark
       RooPlot* bmark = dynamic_cast<RooPlot*>(_refFile->Get(iter->second.c_str())) ;
       if (!bmark) {
-	cout << "RooUnitTest ERROR: cannot retrieve RooPlot " << iter->second << " from reference file, skipping " << endl ;
-	ret = kFALSE ;
-	++iter ;
-	continue ;
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve RooPlot " << iter->second
+                                 << " from reference file, skipping " << std::endl;
+        ret = false ;
+        ++iter ;
+        continue ;
       }
 
-      if (_verb) {
-	cout << "comparing RooPlot " << iter->first << " to benchmark " << iter->second << " = " << bmark << endl ;
-	cout << "reference: " ; iter->first->Print() ;
-	cout << "benchmark: " ; bmark->Print() ;
+      if (_verb > 0) {
+   cout << "comparing RooPlot " << iter->first << " to benchmark " << iter->second << " = " << bmark << endl ;
+   cout << "reference: " ; iter->first->Print() ;
+   cout << "benchmark: " ; bmark->Print() ;
       }
 
       RooPlot* compPlot = _debug ? iter->first->emptyClone(Form("%s_comparison",iter->first->GetName())) : 0 ;
-      Bool_t anyFail=kFALSE ;
+      bool anyFail=false ;
 
       Stat_t nItems = iter->first->numItems() ;
       for (Stat_t i=0 ; i<nItems ; i++) {
-	// coverity[NULL_RETURNS]
-	TObject* obj = iter->first->getObject((Int_t)i) ;
+   // coverity[NULL_RETURNS]
+   TObject* obj = iter->first->getObject((Int_t)i) ;
 
-	// Retrieve corresponding object from reference frame
-	TObject* objRef = bmark->findObject(obj->GetName()) ;
+   // Retrieve corresponding object from reference frame
+   TObject* objRef = bmark->findObject(obj->GetName()) ;
 
-	if (!objRef) {
-	  cout << "RooUnitTest ERROR: cannot retrieve object " << obj->GetName() << " from reference  RooPlot " << iter->second << ", skipping" << endl ;
-	  ret = kFALSE ;
-	  break ;
-	}
+   if (!objRef) {
+     if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve reference object " << obj->GetName()
+                               << " from benchmark RooPlot " << iter->second << ", skipping" << std::endl;
+     ret = false ;
+     break ;
+   }
 
-	// Histogram comparisons
-	if (obj->IsA()==RooHist::Class()) {
-	  RooHist* testHist = static_cast<RooHist*>(obj) ;
-	  RooHist* refHist = static_cast<RooHist*>(objRef) ;
-	  if (!testHist->isIdentical(*refHist,htol())) {
-	    cout << "RooUnitTest ERROR: comparison of object " << obj->IsA()->GetName() << "::" << obj->GetName()
-		 <<   " fails comparison with counterpart in reference RooPlot " << bmark->GetName() << endl ;
+   // Histogram comparisons
+   if (obj->IsA()==RooHist::Class()) {
+     RooHist* testHist = static_cast<RooHist*>(obj) ;
+     RooHist* refHist = static_cast<RooHist*>(objRef) ;
+     if (!testHist->isIdentical(*refHist,htol(),_verb >= 0)) {
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: comparison of object " << obj->ClassName() << "::" << obj->GetName()
+                                 <<   " fails comparison with counterpart in reference RooPlot " << bmark->GetName() << std::endl;
 
-	    if (compPlot) {
-	      compPlot->addPlotable((RooHist*)testHist->Clone(),"P") ;
-	      compPlot->getAttLine()->SetLineColor(kRed) ;
-	      compPlot->getAttMarker()->SetMarkerColor(kRed) ;
-	      compPlot->getAttLine()->SetLineWidth(1) ;
+       if (compPlot) {
+         compPlot->addPlotable((RooHist*)testHist->Clone(),"P") ;
+         compPlot->getAttLine()->SetLineColor(kRed) ;
+         compPlot->getAttMarker()->SetMarkerColor(kRed) ;
+         compPlot->getAttLine()->SetLineWidth(1) ;
 
-	      compPlot->addPlotable((RooHist*)refHist->Clone(),"P") ;
-	      compPlot->getAttLine()->SetLineColor(kBlue) ;
-	      compPlot->getAttMarker()->SetMarkerColor(kBlue) ;
-	      compPlot->getAttLine()->SetLineWidth(1) ;
-	    }
+         compPlot->addPlotable((RooHist*)refHist->Clone(),"P") ;
+         compPlot->getAttLine()->SetLineColor(kBlue) ;
+         compPlot->getAttMarker()->SetMarkerColor(kBlue) ;
+         compPlot->getAttLine()->SetLineWidth(1) ;
+       }
 
-	    anyFail=kTRUE ;
-	    ret = kFALSE ;
-	  }
-	} else if (obj->IsA()==RooCurve::Class()) {
-	  RooCurve* testCurve = static_cast<RooCurve*>(obj) ;
-	  RooCurve* refCurve = static_cast<RooCurve*>(objRef) ;
-	  if (!testCurve->isIdentical(*refCurve,ctol())) {
-	    cout << "RooUnitTest ERROR: comparison of object " << obj->IsA()->GetName() << "::" << obj->GetName()
-		 <<   " fails comparison with counterpart in reference RooPlot " << bmark->GetName() << endl ;
+       anyFail=true ;
+       ret = false ;
+     }
+   } else if (obj->IsA()==RooCurve::Class()) {
+     RooCurve* testCurve = static_cast<RooCurve*>(obj) ;
+     RooCurve* refCurve = static_cast<RooCurve*>(objRef) ;
+     if (!testCurve->isIdentical(*refCurve,ctol(),_verb >= 0)) {
+       if(_verb >= 0) std::cout << "RooUnitTest ERROR: comparison of object " << obj->ClassName() << "::" << obj->GetName()
+                                 <<   " fails comparison with counterpart in reference RooPlot " << bmark->GetName() << std::endl;
 
-	    if (compPlot) {
-	      compPlot->addPlotable((RooCurve*)testCurve->Clone()) ;
-	      compPlot->getAttLine()->SetLineColor(kRed) ;
-	      compPlot->getAttLine()->SetLineWidth(1) ;
-	      compPlot->getAttLine()->SetLineStyle(kSolid) ;
+       if (compPlot) {
+         compPlot->addPlotable((RooCurve*)testCurve->Clone()) ;
+         compPlot->getAttLine()->SetLineColor(kRed) ;
+         compPlot->getAttLine()->SetLineWidth(1) ;
+         compPlot->getAttLine()->SetLineStyle(kSolid) ;
 
-	      compPlot->addPlotable((RooCurve*)refCurve->Clone()) ;
-	      compPlot->getAttLine()->SetLineColor(kBlue) ;
-	      compPlot->getAttLine()->SetLineWidth(1) ;
-	      compPlot->getAttLine()->SetLineStyle(kDashed) ;
-	    }
+         compPlot->addPlotable((RooCurve*)refCurve->Clone()) ;
+         compPlot->getAttLine()->SetLineColor(kBlue) ;
+         compPlot->getAttLine()->SetLineWidth(1) ;
+         compPlot->getAttLine()->SetLineStyle(kDashed) ;
+       }
 
-	    anyFail=kTRUE ;
-	    ret = kFALSE ;
-	  }
+       anyFail=true ;
+       ret = false ;
+     }
 
-	}
+   }
 
       }
 
       if (anyFail && compPlot) {
-	cout << "RooUnitTest INFO: writing comparison plot " << compPlot->GetName() << " of failed test to RooUnitTest_DEBUG.root" << endl ;
-	TFile fdbg("RooUnitTest_DEBUG.root","UPDATE") ;
-	compPlot->Write() ;
-	fdbg.Close() ;
+   cout << "RooUnitTest INFO: writing comparison plot " << compPlot->GetName() << " of failed test to RooUnitTest_DEBUG.root" << endl ;
+   TFile fdbg("RooUnitTest_DEBUG.root","UPDATE") ;
+   compPlot->Write() ;
+   fdbg.Close() ;
       } else {
-	delete compPlot ;
+   delete compPlot ;
       }
 
       // Delete RooPlot when comparison is finished to avoid noise in leak checking
@@ -317,7 +327,7 @@ Bool_t RooUnitTest::runCompTests()
 
       // Writing mode
 
-      cout <<"RooUnitTest: Writing reference RooPlot " << iter->first << " as benchmark " << iter->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooPlot " << iter->first << " as benchmark " << iter->second << endl ;
       _refFile->cd() ;
       iter->first->Write(iter->second.c_str()) ;
       gMemDir->cd() ;
@@ -337,21 +347,22 @@ Bool_t RooUnitTest::runCompTests()
      // Retrieve benchmark
       RooFitResult* bmark = dynamic_cast<RooFitResult*>(_refFile->Get(iter2->second.c_str())) ;
       if (!bmark) {
-	cout << "RooUnitTest ERROR: cannot retrieve RooFitResult " << iter2->second << " from reference file, skipping " << endl ;
-	++iter2 ;
-	ret = kFALSE ;
-	continue ;
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve RooFitResult "
+                                 << iter2->second << " from reference file, skipping " << std::endl ;
+        ++iter2 ;
+        ret = false ;
+        continue ;
       }
 
-      if (_verb) {
-	cout << "comparing RooFitResult " << iter2->first << " to benchmark " << iter2->second << " = " << bmark << endl ;
+      if (_verb > 0) {
+   cout << "comparing RooFitResult " << iter2->first << " to benchmark " << iter2->second << " = " << bmark << endl ;
       }
 
-      if (!iter2->first->isIdentical(*bmark,fptol(),fctol())) {
-	cout << "RooUnitTest ERROR: comparison of object " << iter2->first->IsA()->GetName() << "::" << iter2->first->GetName()
-	     << " from result " << iter2->second
-	     <<   " fails comparison with counterpart in reference RooFitResult " << bmark->GetName() << endl ;
-	ret = kFALSE ;
+      if (!iter2->first->isIdentical(*bmark,fptol(),fctol(),_verb >= 0)) {
+        if(_verb >= 0) cout << "RooUnitTest ERROR: comparison of object " << iter2->first->ClassName() << "::" << iter2->first->GetName()
+             << " from result " << iter2->second
+             <<   " fails comparison with counterpart in reference RooFitResult " << bmark->GetName() << std::endl;
+        ret = false ;
       }
 
       // Delete RooFitResult when comparison is finished to avoid noise in leak checking
@@ -362,7 +373,7 @@ Bool_t RooUnitTest::runCompTests()
 
       // Writing mode
 
-      cout <<"RooUnitTest: Writing reference RooFitResult " << iter2->first << " as benchmark " << iter2->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooFitResult " << iter2->first << " as benchmark " << iter2->second << endl ;
       _refFile->cd() ;
       iter2->first->Write(iter2->second.c_str()) ;
       gMemDir->cd() ;
@@ -371,7 +382,7 @@ Bool_t RooUnitTest::runCompTests()
     ++iter2 ;
   }
 
-  list<pair<Double_t, string> >::iterator iter3 = _regValues.begin() ;
+  list<pair<double, string> >::iterator iter3 = _regValues.begin() ;
   while (iter3!=_regValues.end()) {
 
     if (!_write) {
@@ -381,19 +392,19 @@ Bool_t RooUnitTest::runCompTests()
      // Retrieve benchmark
       RooDouble* ref = dynamic_cast<RooDouble*>(_refFile->Get(iter3->second.c_str())) ;
       if (!ref) {
-	cout << "RooUnitTest ERROR: cannot retrieve RooDouble " << iter3->second << " from reference file, skipping " << endl ;
-	++iter3 ;
-	ret = kFALSE ;
-	continue ;
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: cannot retrieve RooDouble " << iter3->second << " from reference file, skipping " << std::endl;
+        ++iter3 ;
+        ret = false ;
+        continue ;
       }
 
-      if (_verb) {
-	cout << "comparing value " << iter3->first << " to benchmark " << iter3->second << " = " << (Double_t)(*ref) << endl ;
+      if (_verb > 0) {
+   cout << "comparing value " << iter3->first << " to benchmark " << iter3->second << " = " << (double)(*ref) << endl ;
       }
 
-      if (fabs(iter3->first - (Double_t)(*ref))>vtol() ) {
-	cout << "RooUnitTest ERROR: comparison of value " << iter3->first <<   " fails comparison with reference " << ref->GetName() << endl ;
-	ret = kFALSE ;
+      if (std::abs(iter3->first - (double)(*ref))>vtol() ) {
+        if(_verb >= 0) cout << "RooUnitTest ERROR: comparison of value " << iter3->first <<   " fails comparison with reference " << ref->GetName() << endl ;
+        ret = false ;
       }
 
 
@@ -401,7 +412,7 @@ Bool_t RooUnitTest::runCompTests()
 
       // Writing mode
 
-      cout <<"RooUnitTest: Writing reference Double_t " << iter3->first << " as benchmark " << iter3->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference double " << iter3->first << " as benchmark " << iter3->second << endl ;
       _refFile->cd() ;
       RooDouble* rd = new RooDouble(iter3->first) ;
       rd->Write(iter3->second.c_str()) ;
@@ -422,20 +433,20 @@ Bool_t RooUnitTest::runCompTests()
      // Retrieve benchmark
       RooTable* bmark = dynamic_cast<RooTable*>(_refFile->Get(iter4->second.c_str())) ;
       if (!bmark) {
-	cout << "RooUnitTest ERROR: cannot retrieve RooTable " << iter4->second << " from reference file, skipping " << endl ;
-	++iter4 ;
-	ret = kFALSE ;
-	continue ;
+        if(_verb >= 0) cout << "RooUnitTest ERROR: cannot retrieve RooTable " << iter4->second << " from reference file, skipping " << endl ;
+        ++iter4 ;
+        ret = false ;
+        continue ;
       }
 
-      if (_verb) {
-	cout << "comparing RooTable " << iter4->first << " to benchmark " << iter4->second << " = " << bmark << endl ;
+      if (_verb > 0) {
+   cout << "comparing RooTable " << iter4->first << " to benchmark " << iter4->second << " = " << bmark << endl ;
       }
 
-      if (!iter4->first->isIdentical(*bmark)) {
-        cout << "RooUnitTest ERROR: comparison of object " << iter4->first->IsA()->GetName() << "::" << iter4->first->GetName()
-	         <<   " fails comparison with counterpart in reference RooTable " << bmark->GetName() << endl ;
-        if (_verb) {
+      if (!iter4->first->isIdentical(*bmark, _verb >= 0)) {
+        if(_verb >= 0) std::cout << "RooUnitTest ERROR: comparison of object " << iter4->first->ClassName() << "::" << iter4->first->GetName()
+            <<   " fails comparison with counterpart in reference RooTable " << bmark->GetName() << endl ;
+        if (_verb > 0) {
           iter4->first->Print("V");
           bmark->Print("V");
         }
@@ -450,7 +461,7 @@ Bool_t RooUnitTest::runCompTests()
 
       // Writing mode
 
-      cout <<"RooUnitTest: Writing reference RooTable " << iter4->first << " as benchmark " << iter4->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooTable " << iter4->first << " as benchmark " << iter4->second << endl ;
       _refFile->cd() ;
       iter4->first->Write(iter4->second.c_str()) ;
       gMemDir->cd() ;
@@ -467,7 +478,7 @@ Bool_t RooUnitTest::runCompTests()
 
       // Writing mode
 
-      cout <<"RooUnitTest: Writing reference RooWorkspace " << iter5->first << " as benchmark " << iter5->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference RooWorkspace " << iter5->first << " as benchmark " << iter5->second << endl ;
       _refFile->cd() ;
       iter5->first->Write(iter5->second.c_str()) ;
       gMemDir->cd() ;
@@ -487,34 +498,34 @@ Bool_t RooUnitTest::runCompTests()
      // Retrieve benchmark
       TH1* bmark = dynamic_cast<TH1*>(_refFile->Get(iter6->second.c_str())) ;
       if (!bmark) {
-	cout << "RooUnitTest ERROR: cannot retrieve TH1 " << iter6->second << " from reference file, skipping " << endl ;
-	++iter6 ;
-	ret = kFALSE ;
-	continue ;
+        if(_verb >= 0) cout << "RooUnitTest ERROR: cannot retrieve TH1 " << iter6->second << " from reference file, skipping " << endl ;
+        ++iter6 ;
+        ret = false ;
+        continue ;
       }
 
-      if (_verb) {
-	cout << "comparing TH1 " << iter6->first << " to benchmark " << iter6->second << " = " << bmark << endl ;
+      if (_verb > 0) {
+   cout << "comparing TH1 " << iter6->first << " to benchmark " << iter6->second << " = " << bmark << endl ;
       }
 
       if (!areTHidentical(iter6->first,bmark)) {
-	// coverity[NULL_RETURNS]
-	cout << "RooUnitTest ERROR: comparison of object " << iter6->first->IsA()->GetName() << "::" << iter6->first->GetName()
-	     <<   " fails comparison with counterpart in reference TH1 " << bmark->GetName() << endl ;
+   // coverity[NULL_RETURNS]
+   if(_verb >= 0) cout << "RooUnitTest ERROR: comparison of object " << iter6->first->ClassName() << "::" << iter6->first->GetName()
+        <<   " fails comparison with counterpart in reference TH1 " << bmark->GetName() << endl ;
 
 
       if (_debug) {
-	cout << "RooUnitTest INFO: writing THx " << iter6->first->GetName() << " and " << bmark->GetName()
-	     << " of failed test to RooUnitTest_DEBUG.root" << endl ;
-	TFile fdbg("RooUnitTest_DEBUG.root","UPDATE") ;
-	iter6->first->SetName(Form("%s_test",iter6->first->GetName())) ;
-	iter6->first->Write() ;
-	bmark->SetName(Form("%s_ref",bmark->GetName())) ;
-	bmark->Write() ;
-	fdbg.Close() ;
+   cout << "RooUnitTest INFO: writing THx " << iter6->first->GetName() << " and " << bmark->GetName()
+        << " of failed test to RooUnitTest_DEBUG.root" << endl ;
+   TFile fdbg("RooUnitTest_DEBUG.root","UPDATE") ;
+   iter6->first->SetName(Form("%s_test",iter6->first->GetName())) ;
+   iter6->first->Write() ;
+   bmark->SetName(Form("%s_ref",bmark->GetName())) ;
+   bmark->Write() ;
+   fdbg.Close() ;
       }
 
-	ret = kFALSE ;
+   ret = false ;
       }
 
       // Delete TH1 when comparison is finished to avoid noise in leak checking
@@ -525,7 +536,7 @@ Bool_t RooUnitTest::runCompTests()
 
       // Writing mode
 
-      cout <<"RooUnitTest: Writing reference TH1 " << iter6->first << " as benchmark " << iter6->second << endl ;
+      std::cout <<"RooUnitTest: Writing reference TH1 " << iter6->first << " as benchmark " << iter6->second << endl ;
       _refFile->cd() ;
       iter6->first->Write(iter6->second.c_str()) ;
       gMemDir->cd() ;
@@ -545,10 +556,10 @@ Bool_t RooUnitTest::runCompTests()
 
 void RooUnitTest::setSilentMode()
 {
-  RooMsgService::instance().setSilentMode(kTRUE) ;
+  RooMsgService::instance().setSilentMode(true) ;
   for (Int_t i=0 ; i<RooMsgService::instance().numStreams() ; i++) {
     if (RooMsgService::instance().getStream(i).minLevel<RooFit::ERROR) {
-      RooMsgService::instance().setStreamStatus(i,kFALSE) ;
+      RooMsgService::instance().setStreamStatus(i,false) ;
     }
   }
 }
@@ -558,9 +569,9 @@ void RooUnitTest::setSilentMode()
 
 void RooUnitTest::clearSilentMode()
 {
-  RooMsgService::instance().setSilentMode(kFALSE) ;
+  RooMsgService::instance().setSilentMode(false) ;
   for (Int_t i=0 ; i<RooMsgService::instance().numStreams() ; i++) {
-    RooMsgService::instance().setStreamStatus(i,kTRUE) ;
+    RooMsgService::instance().setStreamStatus(i,true) ;
   }
 }
 
@@ -568,14 +579,14 @@ void RooUnitTest::clearSilentMode()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Bool_t RooUnitTest::runTest()
+bool RooUnitTest::runTest()
 {
   gMemDir->cd() ;
 
   if (_verb<2) {
     setSilentMode() ;
   } else {
-    cout << "*** Begin of output of Unit Test at normal verbosity *************" << endl ;
+    std::cout << "*** Begin of output of Unit Test at normal verbosity *************" << endl ;
   }
 
   RooMsgService::instance().clearErrorCount() ;
@@ -585,18 +596,18 @@ Bool_t RooUnitTest::runTest()
   RooRandom::randomGenerator()->SetSeed(12345) ;
 
   RooTrace::callgrind_zero() ;
-  if (!testCode()) return kFALSE ;
+  if (!testCode()) return false ;
   RooTrace::callgrind_dump() ;
 
   if (_verb<2) {
     clearSilentMode() ;
   } else {
-    cout << "*** End of output of Unit Test at normal verbosity ***************" << endl ;
+    std::cout << "*** End of output of Unit Test at normal verbosity ***************" << endl ;
   }
 
   if (RooMsgService::instance().errorCount()>0) {
-    cout << "RooUnitTest: ERROR messages were logged, failing test" << endl ;
-    return kFALSE ;
+    if(_verb >= 0) std::cout << "RooUnitTest: ERROR messages were logged, failing test" << endl ;
+    return false ;
   }
 
   return runCompTests() ;
