@@ -1,5 +1,54 @@
 #include "ntuple_test.hxx"
 
+namespace {
+/// An RPageSink that keeps counters of (vector) commit of (sealed) pages; used to test RPageSinkBuf
+class RPageSinkMock : public RPageSink {
+public:
+   struct {
+      size_t fNCommitPage = 0;
+      size_t fNCommitSealedPage = 0;
+      size_t fNCommitSealedPageV = 0;
+   } fCounters{};
+
+protected:
+   RPageAllocatorHeap fPageAllocator{};
+
+   void CreateImpl(const RNTupleModel &, unsigned char *, std::uint32_t) final {}
+   RNTupleLocator CommitPageImpl(ColumnHandle_t /*columnHandle*/, const RPage & /*page*/) final
+   {
+      fCounters.fNCommitPage++;
+      return {};
+   }
+   RNTupleLocator CommitSealedPageImpl(ROOT::Experimental::DescriptorId_t, const RPageStorage::RSealedPage &) final
+   {
+      fCounters.fNCommitSealedPage++;
+      return {};
+   }
+   std::vector<RNTupleLocator> CommitSealedPageVImpl(std::span<RPageStorage::RSealedPageGroup> ranges) override
+   {
+      fCounters.fNCommitSealedPageV++;
+      auto nLocators =
+         std::accumulate(ranges.begin(), ranges.end(), 0, [](size_t c, const RPageStorage::RSealedPageGroup &r) {
+            return c + std::distance(r.fFirst, r.fLast);
+         });
+      return std::vector<RNTupleLocator>(nLocators);
+   }
+   std::uint64_t CommitClusterImpl(NTupleSize_t) final { return 0; }
+   RNTupleLocator CommitClusterGroupImpl(unsigned char *, std::uint32_t) final { return {}; }
+   void CommitDatasetImpl(unsigned char *, std::uint32_t) final {}
+
+   RPage ReservePage(ColumnHandle_t columnHandle, std::size_t nElements) final
+   {
+      auto elementSize = columnHandle.fColumn->GetElement()->GetSize();
+      return fPageAllocator.NewPage(columnHandle.fId, elementSize, nElements);
+   }
+   void ReleasePage(RPage &page) final { fPageAllocator.DeletePage(page); }
+
+public:
+   RPageSinkMock(const ROOT::Experimental::RNTupleWriteOptions &options) : RPageSink("test", options) {}
+};
+} // namespace
+
 TEST(RNTuple, Basics)
 {
    FileRaii fileGuard("test_ntuple_barefile.ntuple");
@@ -144,18 +193,18 @@ TEST(RNTuple, PageFilling) {
    for (std::int16_t i = 0; i < 8; ++i)
       EXPECT_EQ(i, viewX(i));
 
-   const auto &desc = ntuple->GetDescriptor();
-   EXPECT_EQ(3u, desc.GetNClusters());
-   const auto &cd1 = desc.GetClusterDescriptor(desc.FindClusterId(0, 0));
+   const auto desc = ntuple->GetDescriptor();
+   EXPECT_EQ(3u, desc->GetNClusters());
+   const auto &cd1 = desc->GetClusterDescriptor(desc->FindClusterId(0, 0));
    const auto &pr1 = cd1.GetPageRange(0);
    ASSERT_EQ(2u, pr1.fPageInfos.size());
    EXPECT_EQ(2u, pr1.fPageInfos[0].fNElements);
    EXPECT_EQ(1u, pr1.fPageInfos[1].fNElements);
-   const auto &cd2 = desc.GetClusterDescriptor(desc.FindNextClusterId(cd1.GetId()));
+   const auto &cd2 = desc->GetClusterDescriptor(desc->FindNextClusterId(cd1.GetId()));
    const auto &pr2 = cd2.GetPageRange(0);
    ASSERT_EQ(1u, pr2.fPageInfos.size());
    EXPECT_EQ(2u, pr2.fPageInfos[0].fNElements);
-   const auto &cd3 = desc.GetClusterDescriptor(desc.FindNextClusterId(cd2.GetId()));
+   const auto &cd3 = desc->GetClusterDescriptor(desc->FindNextClusterId(cd2.GetId()));
    const auto &pr3 = cd3.GetPageRange(0);
    ASSERT_EQ(2u, pr3.fPageInfos.size());
    EXPECT_EQ(2u, pr3.fPageInfos[0].fNElements);
@@ -199,20 +248,20 @@ TEST(RNTuple, PageFillingString) {
    EXPECT_EQ("",                         viewX(2));
    EXPECT_EQ("012345678901234567890123", viewX(3));
 
-   const auto &desc = ntuple->GetDescriptor();
-   EXPECT_EQ(4u, desc.GetNClusters());
-   const auto &cd1 = desc.GetClusterDescriptor(desc.FindClusterId(1, 0));
+   const auto desc = ntuple->GetDescriptor();
+   EXPECT_EQ(4u, desc->GetNClusters());
+   const auto &cd1 = desc->GetClusterDescriptor(desc->FindClusterId(1, 0));
    const auto &pr1 = cd1.GetPageRange(1);
    ASSERT_EQ(1u, pr1.fPageInfos.size());
    EXPECT_EQ(17u, pr1.fPageInfos[0].fNElements);
-   const auto &cd2 = desc.GetClusterDescriptor(desc.FindNextClusterId(cd1.GetId()));
+   const auto &cd2 = desc->GetClusterDescriptor(desc->FindNextClusterId(cd1.GetId()));
    const auto &pr2 = cd2.GetPageRange(1);
    ASSERT_EQ(1u, pr2.fPageInfos.size());
    EXPECT_EQ(16u, pr2.fPageInfos[0].fNElements);
-   const auto &cd3 = desc.GetClusterDescriptor(desc.FindNextClusterId(cd2.GetId()));
+   const auto &cd3 = desc->GetClusterDescriptor(desc->FindNextClusterId(cd2.GetId()));
    const auto &pr3 = cd3.GetPageRange(1);
    ASSERT_EQ(0u, pr3.fPageInfos.size());
-   const auto &cd4 = desc.GetClusterDescriptor(desc.FindNextClusterId(cd3.GetId()));
+   const auto &cd4 = desc->GetClusterDescriptor(desc->FindNextClusterId(cd3.GetId()));
    const auto &pr4 = cd4.GetPageRange(1);
    ASSERT_EQ(2u, pr4.fPageInfos.size());
    EXPECT_EQ(16u, pr4.fPageInfos[0].fNElements);
@@ -291,11 +340,11 @@ TEST(RPageSinkBuf, Basics)
 
    std::vector<std::pair<DescriptorId_t, std::int64_t>> pagePositions;
    std::size_t num_columns = 10;
-   const auto &cluster0 = ntupleBuf->GetDescriptor().GetClusterDescriptor(0);
+   const auto &cluster0 = ntupleBuf->GetDescriptor()->GetClusterDescriptor(0);
    for (std::size_t i = 0; i < num_columns; i++) {
       const auto &columnPages = cluster0.GetPageRange(i);
       for (const auto &page: columnPages.fPageInfos) {
-         pagePositions.push_back(std::make_pair(i, page.fLocator.fPosition));
+         pagePositions.push_back(std::make_pair(i, page.fLocator.GetPosition<std::uint64_t>()));
       }
    }
 
@@ -320,7 +369,9 @@ TEST(RPageSinkBuf, Basics)
 }
 
 TEST(RPageSinkBuf, ParallelZip) {
+#ifdef R__USE_IMT
    ROOT::EnableImplicitMT();
+#endif
 
    FileRaii fileGuard("test_ntuple_sinkbuf_pzip.root");
    {
@@ -346,7 +397,11 @@ TEST(RPageSinkBuf, ParallelZip) {
             auto *parallel_zip = ntuple->GetMetrics().GetCounter(
                "RNTupleWriter.RPageSinkBuf.ParallelZip");
             ASSERT_FALSE(parallel_zip == nullptr);
+#ifdef R__USE_IMT
             EXPECT_EQ(1, parallel_zip->GetValueAsInt());
+#else
+            EXPECT_EQ(0, parallel_zip->GetValueAsInt());
+#endif
          }
       }
    }
@@ -364,3 +419,106 @@ TEST(RPageSinkBuf, ParallelZip) {
       EXPECT_EQ("hi" + std::to_string(i), viewKlassVec(i).at(0).s);
    }
 }
+
+TEST(RPageSinkBuf, CommitSealedPageV)
+{
+   RNTupleWriteOptions options;
+   options.SetApproxUnzippedPageSize(8);
+
+#ifdef R__USE_IMT
+   ROOT::DisableImplicitMT();
+#endif
+   {
+      std::unique_ptr<RPageSink> sink(new RPageSinkMock(options));
+      auto &counters = static_cast<RPageSinkMock *>(sink.get())->fCounters;
+
+      auto model = RNTupleModel::Create();
+      auto u32Field = model->MakeField<std::uint32_t>("u32");
+      auto u16Field = model->MakeField<std::uint16_t>("u16");
+      auto ntuple = std::make_unique<RNTupleWriter>(std::move(model), std::make_unique<RPageSinkBuf>(std::move(sink)));
+      ntuple->Fill();
+      ntuple->Fill();
+      ntuple->Fill();
+      ntuple->CommitCluster();
+      // Parallel zip not available; all pages committed separately
+      EXPECT_EQ(3, counters.fNCommitPage);
+      EXPECT_EQ(0, counters.fNCommitSealedPage);
+      EXPECT_EQ(0, counters.fNCommitSealedPageV);
+   }
+#ifdef R__USE_IMT
+   ROOT::EnableImplicitMT();
+#endif
+   {
+      std::unique_ptr<RPageSink> sink(new RPageSinkMock(options));
+      auto &counters = static_cast<RPageSinkMock *>(sink.get())->fCounters;
+
+      auto model = RNTupleModel::Create();
+      auto u32Field = model->MakeField<std::uint32_t>("u32");
+      auto u16Field = model->MakeField<std::uint16_t>("u16");
+      auto ntuple = std::make_unique<RNTupleWriter>(std::move(model), std::make_unique<RPageSinkBuf>(std::move(sink)));
+      ntuple->Fill();
+      ntuple->Fill();
+      ntuple->CommitCluster();
+#ifdef R__USE_IMT
+      // All pages in all columns committed via a single call to `CommitSealedPageV()`
+      EXPECT_EQ(0, counters.fNCommitPage);
+      EXPECT_EQ(1, counters.fNCommitSealedPageV);
+#else
+      EXPECT_EQ(2, counters.fNCommitPage);
+      EXPECT_EQ(0, counters.fNCommitSealedPageV);
+#endif
+      EXPECT_EQ(0, counters.fNCommitSealedPage);
+   }
+}
+
+TEST(RPageSink, Empty)
+{
+   FileRaii fileGuard("test_ntuple_empty.ntuple");
+
+   auto model = RNTupleModel::Create();
+   auto wrPt = model->MakeField<float>("pt", 42.0);
+
+   {
+      auto ntuple = RNTupleWriter::Recreate(std::move(model), "f", fileGuard.GetPath());
+   }
+
+   auto ntuple = RNTupleReader::Open("f", fileGuard.GetPath());
+   EXPECT_EQ(0U, ntuple->GetNEntries());
+   EXPECT_EQ(0U, ntuple->GetDescriptor()->GetNClusterGroups());
+   EXPECT_EQ(0U, ntuple->GetDescriptor()->GetNClusters());
+}
+
+TEST(RPageSink, MultipleClusterGroups)
+{
+   FileRaii fileGuard("test_ntuple_multi_cluster_groups.ntuple");
+
+   auto model = RNTupleModel::Create();
+   auto wrPt = model->MakeField<float>("pt", 42.0);
+
+   {
+      auto ntuple = RNTupleWriter::Recreate(std::move(model), "f", fileGuard.GetPath());
+      ntuple->Fill();
+      ntuple->CommitCluster();
+      // This pattern should work: CommitCluster(false) followed by CommitCluster(true) and
+      // be identical to a single call to CommitCluster(true)
+      ntuple->CommitCluster(true);
+      *wrPt = 24.0;
+      ntuple->Fill();
+      ntuple->CommitCluster();
+      *wrPt = 12.0;
+      ntuple->Fill();
+   }
+
+   auto ntuple = RNTupleReader::Open("f", fileGuard.GetPath());
+   EXPECT_EQ(2U, ntuple->GetDescriptor()->GetNClusterGroups());
+   EXPECT_EQ(3U, ntuple->GetDescriptor()->GetNClusters());
+   EXPECT_EQ(3U, ntuple->GetNEntries());
+   auto rdPt = ntuple->GetModel()->GetDefaultEntry()->Get<float>("pt");
+
+   ntuple->LoadEntry(0);
+   EXPECT_EQ(42.0, *rdPt);
+   ntuple->LoadEntry(1);
+   EXPECT_EQ(24.0, *rdPt);
+   ntuple->LoadEntry(2);
+   EXPECT_EQ(12.0, *rdPt);
+}
